@@ -4,10 +4,11 @@ import wx
 from wx import aui
 
 from meerk40t.gui.icons import (
-    icons8_camera_50,
-    icons8_connected_50,
-    icons8_detective_50,
-    icons8_picture_in_picture_alternative_50,
+    get_default_icon_size,
+    icons8_camera,
+    icons8_connected,
+    icons8_detective,
+    icons8_image_in_frame,
 )
 from meerk40t.gui.mwindow import MWindow
 from meerk40t.gui.scene.sceneconst import (
@@ -18,6 +19,13 @@ from meerk40t.gui.scene.sceneconst import (
 )
 from meerk40t.gui.scene.scenepanel import ScenePanel
 from meerk40t.gui.scene.widget import Widget
+from meerk40t.gui.wxutils import (
+    TextCtrl,
+    wxBitmapButton,
+    wxButton,
+    wxCheckBox,
+    wxListCtrl,
+)
 from meerk40t.kernel import Job, signal_listener
 from meerk40t.svgelements import Color
 
@@ -26,24 +34,34 @@ _ = wx.GetTranslation
 CORNER_SIZE = 25
 
 
+def _get_camera_attribute(kernel, camera, attribute, default=None):
+    if isinstance(camera, int):
+        camera = f"camera/{camera}"
+    label = kernel.read_persistent(str, camera, attribute, default)
+    return label or default
+
+
 def register_panel_camera(window, context):
     for index in range(5):
         panel = CameraPanel(
             window, wx.ID_ANY, context=context, gui=window, index=index, pane=True
         )
+        label = _("Camera {index}").format(index=index)
         pane = (
             aui.AuiPaneInfo()
             .Left()
             .MinSize(200, 150)
             .FloatingSize(640, 480)
-            .Caption(_("Camera {index}").format(index=index))
+            .Caption(label)
             .Name(f"camera{index}")
             .CaptionVisible(not context.pane_lock)
             .Hide()
         )
         pane.dock_proportion = 200
         pane.control = panel
+        panel.pane_aui = pane
         pane.submenu = "_60_" + _("Camera")
+        pane.helptext = _("Show camera capture panel")
         window.on_pane_create(pane)
         context.register(f"pane/camera{index}", pane)
 
@@ -56,8 +74,12 @@ class CameraPanel(wx.Panel, Job):
         wx.Panel.__init__(self, *args, **kwds)
         self.gui = gui
         self.context = context
+        self.context.themes.set_window_colors(self)
+        self.SetHelpText("camera")
         self.index = index
+        self.cam_device_link = {}
         self.pane = pane
+        self.pane_aui = None
 
         if pane:
             job_name = f"CamPane{self.index}"
@@ -70,21 +92,31 @@ class CameraPanel(wx.Panel, Job):
         self.context(f"camera{self.index}\n")  # command activates Camera service
         self.camera = self.context.get_context(f"camera/{self.index}")
         self.camera.setting(int, "frames_per_second", 30)
+        self._remembered = -1
+        self.available_resolutions = []
         # camera service location.
         self.last_frame_index = -1
 
         if not pane:
-            self.button_update = wx.BitmapButton(
-                self, wx.ID_ANY, icons8_camera_50.GetBitmap()
+            self.button_update = wxBitmapButton(
+                self,
+                wx.ID_ANY,
+                icons8_camera.GetBitmap(resize=get_default_icon_size(self.context)),
             )
-            self.button_export = wx.BitmapButton(
-                self, wx.ID_ANY, icons8_picture_in_picture_alternative_50.GetBitmap()
+            self.button_export = wxBitmapButton(
+                self,
+                wx.ID_ANY,
+                icons8_image_in_frame.GetBitmap(
+                    resize=get_default_icon_size(self.context)
+                ),
             )
-            self.button_reconnect = wx.BitmapButton(
-                self, wx.ID_ANY, icons8_connected_50.GetBitmap()
+            self.button_reconnect = wxBitmapButton(
+                self,
+                wx.ID_ANY,
+                icons8_connected.GetBitmap(resize=get_default_icon_size(self.context)),
             )
-            self.check_fisheye = wx.CheckBox(self, wx.ID_ANY, _("Correct Fisheye"))
-            self.check_perspective = wx.CheckBox(
+            self.check_fisheye = wxCheckBox(self, wx.ID_ANY, _("Correct Fisheye"))
+            self.check_perspective = wxCheckBox(
                 self, wx.ID_ANY, _("Correct Perspective")
             )
             self.slider_fps = wx.Slider(
@@ -95,8 +127,10 @@ class CameraPanel(wx.Panel, Job):
                 120,
                 style=wx.SL_AUTOTICKS | wx.SL_HORIZONTAL | wx.SL_LABELS,
             )
-            self.button_detect = wx.BitmapButton(
-                self, wx.ID_ANY, icons8_detective_50.GetBitmap()
+            self.button_detect = wxBitmapButton(
+                self,
+                wx.ID_ANY,
+                icons8_detective.GetBitmap(resize=get_default_icon_size(self.context)),
             )
             scene_name = f"Camera{self.index}"
         else:
@@ -109,6 +143,8 @@ class CameraPanel(wx.Panel, Job):
             style=wx.EXPAND | wx.WANTS_CHARS,
         )
         self.widget_scene = self.display_camera.scene
+        # Allow Scene update from now on (are suppressed by default during startup phase)
+        self.widget_scene.suppress_changes = False
 
         # end wxGlade
         sizer_main = wx.BoxSizer(wx.VERTICAL)
@@ -187,23 +223,55 @@ class CameraPanel(wx.Panel, Job):
         self.widget_scene.add_interfacewidget(
             CamInterfaceWidget(self.widget_scene, self)
         )
+        # Allow Scene update from now on (are suppressed by default during startup phase)
+        self.widget_scene.suppress_changes = False
 
     def pane_show(self, *args):
+        if self.index != self._remembered:
+            self._remembered = self.index
+            self.available_resolutions = []
+            if self.camera.is_physical:
+                self.available_resolutions = self.camera.guess_supported_resolutions()
         if platform.system() == "Darwin" and not hasattr(self.camera, "_first"):
             self.camera(f"camera{self.index} start -t 1\n")
             self.camera._first = False
         else:
             self.camera(f"camera{self.index} start\n")
         self.camera.schedule(self)
+        self.display_camera.start_scene()
         # This listener is because you can have frames and windows and both need to care about the slider.
         self.camera.listen("camera;fps", self.on_fps_change)
         self.camera.listen("camera;stopped", self.on_camera_stop)
         self.camera.gui = self
         self.camera("camera focus -5% -5% 105% 105%\n")
+        label = _get_camera_attribute(
+            self.context.kernel,
+            self.index,
+            "desc",
+            _("Camera {index}").format(index=self.index),
+        )
+        if self.pane and self.pane_aui is not None:
+            # If this is a pane, we set the title of the pane.
+            # print (f"Setting pane title to {label}")
+            self.pane_aui.Caption(label)
+            self.pane_aui.caption = label
+        else:
+            self.GetParent().SetTitle(label)
+
+    @property
+    def caption(self):
+        # Property to get the caption of the corresponding pane menu item
+        return _get_camera_attribute(
+            self.context.kernel,
+            self.index,
+            "desc",
+            _("Camera {index}").format(index=self.index),
+        )
 
     def pane_hide(self, *args):
         self.camera(f"camera{self.index} stop\n")
         self.camera.unschedule(self)
+        self.display_camera.stop_scene()
         if not self.pane:
             self.camera.close(f"Camera{str(self.index)}")
         self.camera.unlisten("camera;fps", self.on_fps_change)
@@ -277,6 +345,19 @@ class CameraPanel(wx.Panel, Job):
         @return:
         """
         self.camera(f"camera{self.index} perspective reset\n")
+        if self.camera.perspective is None:
+            width = self.camera.width
+            height = self.camera.height
+            self.camera.perspective = [
+                [0, 0],
+                [width, 0],
+                [width, height],
+                [0, height],
+            ]
+        for v in self.widget_scene.widget_root.scene_widget:
+            if hasattr(v, "update"):
+                v.update()
+        self.widget_scene.request_refresh()
 
     def reset_fisheye(self, event=None):
         """
@@ -286,6 +367,7 @@ class CameraPanel(wx.Panel, Job):
         @return:
         """
         self.camera(f"camera{self.index} fisheye reset\n")
+        self.widget_scene.request_refresh()
 
     def on_check_perspective(self, event=None):
         """
@@ -354,9 +436,20 @@ class CameraPanel(wx.Panel, Job):
         def swap(event=None):
             self.camera(f"camera{self.index} --uri {str(uri)} stop start\n")
             self.frame_bitmap = None
+            self._remembered = uri
+            self.available_resolutions = []
+            if self.camera.is_physical:
+                self.available_resolutions = self.camera.guess_supported_resolutions()
 
         return swap
 
+    def set_resolution(self, w, h):
+        self.camera.set_resolution(w, h)
+        # restart camera
+        uri = self.camera.uri
+        self.camera(f"camera{self.index} --uri {str(uri)} stop start\n")
+        self.frame_bitmap = None
+
 
 class CamInterfaceWidget(Widget):
     def __init__(self, scene, camera):
@@ -388,6 +481,14 @@ class CamInterfaceWidget(Widget):
     def event(self, window_pos=None, space_pos=None, event_type=None, **kwargs):
         if event_type == "rightdown":
 
+            def set_resolution(width, height):
+                def handler(*args):
+                    self.cam.set_resolution(this_w, this_h)
+
+                this_w = width
+                this_h = height
+                return handler
+
             def enable_aspect(*args):
                 self.cam.camera.aspect = not self.cam.camera.aspect
                 self.scene.widget_root.set_aspect(self.cam.camera.aspect)
@@ -413,6 +514,35 @@ class CamInterfaceWidget(Widget):
 
                 return asp
 
+            def get_device_link():
+                if self.cam.index in self.cam.cam_device_link:
+                    return self.cam.cam_device_link[self.cam.index]
+                return ""
+
+            def set_device_link(devlabel):
+                if devlabel:
+                    self.cam.cam_device_link[self.cam.index] = devlabel
+                else:  # Empty or None
+                    self.cam.cam_device_link.pop(self.cam.index, None)
+
+            def change_label(event=None):
+                """
+                Change the label of the camera.
+                """
+                dialog = wx.TextEntryDialog(
+                    self.cam,
+                    _("Enter a new label for the camera:"),
+                    _("Change camera label"),
+                    str(self.cam.camera.desc or ""),
+                )
+                if dialog.ShowModal() == wx.ID_OK:
+                    new_label = dialog.GetValue().strip()
+                    if new_label:
+                        self.cam.context(
+                            f'camera{self.cam.index} label "{new_label}"\n'
+                        )
+                dialog.Destroy()
+
             menu = wx.Menu()
 
             item = menu.Append(wx.ID_ANY, _("Update Background"), "")
@@ -425,15 +555,16 @@ class CamInterfaceWidget(Widget):
             def live_view(c_frames, c_sec):
                 def runcam(event=None):
                     ratio = c_sec / c_frames
+                    dev_label = get_device_link()
                     self.cam.context(
-                        f"timer.updatebg 0 {ratio} camera{self.cam.index} background\n"
+                        f"timer.updatebg{self.cam.index} 0 {ratio} .camera{self.cam.index} background {dev_label}\n"
                     )
                     return
 
                 return runcam
 
             def live_stop():
-                self.cam.context("timer.updatebg --off\n")
+                self.cam.context(f"timer.updatebg{self.cam.index} --off\n")
 
             submenu = wx.Menu()
             menu.AppendSubMenu(submenu, _("...refresh"))
@@ -447,13 +578,59 @@ class CamInterfaceWidget(Widget):
                     live_view(rate_frame, rate_sec),
                     id=item.GetId(),
                 )
+
+            def has_live_job():
+                we_have_a_job = False
+                try:
+                    obj = self.cam.context.kernel.jobs[
+                        f"timer.updatebg{self.cam.index}"
+                    ]
+                    if obj is not None:
+                        we_have_a_job = True
+                except KeyError:
+                    pass
+                return we_have_a_job
+
+            if has_live_job():
+                submenu.AppendSeparator()
+                item = submenu.Append(wx.ID_ANY, "Disable")
+                self.cam.Bind(
+                    wx.EVT_MENU,
+                    lambda e: live_stop(),
+                    id=item.GetId(),
+                )
             submenu.AppendSeparator()
-            item = submenu.Append(wx.ID_ANY, "Disable")
-            self.cam.Bind(
-                wx.EVT_MENU,
-                lambda e: live_stop(),
-                id=item.GetId(),
+
+            def set_link(devlabel):
+                def handler(event):
+                    set_device_link(devlabel)
+
+                return handler
+
+            def unset_link(devlabel):
+                def handler(event):
+                    set_device_link("")
+
+                return handler
+
+            link = get_device_link()
+            item = submenu.Append(
+                wx.ID_ANY, _("Device independent"), kind=wx.ITEM_RADIO
             )
+            if link == "":
+                item.Check(True)
+                self.cam.Bind(wx.EVT_MENU, unset_link(""), id=item.GetId())
+            else:
+                self.cam.Bind(wx.EVT_MENU, set_link(""), id=item.GetId())
+            available_devices = self.cam.context.kernel.services("device")
+            for i, spool in enumerate(available_devices):
+                item = submenu.Append(wx.ID_ANY, spool.label, kind=wx.ITEM_RADIO)
+                if link == spool.label:
+                    item.Check(True)
+                    self.cam.Bind(wx.EVT_MENU, unset_link(spool.label), id=item.GetId())
+                else:
+                    self.cam.Bind(wx.EVT_MENU, set_link(spool.label), id=item.GetId())
+
             menu.AppendSeparator()
             item = menu.Append(wx.ID_ANY, _("Export Snapshot"), "")
             self.cam.Bind(
@@ -475,6 +652,21 @@ class CamInterfaceWidget(Widget):
                 lambda e: self.cam.context(f"camera{self.cam.index} stop\n"),
                 id=item.GetId(),
             )
+            if len(self.cam.available_resolutions):
+                cam_w, cam_h = self.cam.camera.get_resolution()
+                resmen = wx.Menu()
+                for res_w, res_h, res_desc in self.cam.available_resolutions:
+                    item = resmen.Append(
+                        wx.ID_ANY, f"{res_w}x{res_h} - {res_desc}", kind=wx.ITEM_RADIO
+                    )
+                    if res_h == cam_h and res_w == cam_w:
+                        item.Check(True)
+                    self.cam.Bind(
+                        wx.EVT_MENU,
+                        set_resolution(res_w, res_h),
+                        id=item.GetId(),
+                    )
+                menu.AppendSubMenu(resmen, _("Set camera resolution..."))
 
             item = menu.Append(wx.ID_ANY, _("Open CameraInterface"), "")
             self.cam.Bind(
@@ -536,14 +728,27 @@ class CamInterfaceWidget(Widget):
             def check_perspect(event=None):
                 self.cam.camera.correction_perspective = perspect.IsChecked()
 
+            def reset_perspect(event=None):
+                self.cam.camera(f"camera{self.cam.index} perspective reset\n")
+                if self.cam.camera.perspective is None:
+                    width = self.cam.camera.width
+                    height = self.cam.camera.height
+                    self.cam.camera.perspective = [
+                        [0, 0],
+                        [width, 0],
+                        [width, height],
+                        [0, height],
+                    ]
+                for v in self.scene.widget_root.scene_widget:
+                    if hasattr(v, "update"):
+                        v.update()
+
             self.cam.Bind(wx.EVT_MENU, check_perspect, perspect)
             menu.AppendSeparator()
             item = menu.Append(wx.ID_ANY, _("Reset Perspective"), "")
             self.cam.Bind(
                 wx.EVT_MENU,
-                lambda e: self.cam.camera(
-                    f"camera{self.cam.index} perspective reset\n"
-                ),
+                reset_perspect,
                 id=item.GetId(),
             )
             item = menu.Append(wx.ID_ANY, _("Reset Fisheye"), "")
@@ -598,6 +803,13 @@ class CamInterfaceWidget(Widget):
                 _("Manage URIs"),
                 sub_menu,
             )
+            menu.AppendSeparator()
+            item = menu.Append(wx.ID_ANY, _("Change Label"), "")
+            self.cam.Bind(
+                wx.EVT_MENU,
+                change_label,
+                id=item.GetId(),
+            )
             if menu.MenuItemCount != 0:
                 self.cam.PopupMenu(menu)
                 menu.Destroy()
@@ -679,6 +891,11 @@ class CamSceneWidget(Widget):
             if len(self):
                 self.remove_all_widgets()
 
+    def update(self):
+        for v in self:
+            if hasattr(v, "update"):
+                v.update()
+
 
 class CamImageWidget(Widget):
     def __init__(self, scene, camera):
@@ -702,9 +919,11 @@ class CameraInterface(MWindow):
                 pass
         if index is None:
             index = 0
+        self.index = index
         super().__init__(640, 480, context, path, parent, **kwds)
-        self.camera = self.context.get_context(f"camera/{index}")
-        self.panel = CameraPanel(self, wx.ID_ANY, context=self.camera, index=index)
+        self.camera = self.context.get_context(f"camera/{self.index}")
+        self.panel = CameraPanel(self, wx.ID_ANY, context=self.camera, index=self.index)
+        self.sizer.Add(self.panel, 1, wx.EXPAND, 0)
         self.add_module_delegate(self.panel)
 
         # ==========
@@ -719,12 +938,45 @@ class CameraInterface(MWindow):
         # ==========
 
         _icon = wx.NullIcon
-        _icon.CopyFromBitmap(icons8_camera_50.GetBitmap())
+        _icon.CopyFromBitmap(icons8_camera.GetBitmap())
         self.SetIcon(_icon)
-        self.SetTitle(_("CameraInterface {index}").format(index=index))
+        self.update_title()
         self.Layout()
+        self.restore_aspect()
+
+    def update_title(self):
+        """
+        Updates the title of the window to reflect the current camera index.
+        """
+        label = _get_camera_attribute(
+            self.context.kernel,
+            self.index,
+            "desc",
+            _("Camera {index}").format(index=self.index),
+        )
+        self.SetTitle(label)
 
     def create_menu(self, append):
+        def identify_cameras(event=None):
+            self.context("camdetect\n")
+
+        def change_label(event=None):
+            """
+            Change the label of the camera.
+            """
+            dialog = wx.TextEntryDialog(
+                self,
+                _("Enter a new label for the camera:"),
+                _("Change camera label"),
+                str(self.camera.desc or ""),
+            )
+            if dialog.ShowModal() == wx.ID_OK:
+                new_label = dialog.GetValue().strip()
+                if new_label:
+                    self.context(f'camera{self.index} label "{new_label}"\n')
+                    self.update_title()
+            dialog.Destroy()
+
         wxglade_tmp_menu = wx.Menu()
         item = wxglade_tmp_menu.Append(wx.ID_ANY, _("Reset Fisheye"), "")
         self.Bind(wx.EVT_MENU, self.panel.reset_fisheye, id=item.GetId())
@@ -757,6 +1009,16 @@ class CameraInterface(MWindow):
                 wx.ID_ANY, _("USB {usb_index}").format(usb_index=i), ""
             )
             self.Bind(wx.EVT_MENU, self.panel.swap_camera(i), id=item.GetId())
+        wxglade_tmp_menu.AppendSeparator()
+        item = wxglade_tmp_menu.Append(wx.ID_ANY, _("Change camera label"), "")
+        self.Bind(
+            wx.EVT_MENU,
+            change_label,
+            id=item.GetId(),
+        )
+        item = wxglade_tmp_menu.Append(wx.ID_ANY, _("Identify cameras"), "")
+        self.Bind(wx.EVT_MENU, identify_cameras, id=item.GetId())
+
         append(wxglade_tmp_menu, _("Camera"))
 
     def window_open(self):
@@ -770,106 +1032,51 @@ class CameraInterface(MWindow):
         camera = kernel.get_context("camera")
 
         def camera_click(index=None):
-            s = index
-
             def specific(event=None):
-                index = s
                 camera.default = index
                 v = camera.default
+                if v is None:
+                    v = 0
                 camera(f"window toggle -m {v} CameraInterface {v}\n")
 
             return specific
 
-        def detect_usb_cameras(search=None):
-            def specific(event=None):
-                try:
-                    import cv2
-                except ImportError:
-                    return
-
-                # Max range to look at
-                camera.setting(int, "search_range", 5)
-                camera.setting(list, "uris", [])
-                # Reset stuff...
-                for _index in range(5):
-                    if _index in camera.uris:
-                        camera.uris.remove(_index)
-
-                max_range = camera.search_range
-                if max_range is None or max_range < 1:
-                    max_range = 5
-                found = 0
-                found_camera_string = _("Cameras found: {count}")
-                progress = wx.ProgressDialog(
-                    _("Looking for Cameras (Range={max_range})").format(
-                        max_range=max_range
-                    ),
-                    found_camera_string.format(count=found),
-                    maximum=max_range,
-                    parent=None,
-                    style=wx.PD_APP_MODAL | wx.PD_CAN_ABORT,
-                )
-                # checks for cameras in the first x USB ports
-                index = 0
-                keepgoing = True
-                while index < max_range and keepgoing:
-                    try:
-                        cap = cv2.VideoCapture(index)
-                        if cap.read()[0]:
-                            camera.uris.append(index)
-                            cap.release()
-                            found += 1
-                    except:
-                        pass
-                    keepgoing = progress.Update(
-                        index + 1, found_camera_string.format(count=found)
-                    )
-                    index += 1
-                progress.Destroy()
-
-            return specific
+        def detect_usb_cameras(event=None):
+            camera("camdetect\n")
 
+        caminfo = []
+        for idx in range(5):
+            label = _get_camera_attribute(
+                kernel, idx, "desc", _("Camera {index}").format(index=idx)
+            )
+            caminfo.append(
+                {
+                    "identifier": f"cam{idx}",
+                    "label": label,
+                    "action": camera_click(idx),
+                    "signal": f"camset{idx}",
+                    "multi_autoexec": True,
+                },
+            )
+        caminfo.append(
+            {
+                "identifier": "id_cam",
+                "label": _("Identify cameras"),
+                "tip": _("Detects cameras on the system and sets them up"),
+                "action": detect_usb_cameras,
+                "multi_autoexec": True,
+            },
+        )
         kernel.register(
             "button/preparation/Camera",
             {
                 "label": _("Camera"),
-                "icon": icons8_camera_50,
+                "icon": icons8_camera,
                 "tip": _("Opens Camera Window"),
                 "identifier": "camera_id",
                 "action": camera_click(),
                 "priority": 3,
-                "multi": [
-                    {
-                        "identifier": "cam0",
-                        "label": _("Camera {index}").format(index=0),
-                        "action": camera_click(0),
-                    },
-                    {
-                        "identifier": "cam1",
-                        "label": _("Camera {index}").format(index=1),
-                        "action": camera_click(1),
-                    },
-                    {
-                        "identifier": "cam2",
-                        "label": _("Camera {index}").format(index=2),
-                        "action": camera_click(2),
-                    },
-                    {
-                        "identifier": "cam3",
-                        "label": _("Camera {index}").format(index=3),
-                        "action": camera_click(3),
-                    },
-                    {
-                        "identifier": "cam4",
-                        "label": _("Camera {index}").format(index=4),
-                        "action": camera_click(4),
-                    },
-                    {
-                        "identifier": "id_cam",
-                        "label": _("Identify cameras"),
-                        "action": detect_usb_cameras(True),
-                    },
-                ],
+                "multi": caminfo,
             },
         )
         kernel.register("window/CameraURI", CameraURI)
@@ -881,9 +1088,69 @@ class CameraInterface(MWindow):
         def camera_win(index=None, **kwargs):
             kernel.console(f"window open -m {index} CameraInterface {index}\n")
 
+        @kernel.console_command(
+            "camdetect", help=_("camdetect: Tries to detect cameras on the system")
+        )
+        def cam_detect(**kwargs):
+            try:
+                import cv2
+            except ImportError:
+                return
+
+            # Max range to look at
+            cam_context = kernel.get_context("camera")
+            cam_context.setting(int, "search_range", 5)
+            cam_context.setting(list, "uris", [])
+            # Reset stuff...
+            for _index in range(5):
+                if _index in cam_context.uris:
+                    cam_context.uris.remove(_index)
+
+            max_range = cam_context.search_range
+            if max_range is None or max_range < 1:
+                max_range = 5
+            found = 0
+            found_camera_string = _("Cameras found: {count}")
+            progress = wx.ProgressDialog(
+                _("Looking for Cameras (Range={max_range})").format(
+                    max_range=max_range
+                ),
+                found_camera_string.format(count=found),
+                maximum=max_range,
+                parent=None,
+                style=wx.PD_APP_MODAL | wx.PD_CAN_ABORT,
+            )
+            # checks for cameras in the first x USB ports
+            first_found = -1
+            index = 0
+            keepgoing = True
+            while index < max_range and keepgoing:
+                try:
+                    cap = cv2.VideoCapture(index)
+                    if cap.read()[0]:
+                        if first_found < 0:
+                            first_found = index
+                        cam_context.uris.append(index)
+                        cap.release()
+                        found += 1
+                except:
+                    pass
+                keepgoing = progress.Update(
+                    index + 1, found_camera_string.format(count=found)
+                )
+                index += 1
+            progress.Destroy()
+            if first_found >= 0:
+                kernel.signal(f"camset{first_found}", "camera", (first_found, found))
+
     @staticmethod
     def submenu():
-        return ("Camera", "Camera")
+        # Hint for translation: _("Camera"), _("Camera")
+        return "Camera", "Camera"
+
+    @staticmethod
+    def helptext():
+        return _("Display the camera window")
 
 
 class CameraURIPanel(wx.Panel):
@@ -891,16 +1158,21 @@ class CameraURIPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context.get_context("camera")
+        context.themes.set_window_colors(self)
         if index is None:
             index = 0
         self.index = index
         assert isinstance(self.index, int)
         self.context.setting(list, "uris", [])
-        self.list_uri = wx.ListCtrl(
-            self, wx.ID_ANY, style=wx.LC_HRULES | wx.LC_REPORT | wx.LC_VRULES
+        self.list_uri = wxListCtrl(
+            self,
+            wx.ID_ANY,
+            style=wx.LC_HRULES | wx.LC_REPORT | wx.LC_VRULES,
+            context=self.context,
+            list_name="list_camerauri",
         )
-        self.button_add = wx.Button(self, wx.ID_ANY, _("Add URI"))
-        self.text_uri = wx.TextCtrl(self, wx.ID_ANY, "")
+        self.button_add = wxButton(self, wx.ID_ANY, _("Add URI"))
+        self.text_uri = TextCtrl(self, wx.ID_ANY, "")
 
         self.__set_properties()
         self.__do_layout()
@@ -918,6 +1190,7 @@ class CameraURIPanel(wx.Panel):
         self.list_uri.SetToolTip(_("Displays a list of registered camera URIs"))
         self.list_uri.AppendColumn(_("Index"), format=wx.LIST_FORMAT_LEFT, width=69)
         self.list_uri.AppendColumn(_("URI"), format=wx.LIST_FORMAT_LEFT, width=348)
+        self.list_uri.resize_columns()
         self.button_add.SetToolTip(_("Add a new URL"))
         # end wxGlade
 
@@ -935,9 +1208,11 @@ class CameraURIPanel(wx.Panel):
 
     def pane_show(self):
         self.on_list_refresh()
+        self.list_uri.load_column_widths()
 
     def pane_hide(self):
         self.commit()
+        self.list_uri.save_column_widths()
 
     def commit(self):
         if not self.changed:
@@ -1007,7 +1282,10 @@ class CameraURIPanel(wx.Panel):
                 _("Camera URI"),
                 "",
             )
-            dlg.SetValue(self.context.uris[index])
+            try:
+                dlg.SetValue(self.context.uris[index])
+            except IndexError:
+                return
             if dlg.ShowModal() == wx.ID_OK:
                 self.context.uris[index] = dlg.GetValue()
                 self.changed = True
@@ -1017,9 +1295,12 @@ class CameraURIPanel(wx.Panel):
 
     def on_tree_popup_clear(self, index):
         def delete(event):
-            self.context.uris.clear()
-            self.changed = True
-            self.on_list_refresh()
+            if self.context.kernel.yesno(
+                _("Do you really want to delete all entries?"), caption=_("URI-Manager")
+            ):
+                self.context.uris.clear()
+                self.changed = True
+                self.on_list_refresh()
 
         return delete
 
@@ -1039,13 +1320,14 @@ class CameraURIPanel(wx.Panel):
 class CameraURI(MWindow):
     def __init__(self, *args, index=None, **kwds):
         super().__init__(437, 530, *args, **kwds)
-
         self.panel = CameraURIPanel(self, wx.ID_ANY, context=self.context, index=index)
+        self.sizer.Add(self.panel, 1, wx.EXPAND, 0)
         _icon = wx.NullIcon
-        _icon.CopyFromBitmap(icons8_camera_50.GetBitmap())
+        _icon.CopyFromBitmap(icons8_camera.GetBitmap())
         self.SetIcon(_icon)
         # begin wxGlade: CameraURI.__set_properties
         self.SetTitle(_("URI Manager"))
+        self.restore_aspect()
 
     def window_open(self):
         self.panel.pane_show()
@@ -1055,4 +1337,9 @@ class CameraURI(MWindow):
 
     @staticmethod
     def submenu():
-        return ("Camera", "Sources")
+        # Hint for translation: _("Camera"), _("Sources")
+        return "Camera", "Sources"
+
+    @staticmethod
+    def helptext():
+        return _("Edit camera sources")
