@@ -1,45 +1,72 @@
 from meerk40t.kernel import CommandMatchRejected, Service
 
-# The following dicts consist of a tuple of values, the first of
-# which is the current default, with any prior defaults following.
-# Prior defaults are used to keep a users keymap / aliases up to date with
-# any changes, unless the user has already changed away from the default.
-# If you want to delete a bind / alias, set the first value to a null string.
+"""
+This core class defines two main services: bind and alias. This is a set defined keybinds and alias commands.
 
-# CHOICE OF BINDALIAS vs. MENU
-# These key bindings are in addition to those specified in the menus
-# (like Ctrl+S for File Save). The choice of which of these to use is debatable.
-# On one hand bindalias is a more flexible approach because the user can
-# change keys mappings to whatever they wish whilst menus are fixed.
-# On the other hand, default menu keys can be set per language because the
-# menu strings are translated, whereas bindalias keys are not.
+If a keybind commands starts with a "+" then the command is triggered on the down-key and the "-" version of
+the same command is triggered on the up-key. If the bind is for any other command, it will execute on key-press.
+If you need this functionality it's best to define these commands as aliases of other commands.
+"""
 
-# RESERVED KEYS
-# Regardless of the platform used by individual developers, bindalias needs
-# to conform to BOTH of the following:
-#
-# 1. Mac - common shortcuts defined in https://support.apple.com/en-us/HT201236
-#    should be followed where appropriate and avoided if not. This is important
-#    for MK to be accepted into the Apple app store.
-#    Where keys defined in other sections on this page make sense for this application,
-#    these should also be used by preference.
-#
-# 2. Windows - alt-single letter should be avoided as these correspond to menu
-#    shortcuts (e.g. in english locale, alt-f should activate the file menu).
-#    Common Windows shortcuts can be found at
-#    https://support.microsoft.com/en-us/windows/keyboard-shortcuts-in-windows-dcc61a57-8ff0-cffe-9796-cb9706c75eec
-#
-# In addition where they do not conflict with the above, any synergy with keys commonly used
-# in popular SVG / image editors (e.g. paint.net, inkscape) may be beneficial.
 
-# To change keymaps / alias insert new alias at the beginning of the tuple
-# Later entries in the tuple are used to identify previous defaults and update them to current,
-# so do not delete these until a version change (like 0.8) results in completely new settings anyway.
+"""
+The following dicts consist of a tuple of values, the first of
+which is the current default, with any prior defaults following.
+Prior defaults are used to keep a users keymap / aliases up to date with
+any changes, unless the user has already changed away from the default.
+If you want to delete a bind / alias, set the first value to a null string.
+
+CHOICE OF BINDALIAS vs. MENU
+These key bindings are in addition to those specified in the menus
+(like Ctrl+S for File Save). The choice of which of these to use is debatable.
+On one hand bindalias is a more flexible approach because the user can
+change keys mappings to whatever they wish whilst menus are fixed.
+On the other hand, default menu keys can be set per language because the
+menu strings are translated, whereas bindalias keys are not.
+
+RESERVED KEYS
+Regardless of the platform used by individual developers, bindalias needs
+to conform to BOTH of the following:
+
+1. Mac - common shortcuts defined in https://support.apple.com/en-us/HT201236
+   should be followed where appropriate and avoided if not. This is important
+   for MK to be accepted into the Apple app store.
+   Where keys defined in other sections on this page make sense for this application,
+   these should also be used by preference.
+2. Windows - alt-single letter should be avoided as these correspond to menu
+   shortcuts (e.g. in english locale, alt-f should activate the file menu).
+   Common Windows shortcuts can be found at
+   https://support.microsoft.com/en-us/windows/keyboard-shortcuts-in-windows-dcc61a57-8ff0-cffe-9796-cb9706c75eec
+
+In addition where they do not conflict with the above, any synergy with keys commonly used
+in popular SVG / image editors (e.g. paint.net, inkscape) may be beneficial.
+
+To change keymaps / alias insert new alias at the beginning of the tuple
+Later entries in the tuple are used to identify previous defaults and update them to current,
+so do not delete these until a version change (like 0.8) results in completely new settings anyway.
+"""
+
 DEFAULT_KEYMAP = {
     "right": ("translate 1mm 0",),
     "left": ("translate -1mm 0",),
     "up": ("translate 0 -1mm",),
     "down": ("translate 0 1mm",),
+    "shift+right": ("translate 10mm 0",),
+    "shift+left": ("translate -10mm 0",),
+    "shift+up": ("translate 0 -10mm",),
+    "shift+down": ("translate 0 10mm",),
+    "alt+shift+right": ("translate 10spx 0",),
+    "alt+shift+left": ("translate -10spx 0",),
+    "alt+shift+up": ("translate 0 -10spx",),
+    "alt+shift+down": ("translate 0 10spx",),
+    "alt+right": ("translate 1spx 0",),
+    "alt+left": ("translate -1spx 0",),
+    "alt+up": ("translate 0 -1spx",),
+    "alt+down": ("translate 0 1spx",),
+    "ctrl+shift+right": ("translate 0.1mm 0",),
+    "ctrl+shift+left": ("translate -0.1mm 0",),
+    "ctrl+shift+up": ("translate 0 -0.1mm",),
+    "ctrl+shift+down": ("translate 0 0.1mm",),
     "a": ("+left",),
     "d": ("+right",),
     "w": ("+up",),
@@ -132,8 +159,15 @@ DEFAULT_KEYMAP = {
         "",
         "rotaryscale",
     ),
-    "ctrl+a": ("element* select",),
-    "ctrl+c": ("clipboard copy",),
+    "ctrl+a": (
+        "",
+        "element* select",
+    ),
+    "ctrl+c": (
+        "",
+        "clipboard copy",
+    ),
+    "ctrl+shift+c": ("align bed group xy center center",),
     "ctrl+e": (
         "circle 0.5in 0.5in 0.5in stroke red classify",
         "circle 500 500 500",
@@ -142,9 +176,15 @@ DEFAULT_KEYMAP = {
         "",
         "dialog_fill",
     ),
-    "ctrl+i": ("element* select^",),
+    "ctrl+i": (
+        "",
+        "element* select^",
+    ),
     "ctrl+d": ("element copy",),
-    "ctrl+g": ("planz clear copy preprocess validate blob preopt optimize spool",),
+    "ctrl+g": (
+        "",
+        "planz clear copy preprocess validate blob preopt optimize spool",
+    ),
     "ctrl+o": (
         "",
         "outline",
@@ -157,9 +197,23 @@ DEFAULT_KEYMAP = {
         "",
         "dialog_stroke",
     ),
-    "ctrl+v": ("clipboard paste",),
-    "ctrl+x": ("clipboard cut",),
-    "ctrl+z": ("reset",),
+    "ctrl+v": (
+        "",
+        "clipboard paste",
+    ),
+    "ctrl+x": (
+        "",
+        "clipboard cut",
+    ),
+    "ctrl+z": (
+        "",
+        "undo",
+        "reset",
+    ),
+    "ctrl+shift+z": (
+        "",
+        "redo",
+    ),
     "ctrl+1": ("bind 1 move $x $y",),
     "ctrl+2": ("bind 2 move $x $y",),
     "ctrl+3": ("bind 3 move $x $y",),
@@ -194,6 +248,11 @@ DEFAULT_KEYMAP = {
         "reset_bind_alias",
     ),
     "ctrl+alt+shift+home": ("bind default;alias default",),
+    # That's not working, so we delete it...
+    "ctrl+shift+l": (
+        "",
+        "signal lock_helper",
+    ),
 }
 DEFAULT_ALIAS = {
     "+scale_up": (".timerscale_up 0 0.1 .scale 1.02",),
@@ -208,6 +267,7 @@ DEFAULT_ALIAS = {
     "+left": (".timerleft 0 0.1 left 1mm",),
     "+up": (".timerup 0 0.1 up 1mm",),
     "+down": (".timerdown 0 0.1 down 1mm",),
+    "burn": ("planz clear copy preprocess validate blob preopt optimize spool",),
     "-scale_up": (".timerscale_up off",),
     "-scale_down": (".timerscale_down off",),
     "-rotate_cw": (".timerrotate_cw off",),
@@ -258,7 +318,11 @@ class Bind(Service):
                         key,
                     )
 
-                channel(_("    Key                    Command"))
+                channel(
+                    "    {key} {command}".format(
+                        key=_("Key").ljust(22), command=_("Command")
+                    )
+                )
                 for i, key in enumerate(sorted(self.keymap.keys(), key=keymap_index)):
                     value = self.keymap[key]
                     channel(f"{i:2d}: {key.ljust(22)} {value}")
@@ -304,6 +368,7 @@ class Bind(Service):
         return value, keyvalue
 
     def trigger(self, keyvalue):
+        # print (f"trigger for {keyvalue} started with {self.triggered}")
         fnd, keyvalue = self.is_found(keyvalue, self.keymap)
         if fnd:
             fnd, keyvalue = self.is_found(keyvalue, self.triggered)
@@ -313,10 +378,11 @@ class Bind(Service):
                 cmds = (action,) if action[0] in "+-" else action.split(";")
                 for cmd in cmds:
                     self(f"{cmd}\n")
-                return True
+            return True
         return False
 
     def untrigger(self, keyvalue):
+        # print (f"untrigger for {keyvalue} started with {self.triggered}")
         keymap = self.keymap
         fnd, keyvalue = self.is_found(keyvalue, self.keymap)
         if fnd:
@@ -328,7 +394,7 @@ class Bind(Service):
                 # Keyup commands only trigger if the down command started with +
                 action = "-" + action[1:]
                 self(action + "\n")
-                return True
+            return True
         return False
 
     def shutdown(self, *args, **kwargs):
@@ -379,12 +445,16 @@ class Alias(Service):
         @self.console_command(
             "alias", help=_("alias <alias> <console commands[;console command]*>")
         )
-        def alias(command, channel, _, alias=None, remainder=None, **kwgs):
+        def alias_command(command, channel, _, alias=None, remainder=None, **kwgs):
             _ = self._
             if alias is None:
                 reverse_keymap = {v: k for k, v in self.bind.keymap.items()}
-                channel(_("Aliases (keybind)`:"))
-                channel(_("    Alias                  Command(s)"))
+                channel(_("Aliases (keybind):"))
+                channel(
+                    "    {key} {value}".format(
+                        key=_("Alias").ljust(22), value=_("Command(s)")
+                    )
+                )
                 last = None
                 i = -1
                 for key in sorted(
