@@ -9,40 +9,45 @@ class FileNode(Node):
     Bootstrapped type: 'file'
     """
 
-    def __init__(self, filepath=None, **kwargs):
-        super(FileNode, self).__init__(type="file", **kwargs)
-        self._filepath = filepath
+    def __init__(self, **kwargs):
+        self.filepath = None
+        super().__init__(type="file", **kwargs)
         self._formatter = "{element_type}: {filename}"
 
     def default_map(self, default_map=None):
-        default_map = super(FileNode, self).default_map(default_map=default_map)
+        default_map = super().default_map(default_map=default_map)
         default_map["element_type"] = "File"
         if self.filepath is None:
             s = "None"
         else:
-            s = os.path.basename(self._filepath)
-        default_map["full_filename"] = self._filepath
+            s = os.path.basename(self.filepath)
+        default_map["full_filename"] = self.filepath
         default_map["filename"] = s
         return default_map
 
-    def drop(self, drag_node, modify=True):
+        return False
+
+    def can_drop(self, drag_node):
+        if self.is_a_child_of(drag_node):
+            return False
+        if drag_node.type == "group":
+            return True
+        return False
+
+    def drop(self, drag_node, modify=True, flag=False):
+        # Do not allow dragging onto children
+        if not self.can_drop(drag_node):
+            return False
         if drag_node.type == "group":
             if modify:
                 self.append_child(drag_node)
+            return True
         return False
 
-    @property
-    def filepath(self):
-        return self._filepath
-
-    @filepath.setter
-    def filepath(self, value):
-        self._filepath = value
-
     @property
     def name(self):
         if self.filepath is None:
             s = None
         else:
-            s = os.path.basename(self._filepath)
+            s = os.path.basename(self.filepath)
         return s
