@@ -6,13 +6,13 @@ class LayerNode(Node):
     Bootstrapped type: 'layer'
     """
 
-    def __init__(self, layer_name=None, **kwargs):
-        super(LayerNode, self).__init__(type="layer", **kwargs)
+    def __init__(self, **kwargs):
+        self.layer_name = None
+        super().__init__(type="layer", **kwargs)
         self._formatter = "{element_type} {id} ({children} elems)"
-        self.layer_name = layer_name
 
     def default_map(self, default_map=None):
-        default_map = super(LayerNode, self).default_map(default_map=default_map)
+        default_map = super().default_map(default_map=default_map)
         default_map["name"] = self.layer_name
         default_map["element_type"] = "Layer"
         default_map["children"] = str(len(self.count_children()))
@@ -23,7 +23,7 @@ class LayerNode(Node):
             if hasattr(self.parent, "activate"):
                 self.parent.activate(self.layer_name)
 
-    def drop(self, drag_node, modify=True):
+    def drop(self, drag_node, modify=True, flag=False):
         return False
 
     @property
