@@ -2,22 +2,26 @@ from copy import copy
 
 from meerk40t.kernel import Service
 
-from ..core.cutcode import CutCode
+from ..core.cutcode.cutcode import CutCode
 from .cutplan import CutPlan, CutPlanningFailedError
 from .node.op_cut import CutOpNode
 from .node.op_dots import DotsOpNode
 from .node.op_engrave import EngraveOpNode
-from .node.op_hatch import HatchOpNode
 from .node.op_image import ImageOpNode
 from .node.op_raster import RasterOpNode
 from .node.util_console import ConsoleOperation
 from .node.util_goto import GotoOperation
 from .node.util_home import HomeOperation
-from .node.util_origin import SetOriginOperation
 from .node.util_output import OutputOperation
 from .node.util_wait import WaitOperation
 from .units import Length
 
+"""
+The planner module provides cut planning services. This provides a method of going from operations + elements into
+cutcode which is then put inside a laserjob and sent to a spooler. Most of these operations are called on the
+individual CutPlan objects.
+"""
+
 
 def plugin(kernel, lifecycle=None):
     if lifecycle == "register":
@@ -39,8 +43,44 @@ def plugin(kernel, lifecycle=None):
             },
         ]
         kernel.register_choices("planner", choices)
-
+        INNER_WARNING = _(
+            "Notabene: Reduce Travel Time and Burn Inner First cannot be used at the same time."
+        )
         choices = [
+            {
+                "attr": "opt_raster_optimisation",
+                "object": context,
+                "default": True,
+                "type": bool,
+                "label": _("Cluster raster objects"),
+                "tip": _(
+                    "Separate non-overlapping raster objects.\n"
+                    "Active: this will raster close (i.e. overlapping) objects as one,\n"
+                    "but will separately process objects lying apart from each other.\n"
+                    "Inactive: all objects will be lasered as one single unit."
+                ),
+                "page": "Optimisations",
+                # Hint for translation _("Reducing Movements")
+                "section": "_20_Reducing Movements",
+                # Hint for translation _("Splitting rasters")
+                "subsection": "Splitting rasters",
+            },
+            {
+                "attr": "opt_raster_opt_margin",
+                "object": context,
+                "default": "1mm",
+                "type": Length,
+                "label": _("Margin:"),
+                "tip": _(
+                    "Allowed gap between rasterable objects, to still be counted as one."
+                ),
+                "page": "Optimisations",
+                # Hint for translation _("Reducing Movements")
+                "section": "_20_Reducing Movements",
+                # Hint for translation _("Splitting rasters")
+                "subsection": "Splitting rasters",
+                "conditional": (context, "opt_raster_optimisation"),
+            },
             {
                 "attr": "opt_reduce_travel",
                 "object": context,
@@ -63,9 +103,12 @@ def plugin(kernel, lifecycle=None):
                     "When this option IS checked, Meerk40t will burn each subpath "
                     + "and then move to the nearest remaining subpath instead, "
                     + "reducing the time taken moving between burn items."
-                ),
-                "page": "Optimizations",
-                "section": "",
+                )
+                + "\n"
+                + INNER_WARNING,
+                "page": "Optimisations",
+                # Hint for translation _("Reducing Movements")
+                "section": "_20_Reducing Movements",
             },
             {
                 "attr": "opt_complete_subpaths",
@@ -89,8 +132,10 @@ def plugin(kernel, lifecycle=None):
                     + "It may also avoid minor differences in total burn depth "
                     + "at the point the burns join. "
                 ),
-                "page": "Optimizations",
-                "section": "Burn sequence",
+                "page": "Optimisations",
+                # Hint for translation _("Reducing Movements")
+                "section": "_20_Reducing Movements",
+                "conditional": (context, "opt_reduce_travel"),
             },
             {
                 "attr": "opt_merge_passes",
@@ -117,8 +162,10 @@ def plugin(kernel, lifecycle=None):
                     + "and this can result in greater charring "
                     + "or even an increased risk of the material catching fire."
                 ),
-                "page": "Optimizations",
-                "section": "Merging",
+                "page": "Optimisations",
+                # Hint for translation _("Reducing Movements")
+                "section": "_20_Reducing Movements",
+                "conditional": (context, "opt_reduce_travel"),
             },
             {
                 "attr": "opt_merge_ops",
@@ -141,8 +188,39 @@ def plugin(kernel, lifecycle=None):
                     "If you have a complex design with many paths across multiple consecutive burn operations, "
                     + "using this option can significantly INCREASE the optimisation time. "
                 ),
-                "page": "Optimizations",
-                "section": "Merging",
+                "page": "Optimisations",
+                # Hint for translation _("Reducing Movements")
+                "section": "_20_Reducing Movements",
+                "conditional": (context, "opt_reduce_travel"),
+            },
+            {
+                "attr": "opt_stitching",
+                "object": context,
+                "default": False,
+                "type": bool,
+                "label": _("Combine path segments"),
+                "tip": _(
+                    "Stitch segments together that are very close (ideally having joint start/end points)."
+                )
+                + "\n"
+                + _("Only inside a single cut/engrave operation."),
+                "page": "Optimisations",
+                # Hint for translation _("Stitching")
+                "section": "_05_Stitching",
+            },
+            {
+                "attr": "opt_stitch_tolerance",
+                "object": context,
+                "default": "0",
+                "type": Length,
+                "label": _("Tolerance"),
+                "tip": _(
+                    "Tolerance to decide whether two path segments should be joined."
+                ),
+                "page": "Optimisations",
+                # Hint for translation _("Stitching")
+                "section": "_05_Stitching",
+                "conditional": (context, "opt_stitching"),
             },
             {
                 "attr": "opt_inner_first",
@@ -161,9 +239,24 @@ def plugin(kernel, lifecycle=None):
                     + "* Deselecting Cut Inner First if you are not cutting fully through your material \n"
                     + "* Putting the inner paths into a separate earlier operation(s) and not using Merge Operations or Cut Inner First \n"
                     + "* If you are using multiple passes, check Merge Passes"
-                ),
-                "page": "Optimizations",
-                "section": "Burn sequence",
+                )
+                + "\n"
+                + INNER_WARNING,
+                "page": "Optimisations",
+                # Hint for translation _("Burn sequence")
+                "section": "_10_Burn sequence",
+            },
+            {
+                "attr": "opt_inner_tolerance",
+                "object": context,
+                "default": "0",
+                "type": Length,
+                "label": _("Tolerance"),
+                "tip": _("Tolerance to decide if a shape is truly inside another one."),
+                "page": "Optimisations",
+                # Hint for translation _("Burn sequence")
+                "section": "_10_Burn sequence",
+                "conditional": (context, "opt_inner_first"),
             },
             {
                 "attr": "opt_inners_grouped",
@@ -172,7 +265,7 @@ def plugin(kernel, lifecycle=None):
                 "type": bool,
                 "label": _("Group Inner Burns"),
                 "tip": _(
-                    "Try to complete a set of inner burns and the associated outer cut before moving onto other elements."
+                    "Try to complete a set of inner burns and the associated outer cut before moving onto other elements.\n"
                     + "This option only does something if Burn Inner First is also selected. "
                     + "If your design has multiple separate pieces on it, "
                     + "this should mostly cause each piece to be burned in entirety "
@@ -190,8 +283,10 @@ def plugin(kernel, lifecycle=None):
                     + "inner elements may span multiple design pieces, "
                     + "in which case they may be optimised together."
                 ),
-                "page": "Optimizations",
-                "section": "Burn sequence",
+                "page": "Optimisations",
+                # Hint for translation _("Burn sequence")
+                "section": "_10_Burn sequence",
+                "conditional": (context, "opt_inner_first"),
             },
             {
                 "attr": "opt_closed_distance",
@@ -202,12 +297,87 @@ def plugin(kernel, lifecycle=None):
                 "tip": _(
                     "How close in device specific natural units do endpoints need to be to count as closed?"
                 ),
-                "page": "Optimizations",
-                "section": "",
+                "page": "Optimisations",
+                # Hint for translation _("Reducing Movements")
+                "section": "_20_Reducing Movements",
+                "hidden": True,
+            },
+            {
+                "attr": "opt_effect_combine",
+                "object": context,
+                "default": True,
+                "type": bool,
+                "label": _("Keep effect lines together"),
+                "tip": (
+                    _("Active: effects like hatches are dealt with as a bigger shape")
+                    + "\n"
+                    + _(
+                        "Inactive: every single line segment will be dealt with individually."
+                    )
+                ),
+                "page": "Optimisations",
+                # Hint for translation _("Effects")
+                "section": "_25_Effects",
+            },
+            {
+                "attr": "opt_effect_optimize",
+                "object": context,
+                "default": False,
+                "type": bool,
+                "label": _("Optimize internally"),
+                "tip": (
+                    _("Active: hatch lines will be optimized internally")
+                    + "\n"
+                    + _("Inactive: hatch lines will be burnt sequentially.")
+                ),
+                "page": "Optimisations",
+                # Hint for translation _("Effects")
+                "section": "_25_Effects",
+                "conditional": (context, "opt_effect_combine"),
+            },
+            {
+                "attr": "opt_reduce_details",
+                "object": context,
+                "default": False,
+                "type": bool,
+                "label": _("Reduce polyline details"),
+                "tip": _(
+                    "Active: reduce the details of polyline elements,\n"
+                    + "so that less information needs to be sent to the laser."
+                )
+                + "\n"
+                + _(
+                    "This can reduce the processing and laser time but can as well\n"
+                    + "compromise the quality at higher levels, so use with care and preview in simulation."
+                ),
+                "page": "Optimisations",
+                # Hint for translation _("Details")
+                "section": "_30_Details",
+                "subsection": "_10_",
+            },
+            {
+                "attr": "opt_reduce_tolerance",
+                "object": context,
+                "default": 10,
+                "type": int,
+                "label": _("Level"),
+                "style": "option",
+                "choices": (1, 10, 50, 100),
+                "display": (_("Minimal"), _("Fine"), _("Medium"), _("Coarse")),
+                "tip": _(
+                    "This can reduce the processing and laser time but can as well\n"
+                    + "compromise the quality at higher levels, so use with care and preview in simulation."
+                ),
+                "page": "Optimisations",
+                # Hint for translation _("Details")
+                "section": "_30_Details",
+                "subsection": "_10_",
+                "conditional": (context, "opt_reduce_details"),
             },
         ]
+        for c in choices:
+            c["help"] = "optimisation"
         kernel.register_choices("optimize", choices)
-
         context.setting(bool, "opt_2opt", False)
         context.setting(bool, "opt_nearest_neighbor", True)
         context.setting(bool, "opt_reduce_directions", False)
@@ -222,9 +392,11 @@ def plugin(kernel, lifecycle=None):
     elif lifecycle == "poststart":
         planner = kernel.planner
         auto = hasattr(kernel.args, "auto") and kernel.args.auto
+        console = hasattr(kernel.args, "console") and kernel.args.console
+        quit = hasattr(kernel.args, "quit") and kernel.args.quit
         if auto:
             planner("plan copy preprocess validate blob preopt optimize\n")
-            if hasattr(kernel.args, "quit") and kernel.args.quit:
+            if quit or not console:
                 planner("plan console quit\n")
             planner("plan spool\n")
 
@@ -239,15 +411,16 @@ class Planner(Service):
         Service.__init__(self, kernel, "planner")
         self._plan = dict()
         self._default_plan = "0"
+        self.do_optimization = True
 
     def length(self, v):
         return float(Length(v))
 
     def length_x(self, v):
-        return float(Length(v, relative_length=self.device.width))
+        return float(Length(v, relative_length=self.device.view.width))
 
     def length_y(self, v):
-        return float(Length(v, relative_length=self.device.height))
+        return float(Length(v, relative_length=self.device.view.height))
 
     def get_or_make_plan(self, plan_name):
         """
@@ -314,169 +487,26 @@ class Planner(Service):
             return "plan", cutplan
 
         @self.console_command(
-            "copy-selected",
-            help=_("plan<?> copy-selected"),
-            input_type="plan",
-            output_type="plan",
-        )
-        def plan_copy_selected(command, channel, _, data_type=None, data=None, **kwgs):
-            def init_settings():
-                for prefix in ("prepend", "append"):
-                    str_count = f"{prefix}_op_count"
-                    self.device.setting(int, str_count, 0)
-                    value = getattr(self.device, str_count, 0)
-                    if value > 0:
-                        for idx in range(value):
-                            attr1 = f"{prefix}_op_{idx:02d}"
-                            attr2 = f"{prefix}_op_param_{idx:02d}"
-                            self.device.setting(str, attr1, "")
-                            self.device.setting(str, attr2, "")
-
-            def add_ops(is_prepend):
-                # Do we have any default actions to include first?
-                if is_prepend:
-                    prefix = "prepend"
-                else:
-                    prefix = "append"
-                try:
-                    if is_prepend:
-                        count = self.device.prepend_op_count
-                    else:
-                        count = self.device.append_op_count
-                except AttributeError:
-                    count = 0
-                idx = 0
-                while idx <= count - 1:
-                    addop = None
-                    attr1 = f"{prefix}_op_{idx:02d}"
-                    attr2 = f"{prefix}_op_param_{idx:02d}"
-                    if hasattr(self.device, attr1):
-                        optype = getattr(self.device, attr1, None)
-                        opparam = getattr(self.device, attr2, None)
-                        if optype is not None:
-                            if optype == "util console":
-                                addop = ConsoleOperation(command=opparam)
-                            elif optype == "util home":
-                                addop = HomeOperation()
-                            elif optype == "util output":
-                                if opparam is not None:
-                                    params = opparam.split(",")
-                                    mask = 0
-                                    setvalue = 0
-                                    if len(params) > 0:
-                                        try:
-                                            mask = int(params[0])
-                                        except ValueError:
-                                            mask = 0
-                                    if len(params) > 1:
-                                        try:
-                                            setvalue = int(params[1])
-                                        except ValueError:
-                                            setvalue = 0
-                                    if mask != 0 or setvalue != 0:
-                                        addop = OutputOperation(mask, setvalue)
-                            elif optype == "util goto":
-                                if opparam is not None:
-                                    params = opparam.split(",")
-                                    x = 0
-                                    y = 0
-                                    if len(params) > 0:
-                                        try:
-                                            x = float(Length(params[0]))
-                                        except ValueError:
-                                            x = 0
-                                    if len(params) > 1:
-                                        try:
-                                            y = float(Length(params[1]))
-                                        except ValueError:
-                                            y = 0
-                                    addop = GotoOperation(x=x, y=y)
-                            elif optype == "util origin":
-                                if opparam is not None and opparam != "":
-                                    params = opparam.split(",")
-                                    x = 0
-                                    y = 0
-                                    if len(params) > 0:
-                                        try:
-                                            x = float(Length(params[0]))
-                                        except ValueError:
-                                            x = 0
-                                    if len(params) > 1:
-                                        try:
-                                            y = float(Length(params[1]))
-                                        except ValueError:
-                                            y = 0
-                                    addop = SetOriginOperation(x=x, y=y)
-                                else:
-                                    addop = SetOriginOperation(x=None, y=None)
-                            elif optype == "util wait":
-                                if opparam is not None:
-                                    try:
-                                        opparam = float(opparam)
-                                    except ValueError:
-                                        opparam = None
-                                if opparam is not None:
-                                    addop = WaitOperation(wait=opparam)
-                    if addop is not None:
-                        try:
-                            if not addop.output:
-                                continue
-                        except AttributeError:
-                            pass
-                        try:
-                            if len(addop) == 0:
-                                continue
-                        except TypeError:
-                            pass
-                        if addop.type == "cutcode":
-                            # CutNodes are denuded into normal objects.
-                            addop = addop.cutcode
-                        copy_c = copy(addop)
-                        try:
-                            copy_c.copy_children_as_real(addop)
-                        except AttributeError:
-                            pass
-                        data.plan.append(copy_c)
-
-                    idx += 1
-
-            init_settings()
-            # Add default start ops
-            add_ops(True)
-
-            for c in self.elements.ops(emphasized=True):
-                if c.type == "cutcode":
-                    # CutNodes are denuded into normal objects.
-                    c = c.cutcode
-                elif c.type == "blob":
-                    # BlobNodes are denuded into normal objects.
-                    c = c.blob
-                copy_c = copy(c)
-                try:
-                    copy_c.copy_children_as_real(c)
-                except AttributeError:
-                    pass
-                try:
-                    if not copy_c.output:
-                        copy_c.output = True
-                except AttributeError:
-                    pass
-                data.plan.append(copy_c)
-
-            # Add default start ops
-            add_ops(False)
-
-            channel(_("Copied Operations."))
-            self.signal("plan", data.name, 1)
-            return data_type, data
-
-        @self.console_command(
-            "copy",
-            help=_("plan<?> copy"),
+            ("copy", "copy-selected"),
+            help=_("plan(-selected)<?> copy"),
             input_type="plan",
             output_type="plan",
         )
         def plan_copy(command, channel, _, data_type=None, data=None, **kwgs):
+            # Update Info-panel if displayed
+            busy = self.kernel.busyinfo
+            if busy.shown:
+                busy.change(msg=_("Copy data"), keep=1)
+                busy.show()
+
+            operations = data  # unused.
+            if command == "copy-selected":
+                operations = list(self.elements.ops(emphasized=True))
+                copy_selected = True
+            else:
+                operations = list(self.elements.ops())
+                copy_selected = False
+
             def init_settings():
                 for prefix in ("prepend", "append"):
                     str_count = f"{prefix}_op_count"
@@ -531,7 +561,9 @@ class Planner(Service):
                                         except ValueError:
                                             setvalue = 0
                                     if mask != 0 or setvalue != 0:
-                                        addop = OutputOperation(mask, setvalue)
+                                        addop = OutputOperation(
+                                            output_mask=mask, output_value=setvalue
+                                        )
                             elif optype == "util goto":
                                 if opparam is not None:
                                     params = opparam.split(",")
@@ -547,25 +579,10 @@ class Planner(Service):
                                             y = float(Length(params[1]))
                                         except ValueError:
                                             y = 0
-                                    addop = GotoOperation(x=x, y=y)
-                            elif optype == "util origin":
-                                if opparam is not None and opparam != "":
-                                    params = opparam.split(",")
-                                    x = 0
-                                    y = 0
-                                    if len(params) > 0:
-                                        try:
-                                            x = float(Length(params[0]))
-                                        except ValueError:
-                                            x = 0
-                                    if len(params) > 1:
-                                        try:
-                                            y = float(Length(params[1]))
-                                        except ValueError:
-                                            y = 0
-                                    addop = SetOriginOperation(x=x, y=y)
-                                else:
-                                    addop = SetOriginOperation(x=None, y=None)
+                                    absolute = False
+                                    if len(params) > 2:
+                                        absolute = params[2] not in ("False", "0")
+                                    addop = GotoOperation(x=x, y=y, absolute=absolute)
                             elif optype == "util wait":
                                 if opparam is not None:
                                     try:
@@ -598,50 +615,56 @@ class Planner(Service):
                     idx += 1
 
             init_settings()
-            operations = self.elements.get(type="branch ops")
 
             # Add default start ops
             add_ops(True)
-            for c in operations.flat(
-                types=(
-                    "op cut",
-                    "op raster",
-                    "op image",
-                    "op engrave",
-                    "op dots",
-                    "op hatch",
-                    "cutcode",
-                    "util console",
-                    "util wait",
-                    "util home",
-                    "util goto",
-                    "util origin",
-                    "util input",
-                    "util output",
-                    "lasercode",
-                    "blob",
-                ),
-                depth=1,
-            ):
+            # types = (
+            #     "op cut",
+            #     "op raster",
+            #     "op image",
+            #     "op engrave",
+            #     "op dots",
+            #     "cutcode",
+            #     "util console",
+            #     "util wait",
+            #     "util home",
+            #     "util goto",
+            #     "util input",
+            #     "util output",
+            #     "place point"
+            #     "blob",
+            # )
+            for c in operations:
+                isactive = True
                 try:
                     if not c.output:
-                        continue
+                        isactive = False
                 except AttributeError:
                     pass
-                try:
-                    if c.type.startswith("op") and len(c.children) == 0:
-                        continue
-                except TypeError:
-                    pass
+                if not isactive and copy_selected and len(operations) == 1:
+                    # If it's the only one we make an exception
+                    isactive = True
+                if not isactive:
+                    continue
+                if not hasattr(c, "type") or c.type is None:
+                    # Node must be a type of node.
+                    continue
+                if c.type.startswith("op ") and len(c.children) == 0:
+                    # We don't need empty operations.
+                    continue
                 if c.type == "cutcode":
                     # CutNodes are denuded into normal objects.
                     c = c.cutcode
+
+                # Make copy of node.
                 copy_c = copy(c)
                 try:
+                    # Make copy of real (non-referenced) children if that is permitted.
                     copy_c.copy_children_as_real(c)
                 except AttributeError:
                     pass
                 data.plan.append(copy_c)
+
             # Add default trailing ops
             add_ops(False)
             channel(_("Copied Operations."))
@@ -694,6 +717,12 @@ class Planner(Service):
             output_type="plan",
         )
         def plan_preprocess(command, channel, _, data_type=None, data=None, **kwgs):
+            # Update Info-panel if displayed
+            busy = self.kernel.busyinfo
+            if busy.shown:
+                busy.change(msg=_("Preprocessing"), keep=1)
+                busy.show()
+
             data.preprocess()
             self.signal("plan", data.name, 2)
             return data_type, data
@@ -705,6 +734,12 @@ class Planner(Service):
             output_type="plan",
         )
         def plan_validate(command, channel, _, data_type=None, data=None, **kwgs):
+            # Update Info-panel if displayed
+            busy = self.kernel.busyinfo
+            if busy.shown:
+                busy.change(msg=_("Validating"), keep=1)
+                busy.show()
+
             try:
                 data.execute()
             except CutPlanningFailedError as e:
@@ -714,6 +749,23 @@ class Planner(Service):
             self.signal("plan", data.name, 3)
             return data_type, data
 
+        @self.console_command(
+            "geometry",
+            help=_("plan<?> geometry"),
+            input_type="plan",
+            output_type="plan",
+        )
+        def plan_geometry(data_type=None, data=None, **kwgs):
+            # Update Info-panel if displayed
+            busy = self.kernel.busyinfo
+            if busy.shown:
+                busy.change(msg=_("Converting data"), keep=1)
+                busy.show()
+
+            data.geometry()
+            self.signal("plan", data.name, 4)
+            return data_type, data
+
         @self.console_command(
             "blob",
             help=_("plan<?> blob"),
@@ -721,6 +773,12 @@ class Planner(Service):
             output_type="plan",
         )
         def plan_blob(data_type=None, data=None, **kwgs):
+            # Update Info-panel if displayed
+            busy = self.kernel.busyinfo
+            if busy.shown:
+                busy.change(msg=_("Converting data"), keep=1)
+                busy.show()
+
             data.blob()
             self.signal("plan", data.name, 4)
             return data_type, data
@@ -732,6 +790,12 @@ class Planner(Service):
             output_type="plan",
         )
         def plan_preopt(data_type=None, data=None, **kwgs):
+            # Update Info-panel if displayed
+            busy = self.kernel.busyinfo
+            if busy.shown:
+                busy.change(msg=_("Preparing optimisation"), keep=1)
+                busy.show()
+
             data.preopt()
             self.signal("plan", data.name, 5)
             return data_type, data
@@ -743,6 +807,12 @@ class Planner(Service):
             output_type="plan",
         )
         def plan_optimize(data_type=None, data=None, **kwgs):
+            # Update Info-panel if displayed
+            busy = self.kernel.busyinfo
+            if busy.shown:
+                busy.change(msg=_("Optimising"), keep=1)
+                busy.show()
+
             data.execute()
             self.signal("plan", data.name, 6)
             return data_type, data
@@ -754,6 +824,12 @@ class Planner(Service):
             output_type="plan",
         )
         def plan_clear(data_type=None, data=None, **kwgs):
+            # Update Info-panel if displayed
+            busy = self.kernel.busyinfo
+            if busy.shown:
+                busy.change(msg=_("Clearing data"), keep=1)
+                busy.show()
+
             data.clear()
             self.signal("plan", data.name, 0)
             return data_type, data
@@ -770,7 +846,7 @@ class Planner(Service):
 
             for c in data.plan:
                 if isinstance(c, CutCode):
-                    operations.add(c, type="cutcode")
+                    operations.add(type="cutcode", cutcode=c)
                 if isinstance(
                     c,
                     (
@@ -779,7 +855,6 @@ class Planner(Service):
                         CutOpNode,
                         EngraveOpNode,
                         DotsOpNode,
-                        HatchOpNode,
                     ),
                 ):
                     copy_c = copy(c)
@@ -852,5 +927,4 @@ class Planner(Service):
             return data_type, data
 
     def plan(self, **kwargs):
-        for item in self._plan:
-            yield item
+        yield from self._plan
