@@ -1,3 +1,5 @@
+import random
+
 from meerk40t.tools.zinglplotter import ZinglPlotter
 
 from ..device.basedevice import (
@@ -14,8 +16,7 @@ from ..device.basedevice import (
 from .parameters import Parameters
 
 """
-
-The PlotPlanner simplifies the plotting and pulsing  modifications routines. These are buffered with plottable elements.
+The PlotPlanner simplifies the plotting and pulsing modifications routines. These are buffered with plottable elements.
 These can be submitted as destination graphics commands, or by submitting a plot routine. Which may yield either 2 or
 3 value coordinates. These are x, y, and on. Where on is a number between 0 and 1 which designates the on-value. In the
 graphics commands the move is given a 0 and all other plots are given a 1. All graphics commands take an optional
@@ -43,10 +44,10 @@ class PlotPlanner(Parameters):
         self,
         settings,
         single=True,
-        smooth=True,
         ppi=True,
         shift=True,
         group=True,
+        require_uniform_movement=True,
         **kwargs,
     ):
         super().__init__(settings, **kwargs)
@@ -55,31 +56,34 @@ class PlotPlanner(Parameters):
         self.abort = False
         self.force_shift = False
         self.group_enabled = True  # Grouped Output Required for Lhymicro-gl.
-        self.smooth_limit = 15
+        self.phase_type = 0  # Sequential, random, progressive, static.
+        self.phase_value = 0
+        self.require_uniform_movement = require_uniform_movement
 
         self.queue = []
 
         self.single = None
-        self.smooth = None
         self.ppi = None
         self.shift = None
         self.group = None
 
         if single:
             self.single = Single(self)
-        if smooth:
-            self.smooth = Smooth(self)
-        if ppi:
-            self.ppi = PPI(self)
         if shift:
             self.shift = Shift(self)
         if group:
             self.group = Group(self)
+        self.set_ppi(ppi)
 
         self.pos_x = None
         self.pos_y = None
         self.settings_then_jog = False
 
+    def set_ppi(self, ppi):
+        self.ppi = None
+        if ppi:
+            self.ppi = PPI(self)
+
     def push(self, plot):
         self.abort = False
         self.queue.append(plot)
@@ -131,7 +135,11 @@ class PlotPlanner(Parameters):
             if self.pos_x != new_start_x or self.pos_y != new_start_y:
                 # This location is disjointed. We must flush and jog.
                 # Jog is executed in current settings.
-                if self.pos_x is None or self.raster_step_x != 0:
+                if (
+                    self.pos_x is None
+                    or self.raster_step_x != 0
+                    or self.raster_step_y != 0
+                ):
                     # First movement or raster_step exists we must rapid_jog.
                     # Request rapid move new location
                     flush = True
@@ -184,7 +192,7 @@ class PlotPlanner(Parameters):
                     self.settings = cut.settings
                     yield None, None, PLOT_SETTING
 
-            if jog or self.raster_step_x != 0:
+            if jog or self.raster_step_x != 0 or self.raster_step_y != 0:
                 # set the directions. Post Jog, Post Settings.
                 yield cut.major_axis(), None, PLOT_AXIS
                 yield cut.x_dir(), cut.y_dir(), PLOT_DIRECTION
@@ -230,10 +238,6 @@ class PlotPlanner(Parameters):
             plot = self.single.process(plot)
         if self.debug:
             plot = debug(plot, self.single)
-        if self.smooth is not None:
-            plot = self.smooth.process(plot)
-        if self.debug:
-            plot = debug(plot, self.smooth)
         if self.ppi is not None:
             plot = self.ppi.process(plot)
         if self.debug:
@@ -267,8 +271,6 @@ class PlotPlanner(Parameters):
         self.pos_y = y
         if self.single:
             self.single.warp(x, y)
-        if self.smooth:
-            self.smooth.warp(x, y)
         if self.ppi:
             self.ppi.warp(x, y)
         if self.shift:
@@ -279,8 +281,6 @@ class PlotPlanner(Parameters):
     def reset(self):
         if self.single:
             self.single.clear()
-        if self.smooth:
-            self.smooth.clear()
         if self.shift:
             self.shift.clear()
         if self.ppi:
@@ -312,6 +312,7 @@ class PlotManipulation:
 class Single(PlotManipulation):
     def __init__(self, planner: PlotPlanner):
         super().__init__(planner)
+        self.planner = planner
         self.single_default = 1
         self.single_x = None
         self.single_y = None
@@ -352,7 +353,7 @@ class Single(PlotManipulation):
             dx = 1 if total_dx > 0 else 0 if total_dx == 0 else -1
             dy = 1 if total_dy > 0 else 0 if total_dy == 0 else -1
 
-            if total_dy * dx != total_dx * dy:
+            if total_dy * dx != total_dx * dy and self.planner.require_uniform_movement:
                 # Check for cross-equality.
                 raise ValueError(
                     f"Must be uniformly diagonal or orthogonal: ({total_dx}, {total_dy}) is not."
@@ -380,107 +381,12 @@ class Single(PlotManipulation):
         self.single_y = None
 
 
-class Smooth(PlotManipulation):
-    def __init__(self, planner: PlotPlanner):
-        super().__init__(planner)
-        self.goal_x = None
-        self.goal_y = None
-        self.goal_on = None
-
-        self.smooth_x = None
-        self.smooth_y = None
-
-    def __str__(self):
-        return f"{self.__class__.__name__}({str(self.smooth_x)},{str(self.smooth_y)})"
-
-    def flushed(self):
-        return (
-            self.goal_x == self.smooth_x
-            and self.goal_y == self.smooth_y
-            and self.goal_x is not None
-            and self.goal_y is not None
-        )
-
-    def process(self, plot):
-        """
-        @param plot: Smooth attempts to smooth out values
-        @return:
-        """
-        px = None
-        py = None
-        for x, y, on in plot:
-            if x is None or y is None:
-                # flush the process when if sent a None value.
-                yield from self.flush()
-                yield x, y, on
-                continue
-            if not self.planner.settings.get(
-                "_constant_move_x", False
-            ) and not self.planner.settings.get("_constant_move_y", False):
-                yield x, y, on
-                continue  # We are not smoothing.
-            if px is not None and py is not None:
-                # Ensure we are single stepped values.
-                assert abs(px - x) <= 1 or abs(py - y) <= 1
-            px = x
-            py = y
-            if self.smooth_x is None:
-                self.smooth_x = x
-            if self.smooth_y is None:
-                self.smooth_y = y
-            total_dx = x - self.smooth_x
-            total_dy = y - self.smooth_y
-            self.goal_x = x
-            self.goal_y = y
-            self.goal_on = on
-            if total_dx == 0 and total_dy == 0:
-                yield x, y, on
-                continue
-            dx = 1 if total_dx > 0 else 0 if total_dx == 0 else -1
-            dy = 1 if total_dy > 0 else 0 if total_dy == 0 else -1
-            if self.planner.settings.get("_constant_move_x", False) and dx == 0:
-                # If we are moving x and, we don't move x: skip.
-                if abs(total_dy) < self.planner.smooth_limit:
-                    continue
-            if self.planner.settings.get("_constant_move_y", False) and dy == 0:
-                if abs(total_dx) < self.planner.smooth_limit:
-                    continue
-            self.smooth_x += dx
-            self.smooth_y += dy
-            yield self.smooth_x, self.smooth_y, on
-
-    def flush(self):
-        if not self.flushed():
-            if self.goal_x is None or self.goal_y is None:
-                return
-            if self.smooth_x is None or self.smooth_y is None:
-                return
-            for x, y in ZinglPlotter.plot_line(
-                self.smooth_x, self.smooth_y, self.goal_x, self.goal_y
-            ):
-                yield x, y, self.goal_on
-            self.smooth_x = None
-            self.smooth_y = None
-            self.goal_x = None
-            self.goal_y = None
-            self.goal_on = None
-
-    def warp(self, x, y):
-        self.smooth_x = x
-        self.smooth_y = y
-        self.goal_x = x
-        self.goal_y = y
-
-    def clear(self):
-        self.smooth_x = None
-        self.smooth_y = None
-
-
 class PPI(PlotManipulation):
     def __init__(self, planner: PlotPlanner):
         super().__init__(planner)
         self.ppi_total = 0
         self.dot_left = 0
+        self._phase_set = 0
 
     def __str__(self):
         return f"{self.__class__.__name__}({str(self.ppi_total)},{str(self.dot_left)})"
@@ -499,6 +405,19 @@ class PPI(PlotManipulation):
         for x, y, on in plot:
             if x is None or y is None:
                 yield x, y, on
+                # Sequential, random, progressive, static.
+                if self.planner.phase_type == 1:
+                    # Random.
+                    self.ppi_total = random.randint(0, 1000)
+                elif self.planner.phase_type == 2:
+                    # progressive
+                    self._phase_set += self.planner.phase_value
+                    self._phase_set %= 1000
+                    self.ppi_total = self._phase_set
+                elif self.planner.phase_type == 3:
+                    # static
+                    self._phase_set = self.planner.phase_value
+                    self.ppi_total = self._phase_set
                 continue
             if px is not None and py is not None:
                 assert abs(px - x) <= 1 or abs(py - y) <= 1
@@ -515,8 +434,8 @@ class PPI(PlotManipulation):
                 if self.ppi_total >= 1000.0:
                     # PPI >= 1000: triggers on.
                     on = 1
-                    self.ppi_total -= 1000.0 * self.planner.dot_length
-                    self.dot_left = self.planner.dot_length - 1
+                    self.ppi_total -= 1000.0 * self.planner.implicit_dotlength
+                    self.dot_left = self.planner.implicit_dotlength - 1
                 else:
                     # PPI < 1000: triggers off.
                     on = 0
