@@ -1,11 +1,19 @@
 import time
 from math import isinf
-from threading import Lock
+from threading import Condition
 
-from meerk40t.core.cutcode import CutCode
+from meerk40t.core.laserjob import LaserJob
 from meerk40t.core.units import Length
 from meerk40t.kernel import CommandSyntaxError
 
+"""
+This module defines a set of commands that usually send a single easy command to the spooler. Basic jogging, home,
+unlock rail commands. And it provides the the spooler class which should be provided by each driver.
+
+Spoolers process different jobs in order. A spooler job can be anything, but usually is a LaserJob which is a simple
+list of commands.
+"""
+
 
 def plugin(kernel, lifecycle):
     if lifecycle == "register":
@@ -22,12 +30,15 @@ def plugin(kernel, lifecycle):
             device = kernel.device
 
             spooler = device.spooler
+            # Do we have a filename to use as label?
+            label = kernel.elements.basename
 
             if data is not None:
                 # If plan data is in data, then we copy that and move on to next step.
                 data.final()
                 loops = 1
                 elements = kernel.elements
+                elements("wordlist advance\n")
                 e = elements.op_branch
 
                 if e.loop_continuous:
@@ -35,7 +46,9 @@ def plugin(kernel, lifecycle):
                 else:
                     if e.loop_enabled:
                         loops = e.loop_n
-                spooler.laserjob(data.plan, loops=loops)
+                spooler.laserjob(
+                    data.plan, loops=loops, label=label, outline=data.outline
+                )
                 channel(_("Spooled Plan."))
                 kernel.root.signal("plan", data.name, 6)
 
@@ -66,7 +79,8 @@ def plugin(kernel, lifecycle):
                 raise CommandSyntaxError
             try:
                 for plan_command, command_name, suffix in kernel.find("plan", op):
-                    spooler.laserjob(plan_command)
+                    label = f"Send {op}"
+                    spooler.laserjob(plan_command, label=label)
                     return data_type, spooler
             except (KeyError, IndexError):
                 pass
@@ -194,30 +208,6 @@ def plugin(kernel, lifecycle):
                     channel(_("Busy Error"))
             return "spooler", spooler
 
-        @kernel.console_option("force", "f", type=bool, action="store_true")
-        @kernel.console_argument("x", type=Length, help=_("change in x"))
-        @kernel.console_argument("y", type=Length, help=_("change in y"))
-        @kernel.console_command(
-            "move_origin",
-            input_type=("spooler", None),
-            output_type="spooler",
-            help=_("move <x> <y>: move to position."),
-        )
-        def move_origin(channel, _, x, y, data=None, force=False, **kwgs):
-            if data is None:
-                data = kernel.device.spooler
-            spooler = data
-            if y is None:
-                raise CommandSyntaxError
-            if force:
-                spooler.command("move_ori", x, y)
-            else:
-                if spooler.is_idle:
-                    spooler.command("move_ori", x, y)
-                else:
-                    channel(_("Busy Error"))
-            return "spooler", spooler
-
         @kernel.console_option("force", "f", type=bool, action="store_true")
         @kernel.console_argument("x", type=Length, help=_("change in x"))
         @kernel.console_argument("y", type=Length, help=_("change in y"))
@@ -266,25 +256,6 @@ def plugin(kernel, lifecycle):
                     channel(_("Busy Error"))
             return "spooler", spooler
 
-        @kernel.console_argument("x", type=Length, help=_("change in x"))
-        @kernel.console_argument("y", type=Length, help=_("change in y"))
-        @kernel.console_command(
-            "set_origin",
-            input_type=("spooler", None),
-            output_type="spooler",
-            help=_("set_origin <x> <y>: set origin to position"),
-        )
-        def set_origin(channel, _, x, y, data=None, **kwgs):
-            if data is None:
-                data = kernel.device.spooler
-            spooler = data
-            if y is None:
-                spooler.command("set_origin", None, None)
-            else:
-                x, y = kernel.device.physical_to_device_position(x, y)
-                spooler.command("set_origin", x, y)
-            return "spooler", spooler
-
         @kernel.console_command(
             "home",
             input_type=("spooler", None),
@@ -298,6 +269,19 @@ def plugin(kernel, lifecycle):
             spooler.command("home")
             return "spooler", spooler
 
+        @kernel.console_command(
+            "physical_home",
+            input_type=("spooler", None),
+            output_type="spooler",
+            help=_("home the laser (goto endstops)"),
+        )
+        def physical_home(data=None, **kwgs):
+            if data is None:
+                data = kernel.device.spooler
+            spooler = data
+            spooler.command("physical_home")
+            return "spooler", spooler
+
         @kernel.console_command(
             "unlock",
             input_type=("spooler", None),
@@ -349,259 +333,97 @@ def plugin(kernel, lifecycle):
                 yield "home"
                 yield "wait_finish"
 
-            spooler.laserjob(list(home_dot_test()), label=f"Dot and Home Test", helper=True)
+            spooler.laserjob(
+                list(home_dot_test()), label="Dot and Home Test", helper=True
+            )
             return "spooler", spooler
 
 
-class LaserJob:
-    def __init__(self, label, items, driver=None, priority=0, loops=1):
-        self.items = items
-        self.label = label
-        self.priority = priority
-        self.time_submitted = time.time()
-        self.time_started = None
+class SpoolerJob:
+    """
+    Example of Spooler Job.
+
+    The primary methodology of a spoolerjob is that it has an `execute` function that takes the driver as an argument
+    it should then perform driver-like commands on the given driver to perform whatever actions the job should
+    execute.
+
+    The `priority` attribute is required.
+
+    The job should be permitted to `stop()` and respond to `is_running()`, and other checks as to elapsed_time(),
+    estimate_time(), and status.
+    """
+
+    def __init__(
+        self,
+        service,
+    ):
+        self.stopped = False
+        self.service = service
         self.runtime = 0
-        self.time_pass_started = None
-        self.steps_done = 0
-        self.steps_total = 0
-        self.avg_time_per_pass = None
-        self.loops = loops
-        self.loops_executed = 0
-        self._driver = driver
-        self.helper = False
-        self.item_index = 0
-
-        self._stopped = True
-
-        self._estimate = 0
-
-        for item in self.items:
-            if isinstance(item, CutCode):
-                time_travel = item.duration_travel()
-                time_cuts = item.duration_cut()
-                time_extra = item.extra_time()
-                self._estimate += time_travel + time_cuts + time_extra
-
-    def __str__(self):
-        return f"{self.__class__.__name__}({self.label}: {self.loops_executed}/{self.loops})"
+        self.priority = 0
 
     @property
     def status(self):
-        if self.is_running and self.time_started is not None:
+        """
+        Status is simply a status as to the job. This will be relayed by things that check the job status of jobs
+        in the spooler.
+
+        @return:
+        """
+        if self.is_running():
             return "Running"
-        elif not self.is_running:
-            return "Disabled"
         else:
             return "Queued"
 
-    def is_running(self):
-        return not self._stopped
-
-    def execute(self, driver=None):
+    def execute(self, driver):
         """
-        Execute calls each item in the list of items in order. This is intended to be called by the spooler thread. And
-        hold the spooler while these items are executing.
+        This is the primary method of the SpoolerJob. In this example we call the "home()" function.
+        @param driver:
         @return:
         """
-        self._stopped = False
-        self.time_started = time.time()
-        self.time_pass_started = time.time()
-        self.steps_total = 0
-        self.calc_steps()
         try:
-            while self.loops_executed < self.loops:
-                self.steps_done = 0
-                if self._stopped:
-                    return False
-                while self.item_index < len(self.items):
-                    if self._stopped:
-                        return False
-                    item = self.items[self.item_index]
-                    self.execute_item(item)
-                    if self._stopped:
-                        return False
-                    self.item_index += 1
-                self.item_index = 0
-                self.loops_executed += 1
-                self.time_pass_started = time.time()
-                self.avg_time_per_pass = self.elapsed_time() / self.loops_executed
-        finally:
-            self.runtime += time.time() - self.time_started
-            self._stopped = True
+            driver.home()
+        except AttributeError:
+            pass
+
         return True
 
-    def calc_steps(self):
-        def simple_step(item):
-            if isinstance(item, tuple):
-                attr = item[0]
-                if hasattr(self._driver, attr):
-                    self.steps_total += 1
-            # STRING
-            elif isinstance(item, str):
-                attr = item
-                if hasattr(self._driver, attr):
-                    self.steps_total += 1
-            # .generator is a Generator
-            elif hasattr(item, "generate"):
-                item = getattr(item, "generate")
-                for p in item():
-                    simple_step(p)
-
-        self.steps_total = 0
-        for pitem in self.items:
-            simple_step(pitem)
-
-    def execute_item(self, item):
-        """
-        This executes the different classes of spoolable object.
-
-        * (str, attribute, ...) calls self.driver.str(*attributes)
-        * str, calls self.driver.str()
-        * has_attribute(generator), recursive call to list of lines produced by the
-        generator, recursive call to list of lines produced by generator
-
-        @param item:
-        @return:
-        """
-        if isinstance(item, tuple):
-            attr = item[0]
-            if hasattr(self._driver, attr):
-                function = getattr(self._driver, attr)
-                function(*item[1:])
-                self.steps_done += 1
-            return
-
-        # STRING
-        if isinstance(item, str):
-            attr = item
-
-            if hasattr(self._driver, attr):
-                function = getattr(self._driver, attr)
-                function()
-                self.steps_done += 1
-            return
-
-        # .generator is a Generator
-        if hasattr(item, "generate"):
-            item = getattr(item, "generate")
-
-        # Generator item
-        for p in item():
-            if self._stopped:
-                return
-            self.execute_item(p)
-
     def stop(self):
-        """
-        Stop this current laser-job, cannot be called from the spooler thread.
-        @return:
-        """
-        if not self._stopped:
-            self.runtime += time.time() - self.time_started
-        self._stopped = True
+        self.stopped = True
+
+    def is_running(self):
+        return not self.stopped
 
     def elapsed_time(self):
         """
         How long is this job already running...
         """
         result = 0
-        if self.is_running():
-            result = time.time() - self.time_started
-        else:
-            result = self.runtime
         return result
 
     def estimate_time(self):
-        """
-        Give laser job time estimate.
-        @return:
-        """
-        # This is rather 'simple', we have no clue what exactly this job is doing,
-        # but we have some ideas, if and only if the job is_running:
-        # a) we know the elapsed time
-        # b) we know current and total steps (if the driver has such a property)
-        if isinf(self.loops):
-            return float("inf")
-        result = 0
-        time_for_past_passes = 0
-        time_for_future_passes = self.loops * self._estimate
-        if self.is_running and self.time_started is not None:
-            # We fall back on elapsed and some info from the driver...
-            elapsed = time.time() - self.time_started
-            ratio = 1
-            # As we have mainly disabled the driver preview, we do something simpler:
-            # We know the pass of passes and we know the steps of total steps...
-            if self.avg_time_per_pass is None:
-                time_for_past_passes = 0
-                time_for_future_passes = (
-                    max(self.loops - self.loops_executed - 1, 0) * self._estimate
-                )
-            else:
-                time_for_past_passes = self.time_pass_started - self.time_started
-                time_for_future_passes = self.avg_time_per_pass * max(
-                    self.loops - self.loops_executed - 1, 0
-                )
-
-            if self.time_pass_started is not None:
-                this_pass_seconds = time.time() - self.time_pass_started
-                if this_pass_seconds >= 5:
-                    result = max(
-                        self._estimate,
-                        this_pass_seconds / max(self.steps_done, 1) * self.steps_total,
-                    )
-                else:
-                    result = self._estimate
-            # print (f"Passes: {self.loops_executed} / {self.loops}")
-            # print (f"Past: {time_for_past_passes:.1f}, Current: {result:.1f}, Future: {time_for_future_passes:.1f}")
-            # print (f"Steps: {self.steps_done} / {self.steps_total}, Pass-Estimate: {self._estimate:.1f}")
-            # if hasattr(self._driver, "total_steps"):
-            #     total = self._driver.total_steps
-            #     current = self._driver.current_steps
-            #     # Safety belt, as we have disabled the logic partially
-            #     if total < current:
-            #         total = current + 1
-            #     if current > 10 and total > 0:
-            #         # Arbitrary minimum steps (if too low, value is erratic)
-            #         ratio = total / current
-            # result = elapsed * ratio
-        if result == 0:
-            # Nothing useful came out, so we fall back on the initial value
-            result = self._estimate
-        result += time_for_past_passes + time_for_future_passes
-
-        return result
+        return 0
 
 
 class Spooler:
     """
-    Spoolers store spoolable events in a two synchronous queue, and a single idle job that
-    will be executed in a loop, if the synchronous queues are empty. The two queues are the
-    realtime and the regular queue.
-
-    Spooler should be registered as a service_delegate of the device service running the driver
-    to process data.
-
-    Spoolers have threads that process and run each set of commands. Ultimately all commands are
-    executed against the given driver. So if the command within the spooled element is "unicorn"
-    then driver.unicorn() is called with the given arguments. This permits arbitrary execution of
-    specifically spooled elements in the correct sequence.
-
-    The two queues are the realtime and the regular queue. The realtime queue tries to execute
-    particular events as soon as possible. And will execute even if there is a hold on the current
-    work.
-
-    When the queues are empty the idle job is repeatedly executed in a loop. If there is no idle job
-    then the spooler is inactive.
+    Spoolers are threaded job processors. A series of jobs is added to the spooler and these jobs are
+    processed in order. The driver is checked for any holds it may have preventing new commands from being
+    executed. If that isn't the case, the highest priority job is executed by calling the job's required
+    `execute()` function passing the relevant driver as the one variable. The job itself is agnostic, and will
+    execute whatever it wants calling the driver-like functions that may or may not exist on the driver.
 
+    If execute() returns true then it is fully executed and will be removed. Otherwise, it will be repeatedly
+    called until whatever work it is doing is finished. This also means the driver itself is checked for holds
+    (usually pausing or busy) each cycle.
     """
 
     def __init__(self, context, driver=None, **kwargs):
         self.context = context
         self.driver = driver
-        self.foreground_only = True
         self._current = None
 
-        self._lock = Lock()
+        self._lock = Condition()
         self._queue = []
 
         self._shutdown = False
@@ -628,26 +450,6 @@ class Spooler:
         """
         self.restart()
 
-    def service_attach(self, *args, **kwargs):
-        """
-        device service is attached to the kernel.
-
-        @param args:
-        @param kwargs:
-        @return:
-        """
-        if self.foreground_only:
-            self.restart()
-
-    def service_detach(self):
-        """
-        device service is detached from the kernel.
-
-        @return:
-        """
-        if self.foreground_only:
-            self.shutdown()
-
     def shutdown(self, *args, **kwargs):
         """
         device service is shutdown during the shutdown of the kernel or destruction of the service
@@ -657,6 +459,8 @@ class Spooler:
         @return:
         """
         self._shutdown = True
+        with self._lock:
+            self._lock.notify_all()
 
     def restart(self):
         """
@@ -670,6 +474,13 @@ class Spooler:
             def clear_thread(*a):
                 self._shutdown = True
                 self._thread = None
+                try:
+                    # If something is currently processing stop it.
+                    self._current.stop()
+                except AttributeError:
+                    pass
+                with self._lock:
+                    self._lock.notify_all()
 
             self._thread = self.context.threaded(
                 self.run,
@@ -682,26 +493,40 @@ class Spooler:
         """
         Run thread for the spooler.
 
-        Process the real time queue.
-        Hold work queue if driver requires a hold
-        Process work queue.
-        Hold idle if driver requires idle to be held.
-        Process idle work
+        The thread runs while the spooler is not shutdown. This executes in the spooler thread. It waits, while
+        the queue is empty and is notified when items are added to the queue. Each job in the spooler is called
+        with execute(). If the function returns True, the job is finished and removed. We then move on to the next
+        spooler item. If execute() returns False the job is not finished and will be reattempted. Each spooler
+        cycle checks the priority and whether there's a wait/hold for jobs at that priority level.
 
         @return:
         """
         while not self._shutdown:
             if self.context.kernel.is_shutdown:
                 return  # Kernel shutdown spooler threads should die off.
-            self._lock.acquire()
-            try:
-                program = self._queue[0]
-            except IndexError:
-                self._lock.release()
-                # There is no work to do.
-                time.sleep(0.1)
+            is_valid = False
+            with self._lock:
+                idx = 0
+                while not is_valid:
+                    try:
+                        program = self._queue[idx]
+                        # Not all type of jobs are regular jobs,
+                        # so that needs to be checked...
+                        if hasattr(program, "enabled"):
+                            if program.enabled:
+                                is_valid = True
+                                break
+                            else:
+                                idx += 1
+                        else:
+                            is_valid = True
+                            break
+                    except IndexError:
+                        # There is no work to do.
+                        self._lock.wait()
+                        break
+            if not is_valid:
                 continue
-            self._lock.release()
 
             priority = program.priority
 
@@ -709,16 +534,33 @@ class Spooler:
             if self.driver.hold_work(priority):
                 time.sleep(0.01)
                 continue
+            if program != self._current:
+                # A different job is loaded. If it has a job_start, we call that.
+                if hasattr(self.driver, "job_start"):
+                    function = getattr(self.driver, "job_start")
+                    function(program)
             self._current = program
             try:
                 fully_executed = program.execute(self.driver)
             except ConnectionAbortedError:
                 # Driver could no longer connect to where it was told to send the data.
                 return
+            except ConnectionRefusedError:
+                # Driver connection failed but, we are not aborting the spooler thread
+                if self._shutdown:
+                    return
+                with self._lock:
+                    self._lock.wait()
+                continue
             if fully_executed:
                 # all work finished
                 self.remove(program)
 
+                # If we finished this work we call job_finished.
+                if hasattr(self.driver, "job_finish"):
+                    function = getattr(self.driver, "job_finish")
+                    function(program)
+
     @property
     def is_idle(self):
         return len(self._queue) == 0 or self._queue[0].priority < 0
@@ -731,7 +573,9 @@ class Spooler:
     def queue(self):
         return self._queue
 
-    def laserjob(self, job, priority=0, loops=1, label=None, helper=False):
+    def laserjob(
+        self, job, priority=0, loops=1, label=None, helper=False, outline=None
+    ):
         """
         send a wrapped laser job to the spooler.
         """
@@ -739,35 +583,53 @@ class Spooler:
             label = f"{self.__class__.__name__}:{len(job)} items"
         # label = str(job)
         ljob = LaserJob(
-            label, list(job), driver=self.driver, priority=priority, loops=loops
+            label,
+            list(job),
+            driver=self.driver,
+            priority=priority,
+            loops=loops,
+            outline=outline,
         )
         ljob.helper = helper
+        ljob.uid = self.context.logging.uid("job")
         with self._lock:
             self._stop_lower_priority_running_jobs(priority)
             self._queue.append(ljob)
             self._queue.sort(key=lambda e: e.priority, reverse=True)
+            self._lock.notify()
         self.context.signal("spooler;queue", len(self._queue))
 
-    def command(self, *job, priority=0, helper=True):
-        laserjob = LaserJob(str(job), [job], driver=self.driver, priority=priority)
-        laserjob.helper = helper
+    def command(self, *job, priority=0, helper=True, outline=None):
+        ljob = LaserJob(
+            str(job), [job], driver=self.driver, priority=priority, outline=outline
+        )
+        ljob.helper = helper
+        ljob.uid = self.context.logging.uid("job")
         with self._lock:
             self._stop_lower_priority_running_jobs(priority)
-            self._queue.append(laserjob)
+            self._queue.append(ljob)
             self._queue.sort(key=lambda e: e.priority, reverse=True)
+            self._lock.notify()
         self.context.signal("spooler;queue", len(self._queue))
 
-    def send(self, job):
+    def send(self, job, prevent_duplicate=False):
         """
         Send a job to the spooler queue
 
         @param job: job to send to the spooler.
+        @param prevent_duplicate: prevents the same job from being added again.
         @return:
         """
+        job.uid = self.context.logging.uid("job")
         with self._lock:
+            if prevent_duplicate:
+                for q in self._queue:
+                    if q is job:
+                        return
             self._stop_lower_priority_running_jobs(job.priority)
             self._queue.append(job)
             self._queue.sort(key=lambda e: e.priority, reverse=True)
+            self._lock.notify()
         self.context.signal("spooler;queue", len(self._queue))
 
     def _stop_lower_priority_running_jobs(self, priority):
@@ -777,61 +639,68 @@ class Spooler:
 
     def clear_queue(self):
         with self._lock:
-            for e in self._queue:
-                try:
+            for element in self._queue:
+                loop = getattr(element, "loops_executed", 0)
+                total = getattr(element, "loops", 0)
+                if isinf(total):
+                    status = "stopped"
+                elif loop < total:
+                    status = "stopped"
+                else:
                     status = "completed"
-                    needs_signal = e.is_running() and e.time_started is not None
-                    loop = e.loops_executed
-                    total = e.loops
-                    if isinf(total):
-                        status = "stopped"
-                        total = "∞"
-                    elif loop < total:
-                        status = "stopped"
-                    passinfo = f"{loop}/{total}"
-                    e.stop()
-                    if needs_signal:
-                        info = (
-                            e.label,
-                            e.time_started,
-                            e.runtime,
-                            self.context.label,
-                            passinfo,
-                            status,
-                            e.helper,
-                        )
-                        self.context.signal("spooler;completed", info)
-                except AttributeError:
-                    pass
+                self.context.logging.event(
+                    {
+                        "uid": getattr(element, "uid"),
+                        "status": status,
+                        "loop": getattr(element, "loops_executed", None),
+                        "total": getattr(element, "loops", None),
+                        "label": getattr(element, "label", None),
+                        "start_time": getattr(element, "time_started", None),
+                        "duration": getattr(element, "runtime", None),
+                        "device": self.context.label,
+                        "important": not getattr(element, "helper", False),
+                        "estimate": element.estimate_time()
+                        if hasattr(element, "estimate_time")
+                        else None,
+                        "steps_done": getattr(element, "steps_done", None),
+                        "steps_total": getattr(element, "steps_total", None),
+                    }
+                )
+                self.context.signal("spooler;completed")
+                element.stop()
             self._queue.clear()
-            self.context.signal("spooler;queue", len(self._queue))
+            self._lock.notify()
+        self.context.signal("spooler;queue", len(self._queue))
 
     def remove(self, element):
         with self._lock:
-            try:
-                status = "completed"
-                loop = element.loops_executed
-                total = element.loops
-                if isinf(element.loops):
-                    status = "stopped"
-                    total = "∞"
-                elif loop < total:
-                    status = "stopped"
-                info = (
-                    element.label,
-                    element.time_started,
-                    element.runtime,
-                    self.context.label,
-                    f"{loop}/{total}",
-                    status,
-                    element.helper,
-                )
-                self.context.signal("spooler;completed", info)
-            except AttributeError:
-                pass
+            status = "completed"
+            if element.status == "Running":
+                element.stop()
+                status = "Stopped"
+            self.context.logging.event(
+                {
+                    "uid": getattr(element, "uid"),
+                    "status": status,
+                    "loop": getattr(element, "loops_executed", None),
+                    "total": getattr(element, "loops", None),
+                    "label": getattr(element, "label", None),
+                    "start_time": getattr(element, "time_started", None),
+                    "duration": getattr(element, "runtime", None),
+                    "device": self.context.label,
+                    "important": not getattr(element, "helper", False),
+                    "estimate": element.estimate_time()
+                    if hasattr(element, "estimate_time")
+                    else None,
+                    "steps_done": getattr(element, "steps_done", None),
+                    "steps_total": getattr(element, "steps_total", None),
+                }
+            )
+            self.context.signal("spooler;completed")
             element.stop()
             for i in range(len(self._queue) - 1, -1, -1):
                 e = self._queue[i]
                 if e is element:
                     del self._queue[i]
+            self._lock.notify()
         self.context.signal("spooler;queue", len(self._queue))
