@@ -1,7 +1,7 @@
 def plugin(kernel, lifecycle=None):
     if lifecycle == "invalidate":
         try:
-            import ezdxf
+            import ezdxf  # pylint: disable=unused-import
         except ImportError:
             print("DXF plugin could not load because ezdxf is not installed.")
             return True
@@ -29,6 +29,20 @@ def plugin(kernel, lifecycle=None):
                     "Fit (scale down if necessary) and center a DXF file within the bed"
                 ),
                 "page": "Input/Output",
+                # Hint for translation _("Input")
+                "section": "Input",
+            },
+            {
+                "attr": "dxf_try_unsupported",
+                "object": kernel.elements,
+                "default": True,
+                "type": bool,
+                "label": _("DXF: Try to read 3D-polylines"),
+                "tip": _(
+                    "Tries to use dxf 3D-polylines, disable it if the file contains meshes or other 3D data"
+                ),
+                "page": "Input/Output",
+                # Hint for translation _("Input")
                 "section": "Input",
             },
         ]
