@@ -1,9 +1,9 @@
-from math import atan, sqrt, tau
+from math import atan2, sqrt
 
 import numpy as np
 import wx
 
-from meerk40t.core.element_types import elem_nodes
+from meerk40t.core.elements.element_types import elem_nodes
 from meerk40t.core.node.node import Node
 from meerk40t.svgelements import (
     Arc,
@@ -19,9 +19,18 @@ from meerk40t.svgelements import (
 )
 
 from ..core.units import Length
-from ..gui.wxutils import TextCtrl
+from ..gui.wxutils import (
+    StaticBoxSizer,
+    TextCtrl,
+    dip_size,
+    wxButton,
+    wxCheckBox,
+    wxRadioBox,
+    wxStaticBitmap,
+    wxStaticText,
+)
 from ..kernel import signal_listener
-from .icons import STD_ICON_SIZE, icons8_arrange_50
+from .icons import STD_ICON_SIZE, get_default_icon_size, icons8_arrange
 from .mwindow import MWindow
 
 _ = wx.GetTranslation
@@ -32,19 +41,20 @@ class InfoPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0)
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
-        self.lbl_info_main = wx.StaticText(self, wx.ID_ANY, "")
-        self.lbl_info_default = wx.StaticText(self, wx.ID_ANY, "")
-        self.lbl_info_first = wx.StaticText(self, wx.ID_ANY, "")
-        self.lbl_info_last = wx.StaticText(self, wx.ID_ANY, "")
+        self.context.themes.set_window_colors(self)
+        self.lbl_info_main = wxStaticText(self, wx.ID_ANY, "")
+        self.lbl_info_default = wxStaticText(self, wx.ID_ANY, "")
+        self.lbl_info_first = wxStaticText(self, wx.ID_ANY, "")
+        self.lbl_info_last = wxStaticText(self, wx.ID_ANY, "")
         self.preview_size = 25
-        self.image_default = wx.StaticBitmap(
-            self, wx.ID_ANY, size=wx.Size(self.preview_size, self.preview_size)
+        self.image_default = wxStaticBitmap(
+            self, wx.ID_ANY, size=dip_size(self, self.preview_size, self.preview_size)
         )
-        self.image_first = wx.StaticBitmap(
-            self, wx.ID_ANY, size=wx.Size(self.preview_size, self.preview_size)
+        self.image_first = wxStaticBitmap(
+            self, wx.ID_ANY, size=dip_size(self, self.preview_size, self.preview_size)
         )
-        self.image_last = wx.StaticBitmap(
-            self, wx.ID_ANY, size=wx.Size(self.preview_size, self.preview_size)
+        self.image_last = wxStaticBitmap(
+            self, wx.ID_ANY, size=dip_size(self, self.preview_size, self.preview_size)
         )
         sizer_main = wx.BoxSizer(wx.VERTICAL)
 
@@ -66,7 +76,7 @@ class InfoPanel(wx.Panel):
         sizer_main.Add(sizer_last, 0, wx.EXPAND, 0)
         self.make_raster = None
         self.SetSizer(sizer_main)
-        self.Layout
+        self.Layout()
 
     def show_stuff(self, has_emph):
         def mklabel(label):
@@ -76,37 +86,48 @@ class InfoPanel(wx.Panel):
             return result
 
         def create_image_from_node(node, iconsize):
-            image = wx.NullBitmap
+            image_from_node = None
             c = None
             # Do we have a standard representation?
-            defaultcolor = Color("black")
-            data = None
-            if node.type.startswith("elem "):
-                if (
-                    hasattr(node, "stroke")
-                    and node.stroke is not None
-                    and node.stroke.argb is not None
-                ):
-                    c = node.stroke
+            if (
+                node.type.startswith("elem ")
+                and hasattr(node, "stroke")
+                and node.stroke is not None
+                and node.stroke.argb is not None
+            ):
+                c = node.stroke
             if node.type.startswith("elem ") and node.type != "elem point":
-                data = node
-                bounds = node.paint_bounds
+                try:
+                    image_from_node = self.make_raster(
+                        node,
+                        node.paint_bounds,
+                        width=iconsize,
+                        height=iconsize,
+                        bitmap=True,
+                        keep_ratio=True,
+                    )
+                except Exception:
+                    return None, None
             elif node.type in ("group", "file"):
                 data = list(node.flat(types=elem_nodes))
                 bounds = Node.union_bounds(data, attr="paint_bounds")
-            if data is not None:
-                image = self.make_raster(
-                    data,
-                    bounds,
-                    width=iconsize,
-                    height=iconsize,
-                    bitmap=True,
-                    keep_ratio=True,
-                )
 
+                try:
+                    image_from_node = self.make_raster(
+                        data,
+                        bounds,
+                        width=iconsize,
+                        height=iconsize,
+                        bitmap=True,
+                        keep_ratio=True,
+                    )
+                except Exception:
+                    return None, None
             if c is None:
-                c = defaultcolor
-            return c, image
+                c = Color("black")
+            if image_from_node is None:
+                image_from_node = wx.NullBitmap
+            return c, image_from_node
 
         if self.make_raster is None:
             self.make_raster = self.context.elements.lookup("render-op/make_raster")
@@ -114,7 +135,6 @@ class InfoPanel(wx.Panel):
         count = 0
         first_node = None
         last_node = None
-        msg = ""
         if has_emph:
             xdata = list(self.context.elements.elems_nodes(emphasized=True))
             data = []
@@ -128,6 +148,8 @@ class InfoPanel(wx.Panel):
             if count > 0:
                 node = data[0]
                 c, image = create_image_from_node(node, self.preview_size)
+                if c is None:
+                    return
                 self.image_default.SetBitmap(image)
                 self.lbl_info_default.SetLabel(
                     _("As in Selection: {type} {lbl}").format(
@@ -140,6 +162,8 @@ class InfoPanel(wx.Panel):
                 node = data[0]
                 first_node = node
                 c, image = create_image_from_node(node, self.preview_size)
+                if c is None:
+                    return
                 self.image_first.SetBitmap(image)
                 self.lbl_info_first.SetLabel(
                     _("First selected: {type} {lbl}").format(
@@ -151,6 +175,9 @@ class InfoPanel(wx.Panel):
                 node = data[-1]
                 last_node = node
                 c, image = create_image_from_node(node, self.preview_size)
+                if c is None:
+                    return
+
                 self.image_last.SetBitmap(image)
                 self.lbl_info_last.SetLabel(
                     _("Last selected: {type} {lbl}").format(
@@ -174,6 +201,8 @@ class AlignmentPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0)
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
+        self.SetHelpText("alignment")
         self.scene = scene
         # Amount of currently selected
         self.count = 0
@@ -192,7 +221,7 @@ class AlignmentPanel(wx.Panel):
         self.modeparam = ("default", "first", "last", "bed", "ref")
         self.xyparam = ("none", "min", "center", "max")
 
-        self.rbox_align_x = wx.RadioBox(
+        self.rbox_align_x = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Alignment relative to X-Axis:"),
@@ -207,7 +236,7 @@ class AlignmentPanel(wx.Panel):
             )
         )
 
-        self.rbox_align_y = wx.RadioBox(
+        self.rbox_align_y = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Alignment relative to Y-Axis:"),
@@ -222,7 +251,7 @@ class AlignmentPanel(wx.Panel):
             )
         )
 
-        self.rbox_relation = wx.RadioBox(
+        self.rbox_relation = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Relative to:"),
@@ -232,7 +261,7 @@ class AlignmentPanel(wx.Panel):
         )
         self.rbox_relation.SetSelection(0)
 
-        self.rbox_treatment = wx.RadioBox(
+        self.rbox_treatment = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Treatment:"),
@@ -241,8 +270,10 @@ class AlignmentPanel(wx.Panel):
             style=wx.RA_SPECIFY_COLS,
         )
         self.rbox_treatment.SetSelection(0)
-        self.btn_align = wx.Button(self, wx.ID_ANY, "Align")
-        self.btn_align.SetBitmap(icons8_arrange_50.GetBitmap(resize=25))
+        self.btn_align = wxButton(self, wx.ID_ANY, "Align")
+        self.btn_align.SetBitmap(
+            icons8_arrange.GetBitmap(resize=0.5 * get_default_icon_size(self.context))
+        )
 
         sizer_main.Add(self.rbox_align_x, 0, wx.EXPAND, 0)
         sizer_main.Add(self.rbox_align_y, 0, wx.EXPAND, 0)
@@ -302,7 +333,7 @@ class AlignmentPanel(wx.Panel):
                 active = False
             if (
                 self.scene is None
-                or self.scene.reference_object is None
+                or self.scene.pane.reference_object is None
                 and mode == "ref"
             ):
                 active = False
@@ -322,11 +353,6 @@ class AlignmentPanel(wx.Panel):
             idx = 0
         ypos = self.xyparam[idx]
 
-        idx = self.rbox_align_y.GetSelection()
-        if idx < 0:
-            idx = 0
-        mode = self.xyparam[idx]
-
         idx = self.rbox_relation.GetSelection()
         if idx < 0:
             idx = 0
@@ -335,7 +361,7 @@ class AlignmentPanel(wx.Panel):
         addition = ""
         if mode == "ref":
             if self.scene is not None:
-                node = self.scene.reference_object
+                node = self.scene.pane.reference_object
                 if node is not None:
                     addition = f" --boundaries {node.bounds[0]},{node.bounds[1]},{node.bounds[2]},{node.bounds[3]}"
                 else:
@@ -370,7 +396,7 @@ class AlignmentPanel(wx.Panel):
         self.count, self.first_node, self.last_node = self.info_panel.show_stuff(
             has_emph
         )
-        flag = self.scene.reference_object is not None
+        flag = self.scene.pane.reference_object is not None
         self.rbox_relation.EnableItem(4, flag)
         self.validate_data()
 
@@ -380,6 +406,8 @@ class DistributionPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0)
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
+        self.SetHelpText("distribute")
         self.scene = scene
         # Amount of currently selected
         self.count = 0
@@ -405,7 +433,7 @@ class DistributionPanel(wx.Panel):
         self.xy_param = ("none", "min", "center", "max", "space")
         self.treat_param = ("default", "shape", "points", "bed", "ref")
 
-        self.rbox_dist_x = wx.RadioBox(
+        self.rbox_dist_x = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Position of element relative to point for X-Axis:"),
@@ -420,7 +448,7 @@ class DistributionPanel(wx.Panel):
             )
         )
 
-        self.rbox_dist_y = wx.RadioBox(
+        self.rbox_dist_y = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Position of element relative to point for Y-Axis:"),
@@ -435,7 +463,7 @@ class DistributionPanel(wx.Panel):
             )
         )
 
-        self.check_inside_xy = wx.CheckBox(
+        self.check_inside_xy = wxCheckBox(
             self, id=wx.ID_ANY, label=_("Keep first + last inside")
         )
         self.check_inside_xy.SetValue(True)
@@ -445,10 +473,10 @@ class DistributionPanel(wx.Panel):
             )
         )
 
-        self.check_rotate = wx.CheckBox(self, id=wx.ID_ANY, label=_("Rotate"))
+        self.check_rotate = wxCheckBox(self, id=wx.ID_ANY, label=_("Rotate"))
         self.check_rotate.SetToolTip(_("Rotate elements parallel to the path"))
 
-        self.rbox_sort = wx.RadioBox(
+        self.rbox_sort = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Work-Sequence:"),
@@ -461,7 +489,7 @@ class DistributionPanel(wx.Panel):
             _("Defines the order in which the selection is being processed")
         )
 
-        self.rbox_treatment = wx.RadioBox(
+        self.rbox_treatment = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Treatment:"),
@@ -488,20 +516,21 @@ class DistributionPanel(wx.Panel):
             + _("- Ref-Object: along the boundaries of a reference-object")
         )
 
-        self.btn_dist = wx.Button(self, wx.ID_ANY, "Distribute")
-        self.btn_dist.SetBitmap(icons8_arrange_50.GetBitmap(resize=25))
+        self.btn_dist = wxButton(self, wx.ID_ANY, "Distribute")
+        self.btn_dist.SetBitmap(
+            icons8_arrange.GetBitmap(resize=0.5 * get_default_icon_size(self.context))
+        )
 
-        sizer_check = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("First and last element treatment")),
+        sizer_check = StaticBoxSizer(
+            self,
+            wx.ID_ANY,
+            _("First and last element treatment"),
             wx.HORIZONTAL,
         )
         sizer_check.Add(self.check_inside_xy, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
         sizer_treat = wx.BoxSizer(wx.HORIZONTAL)
-        sizer_rotate = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Rotation")),
-            wx.HORIZONTAL,
-        )
+        sizer_rotate = StaticBoxSizer(self, wx.ID_ANY, _("Rotation"), wx.HORIZONTAL)
         sizer_rotate.Add(self.check_rotate, 1, wx.ALIGN_CENTER_VERTICAL, 0)
 
         sizer_treat.Add(self.rbox_treatment, 1, wx.EXPAND, 0)
@@ -523,10 +552,10 @@ class DistributionPanel(wx.Panel):
         self.Layout()
 
         self.btn_dist.Bind(wx.EVT_BUTTON, self.on_button_dist)
-        self.rbox_dist_x.Bind(wx.EVT_RADIOBOX, self.validate_data)
-        self.rbox_dist_y.Bind(wx.EVT_RADIOBOX, self.validate_data)
-        self.rbox_sort.Bind(wx.EVT_RADIOBOX, self.validate_data)
-        self.rbox_treatment.Bind(wx.EVT_RADIOBOX, self.validate_data)
+        self.Bind(wx.EVT_RADIOBOX, self.validate_data, self.rbox_dist_x)
+        self.Bind(wx.EVT_RADIOBOX, self.validate_data, self.rbox_dist_y)
+        self.Bind(wx.EVT_RADIOBOX, self.validate_data, self.rbox_sort)
+        self.Bind(wx.EVT_RADIOBOX, self.validate_data, self.rbox_treatment)
         self.context.setting(int, "distribute_x", 0)
         self.context.setting(int, "distribute_y", 0)
         self.context.setting(int, "distribute_treatment", 0)
@@ -543,6 +572,8 @@ class DistributionPanel(wx.Panel):
         if event is not None:
             event.Skip()
             obj = event.GetEventObject()
+        xmode = None
+        ymode = None
         if self.context.elements.has_emphasis():
             active = True
             idx = max(0, self.rbox_treatment.GetSelection())
@@ -553,7 +584,6 @@ class DistributionPanel(wx.Panel):
             ymode = self.xy_param[idx]
             idx = max(0, self.rbox_sort.GetSelection())
             esort = self.sort_param[idx]
-            rotate_elem = self.check_rotate.GetValue()
 
             # Have we just selected the treatment? Then set something useful
             if obj == self.rbox_treatment and xmode == "none" and ymode == "none":
@@ -621,13 +651,13 @@ class DistributionPanel(wx.Panel):
                     total_wd = right_edge - left_edge
                     data_wd = 0
                     firstwd = None
-                    lastwd = None
+                    # lastwd = None
                     for node in data:
                         bb = node.bounds
                         wd = bb[2] - bb[0]
                         if firstwd is None:
                             firstwd = wd
-                        lastwd = wd
+                        # lastwd = wd
                         data_wd += wd
                     # Reduce by first and last half width
                     # data_wd -= (firstwd + lastwd) / 2
@@ -661,13 +691,13 @@ class DistributionPanel(wx.Panel):
                     total_ht = bottom_edge - top_edge
                     data_ht = 0
                     firstht = None
-                    lastht = None
+                    # lastht = None
                     for node in data:
                         bb = node.bounds
                         ht = bb[3] - bb[1]
                         if firstht is None:
                             firstht = ht
-                        lastht = ht
+                        # lastht = ht
                         data_ht += ht
                     # Reduce by first and last half height
                     # data_ht -= (firstht + lastht) / 2
@@ -770,7 +800,7 @@ class DistributionPanel(wx.Panel):
                         last_y = pt.y
                 return this_length
 
-            def calc_slope(index):
+            def calc_angle(index):
                 try:
                     this_point = polypoints[index]
                 except IndexError:
@@ -781,43 +811,7 @@ class DistributionPanel(wx.Panel):
                     last_point = (0, 0, 0)
                 dx = this_point[0] - last_point[0]
                 dy = this_point[1] - last_point[1]
-                # TODO: Replace remaining code with atan2
-                # if dx < 1.0E-07:
-                #     dx = 0
-                # if dy < 1.0E-07:
-                #     dy = 0
-                # calc_atan(dx, dy):
-                if dx == 0:
-                    if dy < 0:
-                        c_angle = -1 / 4 * tau
-                        quadrant = 4
-                    elif dy == 0:
-                        c_angle = 0
-                        quadrant = 0
-                    else:
-                        c_angle = +1 / 4 * tau
-                        quadrant = 1
-                elif dx > 0 and dy >= 0:
-                    # Quadrant 1: angle between 0 und 90 (0 - tau / 4)
-                    c_angle = atan(dy / dx)
-                    quadrant = 1
-                elif dx < 0 and dy >= 0:
-                    # Quadrant 2: angle between 90 und 180 (1/4 tau - 2/4 tau)
-                    c_angle = atan(dy / dx) + tau / 2
-                    quadrant = 2
-                elif dx < 0 and dy < 0:
-                    # Quadrant 3: angle between 180 und 270 (2/4 tau - 3/4 tau)
-                    c_angle = atan(dy / dx) + tau / 2
-                    quadrant = 3
-                elif dx > 0 and dy < 0:
-                    # Quadrant 4: angle between 270 und 360 (2/4 tau - 3/4 tau)
-                    c_angle = atan(dy / dx)
-                    quadrant = 4
-                # print(
-                #     f"dx, dy={dx:.2f}, {dy:.2f}, Quadrant={quadrant}, "
-                #     + f"angle={c_angle:.2f} ({c_angle / tau * 360.0:.2f})"
-                # )
-                return c_angle
+                return atan2(dy, dx)
 
             polypoints = []
             poly_length = generate_polygon()
@@ -836,6 +830,7 @@ class DistributionPanel(wx.Panel):
             mydelta = poly_length / segcount
             lastx = 0
             lasty = 0
+            last_angle = None
             lastlen = 0
             segadded = 0
             # print(f"Expected segcount= {segcount}")
@@ -846,11 +841,11 @@ class DistributionPanel(wx.Panel):
                 y = pt[1]
                 if len(polypoints) > 1:
                     if idxpt == 0:
-                        ptangle = calc_slope(idxpt + 1)
+                        pt_angle = calc_angle(idxpt + 1)
                     else:
-                        ptangle = calc_slope(idxpt)
+                        pt_angle = calc_angle(idxpt)
                 else:
-                    ptangle = 0
+                    pt_angle = 0
                 plen = pt[2]
                 if abs(x) > 1.0e8 or abs(y) > 1.0e8:
                     # this does not seem to be a valid coord...
@@ -864,8 +859,8 @@ class DistributionPanel(wx.Panel):
                             fract = (mylen - lastlen) / (plen - lastlen)
                             x = lastx + fract * (x - lastx)
                             y = lasty + fract * (y - lasty)
-                    newpt = (x, y, ptangle)
-                    # print (f"I add: ({x:.1f}, {y:.1f}, {ptangle:.3f}) {ptangle/tau*360.0:.3f} ")
+                    newpt = (x, y, pt_angle)
+                    # print (f"I add: ({x:.1f}, {y:.1f}, {pt_angle:.3f}) {pt_angle/tau*360.0:.3f} ")
                     if newpt not in target:
                         # print ("..and added")
                         target.append(newpt)
@@ -875,7 +870,7 @@ class DistributionPanel(wx.Panel):
                 lastx = pt[0]
                 lasty = pt[1]
                 lastlen = pt[2]
-                last_angle = ptangle
+                last_angle = pt_angle
             # We may have slightly overshot, so in doubt add the last point
             if segadded < segcount:
                 # print ("I would add to it the last point...")
@@ -887,7 +882,7 @@ class DistributionPanel(wx.Panel):
             # print (f"Target points: {len(target)}")
 
         # "default", "shape", "points", "bed", "ref")
-        if treatment == "ref" and self.scene.reference_object is None:
+        if treatment == "ref" and self.scene.pane.reference_object is None:
             treatment = "default"
         if treatment == "default":
             # Let's get the boundaries of the data-set
@@ -904,14 +899,14 @@ class DistributionPanel(wx.Panel):
         elif treatment == "bed":
             left_edge = 0
             top_edge = 0
-            right_edge = float(Length(self.context.device.width))
-            bottom_edge = float(Length(self.context.device.height))
+            right_edge = float(Length(self.context.device.view.width))
+            bottom_edge = float(Length(self.context.device.view.height))
             calc_basic()
         elif treatment == "ref":
-            left_edge = self.scene.reference_object.bounds[0]
-            top_edge = self.scene.reference_object.bounds[1]
-            right_edge = self.scene.reference_object.bounds[2]
-            bottom_edge = self.scene.reference_object.bounds[3]
+            left_edge = self.scene.pane.reference_object.bounds[0]
+            top_edge = self.scene.pane.reference_object.bounds[1]
+            right_edge = self.scene.pane.reference_object.bounds[2]
+            bottom_edge = self.scene.pane.reference_object.bounds[3]
             calc_basic()
         elif treatment == "points":
             # So what's the reference node? And delete it...
@@ -1012,25 +1007,28 @@ class DistributionPanel(wx.Panel):
 
             if dx == 0 and dy == 0 and ptangle == 0:
                 continue
-            if (
-                hasattr(node, "lock")
-                and node.lock
-                and not self.context.elements.lock_allows_move
-            ):
+            if not node.can_move(self.context.elements.lock_allows_move):
                 continue
-            else:
-                try:
-                    cx = (node.bounds[2] + node.bounds[0]) / 2 + dx
-                    cy = (node.bounds[3] + node.bounds[1]) / 2 + dy
-                    if dx != 0 or dy != 0:
-                        node.matrix.post_translate(dx, dy)
-                    # Do we have a rotation to take into account?
-                    if ptangle != 0:
-                        node.matrix.post_rotate(ptangle, cx, cy)
+
+            try:
+                cx = (node.bounds[2] + node.bounds[0]) / 2 + dx
+                cy = (node.bounds[3] + node.bounds[1]) / 2 + dy
+                change = 0
+                if dx != 0 or dy != 0:
+                    node.matrix.post_translate(dx, dy)
+                    change = 1
+                # Do we have a rotation to take into account?
+                if ptangle != 0:
+                    node.matrix.post_rotate(ptangle, cx, cy)
+                    change = 2
+                if change == 1:
+                    node.translated(dx, dy)
+                elif change == 2:
                     node.modified()
-                    modified += 1
-                except AttributeError:
-                    continue
+
+                modified += 1
+            except AttributeError:
+                pass
         # print(f"Modified: {modified}")
 
     def on_button_dist(self, event):
@@ -1101,7 +1099,7 @@ class DistributionPanel(wx.Panel):
         self.count, self.first_node, self.last_node = self.info_panel.show_stuff(
             has_emph
         )
-        flag = self.scene.reference_object is not None
+        flag = self.scene.pane.reference_object is not None
         self.rbox_treatment.EnableItem(4, flag)
 
         if showit:
@@ -1115,6 +1113,8 @@ class ArrangementPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0)
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
+        self.SetHelpText("arrangement")
         self.scene = scene
         # Amount of currently selected
         self.count = 0
@@ -1139,7 +1139,7 @@ class ArrangementPanel(wx.Panel):
         self.ychoices = (_("Top"), _("Center"), _("Bottom"))
         self.xyparam = ("min", "center", "max")
 
-        self.rbox_align_x = wx.RadioBox(
+        self.rbox_align_x = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Alignment relative to X-Axis:"),
@@ -1149,7 +1149,7 @@ class ArrangementPanel(wx.Panel):
         )
         self.rbox_align_x.SetSelection(0)
 
-        self.rbox_align_y = wx.RadioBox(
+        self.rbox_align_y = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Alignment relative to Y-Axis:"),
@@ -1162,10 +1162,10 @@ class ArrangementPanel(wx.Panel):
         self.arrange_x = wx.SpinCtrl(self, wx.ID_ANY, initial=1, min=1, max=100)
         self.arrange_y = wx.SpinCtrl(self, wx.ID_ANY, initial=1, min=1, max=100)
 
-        self.check_same_x = wx.CheckBox(self, wx.ID_ANY, label=_("Same width"))
-        self.check_same_y = wx.CheckBox(self, wx.ID_ANY, label=_("Same height"))
+        self.check_same_x = wxCheckBox(self, wx.ID_ANY, label=_("Same width"))
+        self.check_same_y = wxCheckBox(self, wx.ID_ANY, label=_("Same height"))
 
-        self.rbox_relation = wx.RadioBox(
+        self.rbox_relation = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Arrangement inside grid:"),
@@ -1175,7 +1175,7 @@ class ArrangementPanel(wx.Panel):
         )
         self.rbox_relation.SetSelection(0)
 
-        self.rbox_selection = wx.RadioBox(
+        self.rbox_selection = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Order to process:"),
@@ -1192,19 +1192,25 @@ class ArrangementPanel(wx.Panel):
             self, id=wx.ID_ANY, value="5mm", limited=True, check="length"
         )
 
-        self.btn_arrange = wx.Button(self, wx.ID_ANY, _("Arrange"))
-        self.btn_arrange.SetBitmap(icons8_arrange_50.GetBitmap(resize=25))
+        self.btn_arrange = wxButton(self, wx.ID_ANY, _("Arrange"))
+        self.btn_arrange.SetBitmap(
+            icons8_arrange.GetBitmap(resize=0.5 * get_default_icon_size(self.context))
+        )
 
         sizer_dimensions = wx.BoxSizer(wx.HORIZONTAL)
-        sizer_dim_x = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("X-Axis:")),
+        sizer_dim_x = StaticBoxSizer(
+            self,
+            wx.ID_ANY,
+            _("X-Axis:"),
             wx.VERTICAL,
         )
         sizer_dim_x.Add(self.arrange_x, 0, wx.EXPAND, 0)
         sizer_dim_x.Add(self.check_same_x, 0, wx.EXPAND, 0)
 
-        sizer_dim_y = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Y-Axis:")),
+        sizer_dim_y = StaticBoxSizer(
+            self,
+            wx.ID_ANY,
+            _("Y-Axis:"),
             wx.VERTICAL,
         )
         sizer_dim_y.Add(self.arrange_y, 0, wx.EXPAND, 0)
@@ -1213,23 +1219,17 @@ class ArrangementPanel(wx.Panel):
         sizer_dimensions.Add(sizer_dim_x, 1, wx.EXPAND, 0)
         sizer_dimensions.Add(sizer_dim_y, 1, wx.EXPAND, 0)
 
-        # sizer_gaps = wx.StaticBoxSizer(
-        #     wx.HORIZONTAL,
-        #     wx.StaticBox(self, wx.ID_ANY, _("Gaps:")),
-        # )
         sizer_gaps_x = wx.BoxSizer(wx.HORIZONTAL)
         sizer_gaps_x.Add(
-            wx.StaticText(self, wx.ID_ANY, _("X:")), 0, wx.ALIGN_CENTER_VERTICAL, 0
+            wxStaticText(self, wx.ID_ANY, _("X:")), 0, wx.ALIGN_CENTER_VERTICAL, 0
         )
         sizer_gaps_x.Add(self.txt_gap_x, 1, wx.EXPAND, 0)
         sizer_gaps_y = wx.BoxSizer(wx.HORIZONTAL)
         sizer_gaps_y.Add(
-            wx.StaticText(self, wx.ID_ANY, _("Y:")), 0, wx.ALIGN_CENTER_VERTICAL, 0
+            wxStaticText(self, wx.ID_ANY, _("Y:")), 0, wx.ALIGN_CENTER_VERTICAL, 0
         )
         sizer_gaps_y.Add(self.txt_gap_y, 1, wx.EXPAND, 0)
-        sizer_gaps_xy = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Gaps:")), wx.VERTICAL
-        )
+        sizer_gaps_xy = StaticBoxSizer(self, wx.ID_ANY, _("Gaps:"), wx.VERTICAL)
         sizer_gaps_xy.Add(sizer_gaps_x, 1, wx.EXPAND, 0)
         sizer_gaps_xy.Add(sizer_gaps_y, 1, wx.EXPAND, 0)
         sizer_gaps = wx.BoxSizer(wx.HORIZONTAL)
@@ -1252,22 +1252,22 @@ class ArrangementPanel(wx.Panel):
 
         self.Layout()
         self.btn_arrange.SetToolTip(_("Rearrange all selected elements"))
-        self.rbox_align_x.SetToolTip(_(""))
-        self.rbox_align_y.SetToolTip(_(""))
+        # self.rbox_align_x.SetToolTip(_(""))
+        # self.rbox_align_y.SetToolTip(_(""))
         self.check_same_x.SetToolTip(
             _(
-                "Set if all columns need to have the same size (ie maximum width over all columns)"
+                "Set if all columns need to have the same size (i.e. maximum width over all columns)"
             )
         )
         self.check_same_y.SetToolTip(
             _(
-                "Set if all rows need to have the same size (ie maximum height over all row)"
+                "Set if all rows need to have the same size (i.e. maximum height over all row)"
             )
         )
-        self.rbox_relation.SetToolTip(_(""))
-        self.rbox_selection.SetToolTip(_(""))
-        self.arrange_x.SetToolTip(_(""))
-        self.arrange_y.SetToolTip(_(""))
+        # self.rbox_relation.SetToolTip(_(""))
+        # self.rbox_selection.SetToolTip(_(""))
+        # self.arrange_x.SetToolTip(_(""))
+        # self.arrange_y.SetToolTip(_(""))
         self.txt_gap_x.SetToolTip(_("Set the distance between columns"))
         self.txt_gap_y.SetToolTip(_("Set the distance between rows"))
 
@@ -1306,24 +1306,24 @@ class ArrangementPanel(wx.Panel):
             if self.count < 2 or self.count > num_cols * num_rows:
                 # print(f"Too small: {self.count} vs. {num_cols}x{num_rows}")
                 active = False
-            idx = self.rbox_selection.GetSelection()
-            if idx < 0:
-                idx = 0
-            esort = self.selectparam[idx]
+            # idx = self.rbox_selection.GetSelection()
+            # if idx < 0:
+            #     idx = 0
+            # esort = self.selectparam[idx]
             idx = self.rbox_relation.GetSelection()
             if idx < 0:
                 idx = 0
             relat = self.relparam[idx]
             self.txt_gap_x.Enable(relat == "distance")
             self.txt_gap_y.Enable(relat == "distance")
-            idx = self.rbox_align_x.GetSelection()
-            if idx < 0:
-                idx = 0
-            xpos = self.xyparam[idx]
-            idx = self.rbox_align_y.GetSelection()
-            if idx < 0:
-                idx = 0
-            ypos = self.xyparam[idx]
+            # idx = self.rbox_align_x.GetSelection()
+            # if idx < 0:
+            #     idx = 0
+            # xpos = self.xyparam[idx]
+            # idx = self.rbox_align_y.GetSelection()
+            # if idx < 0:
+            #     idx = 0
+            # ypos = self.xyparam[idx]
             try:
                 gapx = float(Length(self.txt_gap_x.GetValue()))
             except ValueError:
@@ -1400,8 +1400,8 @@ class ArrangementPanel(wx.Panel):
             # Now that we have established the global boundaries,
             # we are going to center it on the scene...
             # By definition the origin was set to 0 0
-            dx = float(Length(self.context.device.width)) / 2 - (0 + max_xx) / 2
-            dy = float(Length(self.context.device.height)) / 2 - (0 + max_yy) / 2
+            dx = float(Length(self.context.device.view.width)) / 2 - (0 + max_xx) / 2
+            dy = float(Length(self.context.device.view.height)) / 2 - (0 + max_yy) / 2
             for idx, bb in enumerate(target):
                 newbb = (bb[0] + dx, bb[1] + dy, bb[2] + dx, bb[3] + dy)
                 target[idx] = newbb
@@ -1449,21 +1449,18 @@ class ArrangementPanel(wx.Panel):
                 # s += f"dx={Length(amount=dy, unitless=1, digits=1).length_mm}"
                 # print (s)
                 if dx != 0 or dy != 0:
-                    if (
-                        hasattr(node, "lock")
-                        and node.lock
-                        and not self.context.elements.lock_allows_move
-                    ):
+                    if not node.can_move(self.context.elements.lock_allows_move):
                         continue
                     else:
                         try:
                             node.matrix.post_translate(dx, dy)
-                            node.modified()
+                            # node.modified()
+                            node.translated(dx, dy)
                         except AttributeError:
                             pass
 
         num_cols = self.arrange_x.GetValue()
-        num_rows = self.arrange_y.GetValue()
+        # num_rows = self.arrange_y.GetValue()
         same_x = self.check_same_x.GetValue()
         same_y = self.check_same_y.GetValue()
         idx = self.rbox_selection.GetSelection()
@@ -1554,8 +1551,8 @@ class ArrangementPanel(wx.Panel):
 class Alignment(MWindow):
     def __init__(self, *args, **kwds):
         super().__init__(
-            350,
-            350,
+            360,
+            485,
             *args,
             style=wx.CAPTION
             | wx.CLOSE_BOX
@@ -1572,6 +1569,12 @@ class Alignment(MWindow):
             | wx.aui.AUI_NB_TAB_SPLIT
             | wx.aui.AUI_NB_TAB_MOVE,
         )
+        self.window_context.themes.set_window_colors(self.notebook_main)
+        bg_std = self.window_context.themes.get("win_bg")
+        bg_active = self.window_context.themes.get("highlight")
+        self.notebook_main.GetArtProvider().SetColour(bg_std)
+        self.notebook_main.GetArtProvider().SetActiveColour(bg_active)
+
         self.scene = getattr(self.context.root, "mainscene", None)
         self.panel_align = AlignmentPanel(
             self, wx.ID_ANY, context=self.context, scene=self.scene
@@ -1585,11 +1588,12 @@ class Alignment(MWindow):
             self, wx.ID_ANY, context=self.context, scene=self.scene
         )
         self.notebook_main.AddPage(self.panel_arrange, _("Arrange"))
-
+        self.sizer.Add(self.notebook_main, 1, wx.EXPAND, 0)
         self.Layout()
+        self.restore_aspect()
 
         _icon = wx.NullIcon
-        _icon.CopyFromBitmap(icons8_arrange_50.GetBitmap(resize=25))
+        _icon.CopyFromBitmap(icons8_arrange.GetBitmap())
         self.SetIcon(_icon)
         self.SetTitle(_("Alignment"))
 
@@ -1608,13 +1612,14 @@ class Alignment(MWindow):
 
     @staticmethod
     def sub_register(kernel):
-        buttonsize = int(STD_ICON_SIZE / 2)
+        buttonsize = STD_ICON_SIZE
         kernel.register(
             "button/align/AlignExpert",
             {
                 "label": _("Expert Mode"),
-                "icon": icons8_arrange_50,
+                "icon": icons8_arrange,
                 "tip": _("Open alignment dialog with advanced options"),
+                "help": "alignment",
                 "action": lambda v: kernel.console("window toggle Alignment\n"),
                 "size": buttonsize,
                 "rule_enabled": lambda cond: len(
@@ -1632,4 +1637,9 @@ class Alignment(MWindow):
 
     @staticmethod
     def submenu():
-        return ("Editing", "Element Alignment")
+        # Hint for Translation: _("Editing"), _("Element Alignment")
+        return "Editing", "Element Alignment"
+
+    @staticmethod
+    def helptext():
+        return _("Align elements to each other")
