@@ -1,8 +1,22 @@
 import wx
 
-from meerk40t.gui.icons import STD_ICON_SIZE, icons8_keyhole_50, icons8_split_table_50
+from meerk40t.gui.icons import (
+    STD_ICON_SIZE,
+    get_default_icon_size,
+    icon_keyhole,
+    icon_split_image,
+)
 from meerk40t.gui.mwindow import MWindow
-from meerk40t.gui.wxutils import TextCtrl
+from meerk40t.gui.wxutils import (
+    StaticBoxSizer,
+    TextCtrl,
+    dip_size,
+    wxButton,
+    wxCheckBox,
+    wxRadioBox,
+    wxStaticBitmap,
+    wxStaticText,
+)
 from meerk40t.kernel import signal_listener
 from meerk40t.svgelements import Color
 
@@ -14,19 +28,20 @@ class InfoPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0)
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
-        self.lbl_info_main = wx.StaticText(self, wx.ID_ANY, "")
-        self.lbl_info_default = wx.StaticText(self, wx.ID_ANY, "")
-        self.lbl_info_first = wx.StaticText(self, wx.ID_ANY, "")
-        self.lbl_info_last = wx.StaticText(self, wx.ID_ANY, "")
+        self.context.themes.set_window_colors(self)
+        self.lbl_info_main = wxStaticText(self, wx.ID_ANY, "")
+        self.lbl_info_default = wxStaticText(self, wx.ID_ANY, "")
+        self.lbl_info_first = wxStaticText(self, wx.ID_ANY, "")
+        self.lbl_info_last = wxStaticText(self, wx.ID_ANY, "")
         self.preview_size = 25
-        self.image_default = wx.StaticBitmap(
-            self, wx.ID_ANY, size=wx.Size(self.preview_size, self.preview_size)
+        self.image_default = wxStaticBitmap(
+            self, wx.ID_ANY, size=dip_size(self, self.preview_size, self.preview_size)
         )
-        self.image_first = wx.StaticBitmap(
-            self, wx.ID_ANY, size=wx.Size(self.preview_size, self.preview_size)
+        self.image_first = wxStaticBitmap(
+            self, wx.ID_ANY, size=dip_size(self, self.preview_size, self.preview_size)
         )
-        self.image_last = wx.StaticBitmap(
-            self, wx.ID_ANY, size=wx.Size(self.preview_size, self.preview_size)
+        self.image_last = wxStaticBitmap(
+            self, wx.ID_ANY, size=dip_size(self, self.preview_size, self.preview_size)
         )
         sizer_main = wx.BoxSizer(wx.VERTICAL)
 
@@ -48,7 +63,7 @@ class InfoPanel(wx.Panel):
         sizer_main.Add(sizer_last, 0, wx.EXPAND, 0)
         self.make_raster = None
         self.SetSizer(sizer_main)
-        self.Layout
+        self.Layout()
 
     def show_stuff(self, has_emph):
         def create_image_from_node(node, iconsize):
@@ -64,14 +79,17 @@ class InfoPanel(wx.Panel):
                 ):
                     c = node.stroke
             if node.type.startswith("elem ") and node.type != "elem point":
-                image = self.make_raster(
-                    node,
-                    node.paint_bounds,
-                    width=iconsize,
-                    height=iconsize,
-                    bitmap=True,
-                    keep_ratio=True,
-                )
+                try:
+                    image = self.make_raster(
+                        node,
+                        node.paint_bounds,
+                        width=iconsize,
+                        height=iconsize,
+                        bitmap=True,
+                        keep_ratio=True,
+                    )
+                except Exception:
+                    return None, None
 
             if c is None:
                 c = defaultcolor
@@ -81,7 +99,6 @@ class InfoPanel(wx.Panel):
             self.make_raster = self.context.elements.lookup("render-op/make_raster")
 
         count = 0
-        msg = ""
         if has_emph:
             data = list(self.context.elements.flat(emphasized=True))
             count = len(data)
@@ -91,6 +108,8 @@ class InfoPanel(wx.Panel):
             if count > 0:
                 node = data[0]
                 c, image = create_image_from_node(node, self.preview_size)
+                if c is None:
+                    return
                 self.image_default.SetBitmap(image)
                 self.lbl_info_default.SetLabel(
                     _("As in Selection: {type} {lbl}").format(
@@ -101,6 +120,8 @@ class InfoPanel(wx.Panel):
                 data.sort(key=lambda n: n.emphasized_time)
                 node = data[0]
                 c, image = create_image_from_node(node, self.preview_size)
+                if c is None:
+                    return
                 self.image_first.SetBitmap(image)
                 self.lbl_info_first.SetLabel(
                     _("First selected: {type} {lbl}").format(
@@ -110,6 +131,8 @@ class InfoPanel(wx.Panel):
 
                 node = data[-1]
                 c, image = create_image_from_node(node, self.preview_size)
+                if c is None:
+                    return
                 self.image_last.SetBitmap(image)
                 self.lbl_info_last.SetLabel(
                     _("Last selected: {type} {lbl}").format(
@@ -132,6 +155,8 @@ class SplitterPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0)
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
+        self.SetHelpText("imagesplit")
         self.scene = scene
         # Amount of currently selected
         self.count = 0
@@ -149,7 +174,7 @@ class SplitterPanel(wx.Panel):
         self.split_x = wx.SpinCtrl(self, wx.ID_ANY, initial=1, min=1, max=25)
         self.split_y = wx.SpinCtrl(self, wx.ID_ANY, initial=1, min=1, max=25)
 
-        self.rbox_selection = wx.RadioBox(
+        self.rbox_selection = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Order to process:"),
@@ -160,28 +185,30 @@ class SplitterPanel(wx.Panel):
         self.rbox_selection.SetSelection(0)
         self.text_dpi = TextCtrl(self, wx.ID_ANY, limited=True, check="int")
         self.text_dpi.SetValue("500")
-        self.lbl_info = wx.StaticText(self, wx.ID_ANY, "")
-        self.btn_align = wx.Button(self, wx.ID_ANY, _("Create split images"))
-        self.btn_align.SetBitmap(icons8_split_table_50.GetBitmap(resize=25))
+        self.text_dpi.set_default_values(
+            [
+                (str(dpi), _("Set DPI to {value}").format(value=str(dpi)))
+                for dpi in self.context.device.view.get_sensible_dpi_values()
+            ]
+        )
+        self.lbl_info = wxStaticText(self, wx.ID_ANY, "")
+        self.btn_align = wxButton(self, wx.ID_ANY, _("Create split images"))
+        self.btn_align.SetBitmap(
+            icon_split_image.GetBitmap(resize=0.5 * get_default_icon_size(self.context))
+        )
 
-        lbl_dpi = wx.StaticText(self, wx.ID_ANY, "DPI:")
-        sizer_dpi = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Image resolution:")), wx.HORIZONTAL
+        lbl_dpi = wxStaticText(self, wx.ID_ANY, "DPI:")
+        sizer_dpi = StaticBoxSizer(
+            self, wx.ID_ANY, _("Image resolution:"), wx.HORIZONTAL
         )
         sizer_dpi.Add(lbl_dpi, 0, wx.ALIGN_CENTER_VERTICAL, 0)
         sizer_dpi.Add(self.text_dpi, 1, wx.EXPAND, 0)
 
         sizer_dimensions = wx.BoxSizer(wx.HORIZONTAL)
-        sizer_dim_x = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("X-Axis:")),
-            wx.VERTICAL,
-        )
+        sizer_dim_x = StaticBoxSizer(self, wx.ID_ANY, _("X-Axis:"), wx.VERTICAL)
         sizer_dim_x.Add(self.split_x, 0, wx.EXPAND, 0)
 
-        sizer_dim_y = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Y-Axis:")),
-            wx.VERTICAL,
-        )
+        sizer_dim_y = StaticBoxSizer(self, wx.ID_ANY, _("Y-Axis:"), wx.VERTICAL)
         sizer_dim_y.Add(self.split_y, 0, wx.EXPAND, 0)
 
         sizer_dimensions.Add(sizer_dim_x, 1, wx.EXPAND, 0)
@@ -219,12 +246,12 @@ class SplitterPanel(wx.Panel):
             event.Skip()
         if self.context.elements.has_emphasis():
             active = True
-            num_cols = self.split_x.GetValue()
-            num_rows = self.split_y.GetValue()
+            # num_cols = self.split_x.GetValue()
+            # num_rows = self.split_y.GetValue()
             idx = self.rbox_selection.GetSelection()
             if idx < 0:
                 idx = 0
-            esort = self.selectparam[idx]
+            # esort = self.selectparam[idx]
             try:
                 dpi = int(self.text_dpi.GetValue())
             except ValueError:
@@ -280,6 +307,8 @@ class KeyholePanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0)
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
+        self.SetHelpText("keyhole")
         self.scene = scene
         # Amount of currently selected
         self.count = 0
@@ -293,7 +322,7 @@ class KeyholePanel(wx.Panel):
         )
         self.selectparam = ("first", "last")
 
-        self.rbox_selection = wx.RadioBox(
+        self.rbox_selection = wxRadioBox(
             self,
             wx.ID_ANY,
             _("Keyhole Object:"),
@@ -304,29 +333,35 @@ class KeyholePanel(wx.Panel):
         self.rbox_selection.SetSelection(0)
         self.text_dpi = TextCtrl(self, wx.ID_ANY, limited=True, check="int")
         self.text_dpi.SetValue("500")
+        self.text_dpi.set_default_values(
+            [
+                (str(dpi), _("Set DPI to {value}").format(value=str(dpi)))
+                for dpi in self.context.device.view.get_sensible_dpi_values()
+            ]
+        )
         self.info_panel = InfoPanel(self, wx.ID_ANY, context=self.context)
 
-        self.btn_align = wx.Button(self, wx.ID_ANY, _("Create keyhole image"))
-        self.btn_align.SetBitmap(icons8_keyhole_50.GetBitmap(resize=25))
+        self.btn_align = wxButton(self, wx.ID_ANY, _("Create keyhole image"))
+        self.btn_align.SetBitmap(
+            icon_keyhole.GetBitmap(resize=0.5 * get_default_icon_size(self.context))
+        )
 
-        lbl_dpi = wx.StaticText(self, wx.ID_ANY, "DPI:")
-        sizer_dpi = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Image resolution:")), wx.HORIZONTAL
+        lbl_dpi = wxStaticText(self, wx.ID_ANY, "DPI:")
+        sizer_dpi = StaticBoxSizer(
+            self, wx.ID_ANY, _("Image resolution:"), wx.HORIZONTAL
         )
         sizer_dpi.Add(lbl_dpi, 0, wx.ALIGN_CENTER_VERTICAL, 0)
         sizer_dpi.Add(self.text_dpi, 1, wx.EXPAND, 0)
 
         sizer_options = wx.BoxSizer(wx.HORIZONTAL)
-        sizer_check_invert = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Invert Mask:")),
-            wx.HORIZONTAL,
+        sizer_check_invert = StaticBoxSizer(
+            self, wx.ID_ANY, _("Invert Mask:"), wx.HORIZONTAL
         )
-        sizer_check_outline = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Trace Keyhole:")),
-            wx.HORIZONTAL,
+        sizer_check_outline = StaticBoxSizer(
+            self, wx.ID_ANY, _("Trace Keyhole:"), wx.HORIZONTAL
         )
-        self.check_invert = wx.CheckBox(self, wx.ID_ANY, "Invert")
-        self.check_outline = wx.CheckBox(self, wx.ID_ANY, "Trace")
+        self.check_invert = wxCheckBox(self, wx.ID_ANY, "Invert")
+        self.check_outline = wxCheckBox(self, wx.ID_ANY, "Trace")
 
         sizer_check_invert.Add(self.check_invert, 0, wx.ALIGN_CENTER_VERTICAL, 0)
         sizer_check_outline.Add(self.check_outline, 0, wx.ALIGN_CENTER_VERTICAL, 0)
@@ -362,12 +397,12 @@ class KeyholePanel(wx.Panel):
             event.Skip()
         if self.context.elements.has_emphasis():
             active = True
-            invert = self.check_invert.GetValue()
-            outline = self.check_outline.GetValue()
+            # invert = self.check_invert.GetValue()
+            # outline = self.check_outline.GetValue()
             idx = self.rbox_selection.GetSelection()
             if idx < 0:
                 idx = 0
-            esort = self.selectparam[idx]
+            # esort = self.selectparam[idx]
             try:
                 dpi = int(self.text_dpi.GetValue())
             except ValueError:
@@ -391,11 +426,11 @@ class KeyholePanel(wx.Panel):
         except ValueError:
             mydpi = 500
         if self.check_invert.GetValue():
-            invert = " --invert 1"
+            invert = " --invert"
         else:
             invert = ""
         if self.check_outline.GetValue():
-            outline = " --outline 1"
+            outline = " --outline"
         else:
             outline = ""
         cmdstr = f"render_keyhole {mydpi} --order {esort}{invert}{outline}\n"
@@ -447,6 +482,13 @@ class RenderSplit(MWindow):
             | wx.aui.AUI_NB_TAB_SPLIT
             | wx.aui.AUI_NB_TAB_MOVE,
         )
+        self.window_context.themes.set_window_colors(self.notebook_main)
+        bg_std = self.window_context.themes.get("win_bg")
+        bg_active = self.window_context.themes.get("highlight")
+        self.notebook_main.GetArtProvider().SetColour(bg_std)
+        self.notebook_main.GetArtProvider().SetActiveColour(bg_active)
+
+        self.sizer.Add(self.notebook_main, 1, wx.EXPAND, 0)
         self.scene = getattr(self.context.root, "mainscene", None)
         # Hide Arrangement until ready...
         self.panel_split = SplitterPanel(
@@ -459,9 +501,10 @@ class RenderSplit(MWindow):
         self.notebook_main.AddPage(self.panel_keyhole, _("Keyhole operation"))
 
         self.Layout()
+        self.restore_aspect()
 
         _icon = wx.NullIcon
-        _icon.CopyFromBitmap(icons8_split_table_50.GetBitmap(resize=25))
+        _icon.CopyFromBitmap(icon_split_image.GetBitmap())
         self.SetIcon(_icon)
         self.SetTitle(_("Create split images"))
 
@@ -479,14 +522,15 @@ class RenderSplit(MWindow):
     @staticmethod
     def sub_register(kernel):
         bsize_normal = STD_ICON_SIZE
-        bsize_small = int(STD_ICON_SIZE / 2)
+        # bsize_small = int(STD_ICON_SIZE / 2)
 
         kernel.register(
             "button/align/SplitImage",
             {
-                "label": _("Splitter"),
-                "icon": icons8_split_table_50,
-                "tip": _("Open create split image dialog"),
+                "label": _("Image ops"),
+                "icon": icon_split_image,
+                "tip": _("Open create split image dialog / keyhole generation"),
+                "help": "imagesplit",
                 "action": lambda v: kernel.console("window toggle SplitImage\n"),
                 "size": bsize_normal,
                 "rule_enabled": lambda cond: len(
@@ -504,4 +548,9 @@ class RenderSplit(MWindow):
 
     @staticmethod
     def submenu():
-        return ("Editing", "Image Splitting")
+        # Hint for translation: _("Editing"), _("Image Splitting")
+        return "Editing", "Image Splitting"
+
+    @staticmethod
+    def helptext():
+        return _("Split images for large jobs")
