@@ -1,15 +1,22 @@
 from math import atan, sqrt, tau
 
 import wx
-from wx import aui
 
 from meerk40t.core.units import Length
 from meerk40t.gui.icons import (
-    instruction_circle,
-    instruction_frame,
-    instruction_rectangle,
+    icon_instruct_circle,
+    icon_instruct_rect,
+    icon_instruct_square,
 )
 from meerk40t.gui.mwindow import MWindow
+from meerk40t.gui.wxutils import (
+    TextCtrl,
+    dip_size,
+    wxButton,
+    wxCheckBox,
+    wxStaticBitmap,
+    wxStaticText,
+)
 from meerk40t.kernel import signal_listener
 
 _ = wx.GetTranslation
@@ -24,6 +31,8 @@ class LaserToolPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
+        self.SetHelpText("templates")
         self.coord_a = None
         self.coord_b = None
         self.coord_c = None
@@ -32,6 +41,8 @@ class LaserToolPanel(wx.Panel):
         sizer_main = wx.BoxSizer(wx.VERTICAL)
 
         self.nbook_lasertools = wx.Notebook(self, wx.ID_ANY)
+        self.context.themes.set_window_colors(self.nbook_lasertools)
+
         sizer_main.Add(self.nbook_lasertools, 1, wx.EXPAND, 0)
 
         # ------------------------ Circle with 3 points
@@ -50,50 +61,55 @@ class LaserToolPanel(wx.Panel):
         sizer_1 = wx.BoxSizer(wx.HORIZONTAL)
         sizer_10.Add(sizer_1, 1, wx.EXPAND, 0)
 
-        label_1 = wx.StaticText(self.nb_circle, wx.ID_ANY, _("A"))
+        label_1 = wxStaticText(self.nb_circle, wx.ID_ANY, _("A"))
         sizer_1.Add(label_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_set_circle_1 = wx.Button(self.nb_circle, wx.ID_ANY, _("Use position"))
+        self.btn_set_circle_1 = wxButton(self.nb_circle, wx.ID_ANY, _("Use position"))
         self.btn_set_circle_1.SetToolTip(
             _("Place the laser over the desired point and click...")
         )
         sizer_1.Add(self.btn_set_circle_1, 0, wx.EXPAND, 0)
 
-        self.lbl_pos_1 = wx.StaticText(self.nb_circle, wx.ID_ANY, _("<empty>"))
+        # How big should graphics become?
+        desired_height = 6 * self.btn_set_circle_1.Size[1]
+
+        self.lbl_pos_1 = wxStaticText(self.nb_circle, wx.ID_ANY, _("<empty>"))
         sizer_1.Add(self.lbl_pos_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
         sizer_2 = wx.BoxSizer(wx.HORIZONTAL)
         sizer_10.Add(sizer_2, 1, wx.EXPAND, 0)
 
-        label_2 = wx.StaticText(self.nb_circle, wx.ID_ANY, _("B"))
+        label_2 = wxStaticText(self.nb_circle, wx.ID_ANY, _("B"))
         sizer_2.Add(label_2, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_set_circle_2 = wx.Button(self.nb_circle, wx.ID_ANY, _("Use position"))
+        self.btn_set_circle_2 = wxButton(self.nb_circle, wx.ID_ANY, _("Use position"))
         self.btn_set_circle_2.SetToolTip(
             _("Place the laser over the desired point and click...")
         )
         sizer_2.Add(self.btn_set_circle_2, 0, wx.EXPAND, 0)
 
-        self.lbl_pos_2 = wx.StaticText(self.nb_circle, wx.ID_ANY, _("<empty>"))
+        self.lbl_pos_2 = wxStaticText(self.nb_circle, wx.ID_ANY, _("<empty>"))
         sizer_2.Add(self.lbl_pos_2, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
         sizer_3 = wx.BoxSizer(wx.HORIZONTAL)
         sizer_10.Add(sizer_3, 1, wx.EXPAND, 0)
 
-        label_3 = wx.StaticText(self.nb_circle, wx.ID_ANY, _("C"))
+        label_3 = wxStaticText(self.nb_circle, wx.ID_ANY, _("C"))
         sizer_3.Add(label_3, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_set_circle_3 = wx.Button(self.nb_circle, wx.ID_ANY, _("Use position"))
+        self.btn_set_circle_3 = wxButton(self.nb_circle, wx.ID_ANY, _("Use position"))
         self.btn_set_circle_3.SetToolTip(
             _("Place the laser over the desired point and click...")
         )
         sizer_3.Add(self.btn_set_circle_3, 0, wx.EXPAND, 0)
 
-        self.lbl_pos_3 = wx.StaticText(self.nb_circle, wx.ID_ANY, _("<empty>"))
+        self.lbl_pos_3 = wxStaticText(self.nb_circle, wx.ID_ANY, _("<empty>"))
         sizer_3.Add(self.lbl_pos_3, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        img_instruction_1 = wx.StaticBitmap(
-            self.nb_circle, wx.ID_ANY, instruction_circle.GetBitmap()
+        img_instruction_1 = wxStaticBitmap(
+            self.nb_circle,
+            wx.ID_ANY,
+            self.fitted_bitmap(icon_instruct_circle, desired_height),
         )
         instructions = _(
             "Instruction: place the laser on three points on the circumference of the circle on the bed and confirm the position by clicking on the buttons below.\nMK will find the center for you and place the laser above it or will recreate the circle for further processing."
@@ -103,24 +119,22 @@ class LaserToolPanel(wx.Panel):
 
         sizer_chk = wx.BoxSizer(wx.HORIZONTAL)
         self.sizer_circle.Add(sizer_chk, 0, wx.EXPAND, 0)
-        self.check_ref_circle = wx.CheckBox(
+        self.check_ref_circle = wxCheckBox(
             self.nb_circle, wx.ID_ANY, _("Make reference")
         )
         sizer_chk.Add(self.check_ref_circle, 0, wx.ALIGN_CENTER_VERTICAL, 0)
-        self.check_circle = wx.CheckBox(self.nb_circle, wx.ID_ANY, _("Mark Center"))
+        self.check_circle = wxCheckBox(self.nb_circle, wx.ID_ANY, _("Mark Center"))
         sizer_chk.Add(self.check_circle, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
         sizer_4 = wx.BoxSizer(wx.HORIZONTAL)
         self.sizer_circle.Add(sizer_4, 0, wx.EXPAND, 0)
 
-        self.btn_move_to_center = wx.Button(
+        self.btn_move_to_center = wxButton(
             self.nb_circle, wx.ID_ANY, _("Move to center")
         )
         sizer_4.Add(self.btn_move_to_center, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_create_circle = wx.Button(
-            self.nb_circle, wx.ID_ANY, _("Create circle")
-        )
+        self.btn_create_circle = wxButton(self.nb_circle, wx.ID_ANY, _("Create circle"))
         sizer_4.Add(self.btn_create_circle, 0, wx.EXPAND, 0)
 
         # ------------------------ Rectangle with 2 points
@@ -139,35 +153,37 @@ class LaserToolPanel(wx.Panel):
         sizer_5a = wx.BoxSizer(wx.HORIZONTAL)
         sizer_rect_vert.Add(sizer_5a, 1, wx.EXPAND, 0)
 
-        label_corner_1 = wx.StaticText(self.nb_rectangle, wx.ID_ANY, _("Corner 1"))
+        label_corner_1 = wxStaticText(self.nb_rectangle, wx.ID_ANY, _("Corner 1"))
         sizer_5a.Add(label_corner_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_set_rect_1 = wx.Button(self.nb_rectangle, wx.ID_ANY, _("Use position"))
+        self.btn_set_rect_1 = wxButton(self.nb_rectangle, wx.ID_ANY, _("Use position"))
         self.btn_set_rect_1.SetToolTip(
             _("Place the laser over the desired point and click...")
         )
         sizer_5a.Add(self.btn_set_rect_1, 0, wx.EXPAND, 0)
 
-        self.lbl_pos_7 = wx.StaticText(self.nb_rectangle, wx.ID_ANY, _("<empty>"))
+        self.lbl_pos_7 = wxStaticText(self.nb_rectangle, wx.ID_ANY, _("<empty>"))
         sizer_5a.Add(self.lbl_pos_7, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
         sizer_6a = wx.BoxSizer(wx.HORIZONTAL)
         sizer_rect_vert.Add(sizer_6a, 1, wx.EXPAND, 0)
 
-        label_corner_2 = wx.StaticText(self.nb_rectangle, wx.ID_ANY, _("Corner 2"))
+        label_corner_2 = wxStaticText(self.nb_rectangle, wx.ID_ANY, _("Corner 2"))
         sizer_6a.Add(label_corner_2, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_set_rect_2 = wx.Button(self.nb_rectangle, wx.ID_ANY, _("Use position"))
+        self.btn_set_rect_2 = wxButton(self.nb_rectangle, wx.ID_ANY, _("Use position"))
         self.btn_set_rect_2.SetToolTip(
             _("Place the laser over the desired point and click...")
         )
         sizer_6a.Add(self.btn_set_rect_2, 0, wx.EXPAND, 0)
 
-        self.lbl_pos_8 = wx.StaticText(self.nb_rectangle, wx.ID_ANY, _("<empty>"))
+        self.lbl_pos_8 = wxStaticText(self.nb_rectangle, wx.ID_ANY, _("<empty>"))
         sizer_6a.Add(self.lbl_pos_8, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.img_instruction_2 = wx.StaticBitmap(
-            self.nb_rectangle, wx.ID_ANY, instruction_frame.GetBitmap()
+        self.img_instruction_2 = wxStaticBitmap(
+            self.nb_rectangle,
+            wx.ID_ANY,
+            self.fitted_bitmap(icon_instruct_rect, desired_height),
         )
         instructions = _(
             "Instruction: place the laser on one corner of the encompassing rectangle and confirm the position by clicking on the buttons below. Then choose the opposing corner.\nMK will create a rectangle for you for further processing."
@@ -177,7 +193,7 @@ class LaserToolPanel(wx.Panel):
 
         sizer_chk_rect = wx.BoxSizer(wx.HORIZONTAL)
         self.sizer_rectangle.Add(sizer_chk_rect, 0, wx.EXPAND, 0)
-        self.check_ref_frame = wx.CheckBox(
+        self.check_ref_frame = wxCheckBox(
             self.nb_rectangle, wx.ID_ANY, _("Make reference")
         )
         sizer_chk_rect.Add(self.check_ref_frame, 0, wx.ALIGN_CENTER_VERTICAL, 0)
@@ -185,7 +201,7 @@ class LaserToolPanel(wx.Panel):
         sizer_8a = wx.BoxSizer(wx.HORIZONTAL)
         self.sizer_rectangle.Add(sizer_8a, 0, wx.EXPAND, 0)
 
-        self.btn_create_frame = wx.Button(
+        self.btn_create_frame = wxButton(
             self.nb_rectangle, wx.ID_ANY, _("Create frame")
         )
         sizer_8a.Add(self.btn_create_frame, 0, wx.EXPAND, 0)
@@ -206,64 +222,66 @@ class LaserToolPanel(wx.Panel):
         sizer_5 = wx.BoxSizer(wx.HORIZONTAL)
         sizer_sqare_vert.Add(sizer_5, 1, wx.EXPAND, 0)
 
-        label_4 = wx.StaticText(self.nb_square, wx.ID_ANY, _("Side A 1"))
-        label_4.SetMinSize((45, -1))
+        label_4 = wxStaticText(self.nb_square, wx.ID_ANY, _("Side A 1"))
+        label_4.SetMinSize(dip_size(self.nb_square, 45, -1))
         sizer_5.Add(label_4, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_set_square_1 = wx.Button(self.nb_square, wx.ID_ANY, _("Use position"))
+        self.btn_set_square_1 = wxButton(self.nb_square, wx.ID_ANY, _("Use position"))
         self.btn_set_square_1.SetToolTip(
             _("Place the laser over the desired point and click...")
         )
         sizer_5.Add(self.btn_set_square_1, 0, wx.EXPAND, 0)
 
-        self.lbl_pos_4 = wx.StaticText(self.nb_square, wx.ID_ANY, _("<empty>"))
+        self.lbl_pos_4 = wxStaticText(self.nb_square, wx.ID_ANY, _("<empty>"))
         sizer_5.Add(self.lbl_pos_4, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
         sizer_6 = wx.BoxSizer(wx.HORIZONTAL)
         sizer_sqare_vert.Add(sizer_6, 1, wx.EXPAND, 0)
 
-        label_5 = wx.StaticText(self.nb_square, wx.ID_ANY, _("Side A 2"))
-        label_5.SetMinSize((45, -1))
+        label_5 = wxStaticText(self.nb_square, wx.ID_ANY, _("Side A 2"))
+        label_5.SetMinSize(dip_size(self.nb_square, 45, -1))
         sizer_6.Add(label_5, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_set_square_2 = wx.Button(self.nb_square, wx.ID_ANY, _("Use position"))
+        self.btn_set_square_2 = wxButton(self.nb_square, wx.ID_ANY, _("Use position"))
         self.btn_set_square_2.SetToolTip(
             _("Place the laser over the desired point and click...")
         )
         sizer_6.Add(self.btn_set_square_2, 0, wx.EXPAND, 0)
 
-        self.lbl_pos_5 = wx.StaticText(self.nb_square, wx.ID_ANY, _("<empty>"))
+        self.lbl_pos_5 = wxStaticText(self.nb_square, wx.ID_ANY, _("<empty>"))
         sizer_6.Add(self.lbl_pos_5, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
         sizer_7 = wx.BoxSizer(wx.HORIZONTAL)
         sizer_sqare_vert.Add(sizer_7, 1, wx.EXPAND, 0)
 
-        label_6 = wx.StaticText(self.nb_square, wx.ID_ANY, _("Side B"))
-        label_6.SetMinSize((45, -1))
+        label_6 = wxStaticText(self.nb_square, wx.ID_ANY, _("Side B"))
+        label_6.SetMinSize(dip_size(self.nb_square, 45, -1))
         sizer_7.Add(label_6, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_set_square_3 = wx.Button(self.nb_square, wx.ID_ANY, _("Use position"))
+        self.btn_set_square_3 = wxButton(self.nb_square, wx.ID_ANY, _("Use position"))
         self.btn_set_square_3.SetToolTip(
             _("Place the laser over the desired point and click...")
         )
         sizer_7.Add(self.btn_set_square_3, 0, wx.EXPAND, 0)
 
-        self.lbl_pos_6 = wx.StaticText(self.nb_square, wx.ID_ANY, _("<empty>"))
+        self.lbl_pos_6 = wxStaticText(self.nb_square, wx.ID_ANY, _("<empty>"))
         sizer_7.Add(self.lbl_pos_6, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
         size_width = wx.BoxSizer(wx.HORIZONTAL)
         sizer_sqare_vert.Add(size_width, 1, wx.EXPAND, 0)
 
-        label_wd = wx.StaticText(self.nb_square, wx.ID_ANY, _("Dimension"))
+        label_wd = wxStaticText(self.nb_square, wx.ID_ANY, _("Dimension"))
         size_width.Add(label_wd, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.txt_width = wx.TextCtrl(self.nb_square, wx.ID_ANY, DEFAULT_LEN)
+        self.txt_width = TextCtrl(self.nb_square, wx.ID_ANY, DEFAULT_LEN)
         self.txt_width.SetToolTip(_("Extension of the square to create"))
-        self.txt_width.SetMinSize((60, -1))
+        self.txt_width.SetMinSize(dip_size(self.nb_square, 60, -1))
         size_width.Add(self.txt_width, 0, wx.EXPAND, 0)
 
-        self.img_instruction_3 = wx.StaticBitmap(
-            self.nb_square, wx.ID_ANY, instruction_rectangle.GetBitmap()
+        self.img_instruction_3 = wxStaticBitmap(
+            self.nb_square,
+            wx.ID_ANY,
+            self.fitted_bitmap(icon_instruct_square, desired_height),
         )
         instructions = _(
             "Instruction: place the laser on two points of one side of a square on the bed and confirm the position by clicking on the buttons below. Then choose one point on the other side of the corner.\nMK will create a square for you for further processing."
@@ -273,19 +291,17 @@ class LaserToolPanel(wx.Panel):
 
         sizer_chk_square = wx.BoxSizer(wx.HORIZONTAL)
         self.sizer_square.Add(sizer_chk_square, 0, wx.EXPAND, 0)
-        self.check_ref_square = wx.CheckBox(
+        self.check_ref_square = wxCheckBox(
             self.nb_square, wx.ID_ANY, _("Make reference")
         )
         sizer_chk_square.Add(self.check_ref_square, 0, wx.ALIGN_CENTER_VERTICAL, 0)
-        self.check_square = wx.CheckBox(self.nb_square, wx.ID_ANY, _("Mark Corner"))
+        self.check_square = wxCheckBox(self.nb_square, wx.ID_ANY, _("Mark Corner"))
         sizer_chk_square.Add(self.check_square, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
         sizer_8 = wx.BoxSizer(wx.HORIZONTAL)
         self.sizer_square.Add(sizer_8, 0, wx.EXPAND, 0)
 
-        self.btn_create_square = wx.Button(
-            self.nb_square, wx.ID_ANY, _("Create square")
-        )
+        self.btn_create_square = wxButton(self.nb_square, wx.ID_ANY, _("Create square"))
         sizer_8.Add(self.btn_create_square, 0, wx.EXPAND, 0)
 
         self.nb_square.SetSizer(self.sizer_square)
@@ -316,6 +332,18 @@ class LaserToolPanel(wx.Panel):
         # self.img_instruction_3.Bind(wx.EVT_LEFT_DCLICK, self.create_scenario)
         # end wxGlade
 
+    def fitted_bitmap(self, picture, available_height):
+        bmp = picture.GetBitmap()
+        bmp_width, bmp_height = bmp.Size
+        factor = int(4 * available_height / bmp_height) / 4
+        # print(f"Img: {factor}, Available:{available_height} vs ht={bmp_height}")
+        if factor > 1:
+            bmp_width *= factor
+            bmp_height *= factor
+        bmp_width *= self.context.root.bitmap_correction_scale
+        bmp_height *= self.context.root.bitmap_correction_scale
+        return picture.GetBitmap(resize=(bmp_width, bmp_height))
+
     # scenario = 8
     #
     # def create_scenario(self, event):
@@ -428,7 +456,6 @@ class LaserToolPanel(wx.Panel):
         2x_1a+2y_1b+c + x_1^2+y_1^2+=0\\2x_2a+2y_2b+c+x_2^2+y_2^2=0\\2x_3a+2y_3b+c+x_3^2+y_3^2=0
         """
         result = True
-        center = None
         radius = None
         x12 = self.coord_a[0] - self.coord_b[0]
         x13 = self.coord_a[0] - self.coord_c[0]
@@ -451,12 +478,12 @@ class LaserToolPanel(wx.Panel):
         sx21 = pow(self.coord_b[0], 2) - pow(self.coord_a[0], 2)
         sy21 = pow(self.coord_b[1], 2) - pow(self.coord_a[1], 2)
         try:
-            f = ((sx13) * (x12) + (sy13) * (x12) + (sx21) * (x13) + (sy21) * (x13)) // (
-                2 * ((y31) * (x12) - (y21) * (x13))
+            f = (sx13 * x12 + sy13 * x12 + sx21 * x13 + sy21 * x13) // (
+                2 * (y31 * x12 - y21 * x13)
             )
 
-            g = ((sx13) * (y12) + (sy13) * (y12) + (sx21) * (y13) + (sy21) * (y13)) // (
-                2 * ((x31) * (y12) - (x21) * (y13))
+            g = (sx13 * y12 + sy13 * y12 + sx21 * y13 + sy21 * y13) // (
+                2 * (x31 * y12 - x21 * y13)
             )
         except (ZeroDivisionError, ArithmeticError):
             return False, None, None
@@ -479,17 +506,15 @@ class LaserToolPanel(wx.Panel):
         if sqr_of_r < 0:
             result = False
         else:
-            r = round(sqrt(sqr_of_r), 5)
+            radius = round(sqrt(sqr_of_r), 5)
 
         # print("Centre = (", h, ", ", k, ")")
         # print("Radius = ", r)
         center = (h, k)
-        radius = r
         return result, center, radius
 
     def calculate_square(self):
         result = True
-        center = None
         angle = 0
         signx = 1
         signy = 1
@@ -574,6 +599,9 @@ class LaserToolPanel(wx.Panel):
 
     def calculate_frame(self):
         result = False
+        left_top = (0, 0)
+        width = 0
+        height = 0
         if self.coord_a is not None and self.coord_b is not None:
             x0 = self.coord_a[0]
             x1 = self.coord_b[0]
@@ -613,8 +641,8 @@ class LaserToolPanel(wx.Panel):
             if (
                 cx < 0
                 or cy < 0
-                or cx > self.context.device.unit_width
-                or cy > self.context.device.unit_height
+                or cx > self.context.device.view.unit_width
+                or cy > self.context.device.view.unit_height
             ):
                 message = (
                     _("The circles center seems to lie outside the bed-dimensions!")
@@ -625,7 +653,7 @@ class LaserToolPanel(wx.Panel):
                     + "\n"
                 )
                 message += _(
-                    "slam into the walls and get damaged! Do you really want to proeed?"
+                    "slam into the walls and get damaged! Do you really want to proceed?"
                 )
                 caption = _("Dangerous coordinates")
                 dlg = wx.MessageDialog(
@@ -743,6 +771,7 @@ class LaserToolPanel(wx.Panel):
 
     @signal_listener("driver;position")
     @signal_listener("emulator;position")
+    @signal_listener("status;position")
     def on_update_laser(self, origin, pos):
         self.laserposition = (pos[2], pos[3])
 
@@ -751,10 +780,12 @@ class LaserTool(MWindow):
     def __init__(self, *args, **kwds):
         super().__init__(551, 234, submenu="Operations", *args, **kwds)
         self.panel = LaserToolPanel(self, wx.ID_ANY, context=self.context)
+        self.sizer.Add(self.panel, 1, wx.EXPAND, 0)
         _icon = wx.NullIcon
-        # _icon.CopyFromBitmap(icons8_computer_support_50.GetBitmap())
+        # _icon.CopyFromBitmap(icons8_computer_support.GetBitmap())
         self.SetIcon(_icon)
-        self.SetTitle(_("Laser-Tools"))
+        self.SetTitle(_("Place Template"))
+        self.restore_aspect()
 
     def window_open(self):
         pass
@@ -767,4 +798,9 @@ class LaserTool(MWindow):
 
     @staticmethod
     def submenu():
-        return ("Laser-Tools", "Laser-Tools")
+        # Hint for translation: _("Laser-Tools"), _("Place Template")
+        return "Laser-Tools", "Place Template"
+
+    @staticmethod
+    def helptext():
+        return _("Define and use some templates for repetitive laser jobs")
