@@ -24,12 +24,45 @@ class MWindow(wx.Frame, Module):
                 style = (
                     wx.DEFAULT_FRAME_STYLE | wx.FRAME_FLOAT_ON_PARENT | wx.TAB_TRAVERSAL
                 )
-        wx.Frame.__init__(self, parent, style=style)
-        Module.__init__(self, context, path)
+        self._start_width = width
+        self._start_height = height
         self.root_context = context.root
         self.window_context = context.get_context(path)
         self.root_context.setting(bool, "windows_save", True)
         self.window_save = self.root_context.windows_save
+
+        wx.Frame.__init__(self, parent, style=style)
+        Module.__init__(self, context, path)
+        self.window_context.themes.set_window_colors(self)
+        self.Bind(wx.EVT_CLOSE, self.on_close, self)
+        self.Bind(wx.EVT_LEFT_DOWN, self.on_mouse_left_down, self)
+        self.Bind(wx.EVT_RIGHT_DOWN, self.on_menu_request, self)
+        self.Bind(wx.EVT_MOVE, self.on_change_window, self)
+        self.Bind(wx.EVT_SIZE, self.on_change_window, self)
+        self.sizer = wx.BoxSizer(wx.VERTICAL)
+        self.SetSizer(self.sizer)
+        self.context.themes.set_window_colors(self)
+
+    def restore_aspect(self, honor_initial_values=False):
+        width = self._start_width
+        height = self._start_height
+        try:
+            dipsize = self.FromDIP(wx.Size(width, height))
+            d_width = dipsize[0]
+            d_height = dipsize[1]
+        except AttributeError:
+            d_width = width
+            d_height = height
+        if honor_initial_values:
+            width = d_width
+            height = d_height
+        else:
+            # Forget about a default size, try to fit
+            # the content as good as you can.
+            self.sizer.Layout()
+            self.sizer.Fit(self)
+            width, height = self.GetSize()
+
         if self.window_save:
             self.window_context.setting(int, "width", width)
             self.window_context.setting(int, "height", height)
@@ -37,14 +70,17 @@ class MWindow(wx.Frame, Module):
                 self.window_context.width = 100
             if self.window_context.height < 100:
                 self.window_context.height = 100
+            display_idx = wx.Display.GetFromWindow(self)
+            if display_idx != wx.NOT_FOUND:
+                area = wx.Display(display_idx).GetClientArea()
+                if self.window_context.width >= area[2]:
+                    self.window_context.width = area[2]
+                    self.window_context.setting(int, "x", 0)
+                if self.window_context.height >= area[3]:
+                    self.window_context.height = area[3]
+                    self.window_context.setting(int, "y", 0)
             self.SetSize((self.window_context.width, self.window_context.height))
-        else:
-            self.SetSize(width, height)
-        self.Bind(wx.EVT_CLOSE, self.on_close, self)
-        self.Bind(wx.EVT_LEFT_DOWN, self.on_mouse_left_down, self)
-        self.Bind(wx.EVT_RIGHT_DOWN, self.on_menu_request, self)
-        self.Bind(wx.EVT_MOVE, self.on_change_window, self)
-        self.Bind(wx.EVT_SIZE, self.on_change_window, self)
+        self.Raise()
 
     def on_mouse_left_down(self, event):
         # Convert mac Control+left click into right click
@@ -85,7 +121,11 @@ class MWindow(wx.Frame, Module):
             event.Skip()  # Call 'destroy' as regular.
 
     def window_open(self):
-        pass
+        self.window_context.themes.set_window_colors(self)
+
+    def restore(self, *args, **kwargs):
+        if self.IsIconized():
+            self.Iconize(False)
 
     def window_close(self):
         pass
@@ -107,6 +147,7 @@ class MWindow(wx.Frame, Module):
             if display == wx.NOT_FOUND:
                 self.SetPosition((x, y))
         self.Show()
+        self.Raise()
         self.window_open()
 
     def module_close(self, *args, shutdown=False, **kwargs):
