@@ -1,8 +1,9 @@
 import wx
 from wx import aui
 
-from .icons import STD_ICON_SIZE, icons8_comments_50
+from .icons import STD_ICON_SIZE, icons8_comments
 from .mwindow import MWindow
+from .wxutils import wxCheckBox, TextCtrl
 
 _ = wx.GetTranslation
 
@@ -23,7 +24,8 @@ def register_panel(window, context):
     pane.dock_proportion = 100
     pane.control = panel
     pane.submenu = "_50_" + _("Tools")
-
+    pane.helptext = _("Edit job notes")
+ 
     window.on_pane_create(pane)
     context.register("pane/notes", pane)
 
@@ -33,12 +35,14 @@ class NotePanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
+        self.SetHelpText("notes")
         self.pane = pane
         if not self.pane:
-            self.check_auto_open_notes = wx.CheckBox(
+            self.check_auto_open_notes = wxCheckBox(
                 self, wx.ID_ANY, _("Automatically Open Notes")
             )
-        self.text_notes = wx.TextCtrl(
+        self.text_notes = TextCtrl(
             self,
             wx.ID_ANY,
             "",
@@ -110,29 +114,21 @@ class NotePanel(wx.Panel):
 
 class Notes(MWindow):
     def __init__(self, *args, **kwds):
-        super().__init__(730, 621, *args, **kwds)
+        super().__init__(450, 350, *args, **kwds)
 
         self.panel = NotePanel(self, wx.ID_ANY, context=self.context)
+        self.sizer.Add(self.panel, 1, wx.EXPAND, 0)
         self.add_module_delegate(self.panel)
         _icon = wx.NullIcon
-        _icon.CopyFromBitmap(icons8_comments_50.GetBitmap())
+        _icon.CopyFromBitmap(icons8_comments.GetBitmap())
         self.SetIcon(_icon)
         self.SetTitle(_("Notes"))
         self.Children[0].SetFocus()
+        self.restore_aspect(honor_initial_values=True)
 
     @staticmethod
     def sub_register(kernel):
         kernel.register("wxpane/Notes", register_panel)
-        kernel.register(
-            "button/project/Notes",
-            {
-                "label": _("Notes"),
-                "icon": icons8_comments_50,
-                "tip": _("Open Notes Window"),
-                "action": lambda v: kernel.console("window toggle Notes\n"),
-                "size": STD_ICON_SIZE,
-            },
-        )
 
     def window_open(self):
         self.context.close(self.name)
@@ -140,3 +136,8 @@ class Notes(MWindow):
 
     def window_close(self):
         self.panel.pane_hide()
+
+    @staticmethod
+    def helptext():
+        return _("Edit job notes")
+    
\ No newline at end of file
