@@ -3,41 +3,59 @@ from copy import deepcopy
 import wx
 
 from meerk40t.core.node.elem_image import ImageNode
+from meerk40t.gui.wxutils import StaticBoxSizer, TextCtrl, dip_size, wxButton, wxCheckBox
 
 _ = wx.GetTranslation
 
+"""
+Establishes property panels for some image operations
+The priority of appearance is defined dynamically based
+on the order within the node operations
 
+"""
 class ContrastPanel(wx.Panel):
     name = _("Contrast")
-    priority = 10
+
+    @staticmethod
+    def priority(node):
+        base_priority = 20
+        prop = "contrast"
+        if hasattr(node, "as_image"):
+            for idx, n in enumerate(node.operations):
+                if n.get("name") == prop:
+                    return base_priority + idx + 1
+        return base_priority
 
     @staticmethod
     def accepts(node):
-        if node.type != "elem image":
+        if not hasattr(node, "as_image"):
             return False
-        for n in node.operations:
-            if n.get("name") == "contrast":
-                return True
-        return False
+        # Show it for all images...
+        return True
+        # for n in node.operations:
+        #     if n.get("name") == "contrast":
+        #         return True
+        # return False
 
     def __init__(self, *args, context=None, node=None, **kwds):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
         self.node = node
 
-        self.check_enable_contrast = wx.CheckBox(self, wx.ID_ANY, _("Enable"))
-        self.button_reset_contrast = wx.Button(self, wx.ID_ANY, _("Reset"))
+        self.check_enable_contrast = wxCheckBox(self, wx.ID_ANY, _("Enable"))
+        self.button_reset_contrast = wxButton(self, wx.ID_ANY, _("Reset"))
         self.slider_contrast_contrast = wx.Slider(
             self, wx.ID_ANY, 0, -127, 127, style=wx.SL_AUTOTICKS | wx.SL_HORIZONTAL
         )
-        self.text_contrast_contrast = wx.TextCtrl(
+        self.text_contrast_contrast = TextCtrl(
             self, wx.ID_ANY, "", style=wx.TE_READONLY
         )
         self.slider_contrast_brightness = wx.Slider(
             self, wx.ID_ANY, 0, -127, 127, style=wx.SL_AUTOTICKS | wx.SL_HORIZONTAL
         )
-        self.text_contrast_brightness = wx.TextCtrl(
+        self.text_contrast_brightness = TextCtrl(
             self, wx.ID_ANY, "", style=wx.TE_READONLY
         )
 
@@ -67,14 +85,21 @@ class ContrastPanel(wx.Panel):
             if n.get("name") == "contrast":
                 op = n
         if op is None:
-            raise ValueError
+            op = {
+                    "name": "contrast",
+                    "enable": False,
+                    "contrast": 0,
+                    "brightness": 0,
+                }
+            node.operations.append(op)
+            # raise ValueError
         self.op = op
         self.original_op = deepcopy(op)
         self.check_enable_contrast.SetValue(self.op["enable"])
         self.text_contrast_contrast.SetValue(str(self.op["contrast"]))
         self.text_contrast_brightness.SetValue(str(self.op["brightness"]))
-        self.slider_contrast_contrast.SetValue(self.op["contrast"])
-        self.slider_contrast_brightness.SetValue(self.op["brightness"])
+        self.slider_contrast_contrast.SetValue(int(self.op["contrast"]))
+        self.slider_contrast_brightness.SetValue(int(self.op["brightness"]))
 
     def __set_properties(self):
         # begin wxGlade: ContrastPanel.__set_properties
@@ -93,14 +118,12 @@ class ContrastPanel(wx.Panel):
 
     def __do_layout(self):
         # begin wxGlade: ContrastPanel.__do_layout
-        sizer_contrast = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Contrast")), wx.VERTICAL
+        sizer_contrast = StaticBoxSizer(self, wx.ID_ANY, _("Contrast"), wx.VERTICAL)
+        sizer_contrast_brightness = StaticBoxSizer(
+            self, wx.ID_ANY, _("Brightness Amount"), wx.HORIZONTAL
         )
-        sizer_contrast_brightness = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Brightness Amount")), wx.HORIZONTAL
-        )
-        sizer_contrast_contrast = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Contrast Amount")), wx.HORIZONTAL
+        sizer_contrast_contrast = StaticBoxSizer(
+            self, wx.ID_ANY, _("Contrast Amount"), wx.HORIZONTAL
         )
         sizer_contrast_main = wx.BoxSizer(wx.HORIZONTAL)
         sizer_contrast_main.Add(self.check_enable_contrast, 0, 0, 0)
@@ -121,7 +144,7 @@ class ContrastPanel(wx.Panel):
         self, event=None
     ):  # wxGlade: ContrastPanel.<event_handler>
         self.op["enable"] = self.check_enable_contrast.GetValue()
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_button_reset_contrast(
         self, event=None
@@ -132,30 +155,39 @@ class ContrastPanel(wx.Panel):
         self.text_contrast_brightness.SetValue(str(self.op["brightness"]))
         self.slider_contrast_contrast.SetValue(self.op["contrast"])
         self.slider_contrast_brightness.SetValue(self.op["brightness"])
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_slider_contrast_contrast(
         self, event=None
     ):  # wxGlade: ContrastPanel.<event_handler>
         self.op["contrast"] = int(self.slider_contrast_contrast.GetValue())
         self.text_contrast_contrast.SetValue(str(self.op["contrast"]))
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_slider_contrast_brightness(
         self, event=None
     ):  # wxGlade: ContrastPanel.<event_handler>
         self.op["brightness"] = int(self.slider_contrast_brightness.GetValue())
         self.text_contrast_brightness.SetValue(str(self.op["brightness"]))
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
 
 class HalftonePanel(wx.Panel):
     name = _("Halftone")
-    priority = 15
+
+    @staticmethod
+    def priority(node):
+        base_priority = 20
+        prop = "halftone"
+        if hasattr(node, "as_image"):
+            for idx, n in enumerate(node.operations):
+                if n.get("name") == prop:
+                    return base_priority + idx + 1
+        return base_priority
 
     @staticmethod
     def accepts(node):
-        if node.type != "elem image":
+        if not hasattr(node, "as_image"):
             return False
         for n in node.operations:
             if n.get("name") == "halftone":
@@ -166,27 +198,28 @@ class HalftonePanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
         self.node = node
 
-        self.check_enable_halftone = wx.CheckBox(self, wx.ID_ANY, "Enable")
-        self.button_reset_halftone = wx.Button(self, wx.ID_ANY, "Reset")
-        self.check_halftone_black = wx.CheckBox(self, wx.ID_ANY, "Black")
+        self.check_enable_halftone = wxCheckBox(self, wx.ID_ANY, "Enable")
+        self.button_reset_halftone = wxButton(self, wx.ID_ANY, "Reset")
+        self.check_halftone_black = wxCheckBox(self, wx.ID_ANY, "Black")
         self.slider_halftone_sample = wx.Slider(
             self, wx.ID_ANY, 10, 0, 50, style=wx.SL_AUTOTICKS | wx.SL_HORIZONTAL
         )
-        self.text_halftone_sample = wx.TextCtrl(
+        self.text_halftone_sample = TextCtrl(
             self, wx.ID_ANY, "", style=wx.TE_READONLY
         )
         self.slider_halftone_angle = wx.Slider(
             self, wx.ID_ANY, 22, 0, 90, style=wx.SL_AUTOTICKS | wx.SL_HORIZONTAL
         )
-        self.text_halftone_angle = wx.TextCtrl(
+        self.text_halftone_angle = TextCtrl(
             self, wx.ID_ANY, "", style=wx.TE_READONLY
         )
         self.slider_halftone_oversample = wx.Slider(
             self, wx.ID_ANY, 2, 0, 50, style=wx.SL_AUTOTICKS | wx.SL_HORIZONTAL
         )
-        self.text_halftone_oversample = wx.TextCtrl(
+        self.text_halftone_oversample = TextCtrl(
             self, wx.ID_ANY, "", style=wx.TE_READONLY
         )
 
@@ -225,11 +258,11 @@ class HalftonePanel(wx.Panel):
         self.check_enable_halftone.SetValue(self.op["enable"])
         self.check_halftone_black.SetValue(self.op["black"])
         self.text_halftone_sample.SetValue(str(self.op["sample"]))
-        self.slider_halftone_sample.SetValue(self.op["sample"])
+        self.slider_halftone_sample.SetValue(int(self.op["sample"]))
         self.text_halftone_angle.SetValue(str(self.op["angle"]))
-        self.slider_halftone_angle.SetValue(self.op["angle"])
+        self.slider_halftone_angle.SetValue(int(self.op["angle"]))
         self.text_halftone_oversample.SetValue(str(self.op["oversample"]))
-        self.slider_halftone_oversample.SetValue(self.op["oversample"])
+        self.slider_halftone_oversample.SetValue(int(self.op["oversample"]))
 
     def __set_properties(self):
         # begin wxGlade: HalftonePanel.__set_properties
@@ -249,17 +282,15 @@ class HalftonePanel(wx.Panel):
 
     def __do_layout(self):
         # begin wxGlade: HalftonePanel.__do_layout
-        sizer_halftone = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Halftone")), wx.VERTICAL
+        sizer_halftone = StaticBoxSizer(self, wx.ID_ANY, _("Halftone"), wx.VERTICAL)
+        sizer_halftone_oversample = StaticBoxSizer(
+            self, wx.ID_ANY, _("Oversample"), wx.HORIZONTAL
         )
-        sizer_halftone_oversample = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Oversample")), wx.HORIZONTAL
+        sizer_halftone_angle = StaticBoxSizer(
+            self, wx.ID_ANY, _("Angle"), wx.HORIZONTAL
         )
-        sizer_halftone_angle = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Angle")), wx.HORIZONTAL
-        )
-        sizer_halftone_sample = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Sample")), wx.HORIZONTAL
+        sizer_halftone_sample = StaticBoxSizer(
+            self, wx.ID_ANY, _("Sample"), wx.HORIZONTAL
         )
         sizer_halftone_main = wx.BoxSizer(wx.HORIZONTAL)
         sizer_halftone_main.Add(self.check_enable_halftone, 0, 0, 0)
@@ -285,7 +316,7 @@ class HalftonePanel(wx.Panel):
         self, event=None
     ):  # wxGlade: HalftonePanel.<event_handler>
         self.op["enable"] = self.check_enable_halftone.GetValue()
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_button_reset_halftone(
         self, event=None
@@ -297,48 +328,57 @@ class HalftonePanel(wx.Panel):
         self.check_enable_halftone.SetValue(self.op["enable"])
         self.check_halftone_black.SetValue(self.op["black"])
         self.text_halftone_sample.SetValue(str(self.op["sample"]))
-        self.slider_halftone_sample.SetValue(self.op["sample"])
+        self.slider_halftone_sample.SetValue(int(self.op["sample"]))
         self.text_halftone_angle.SetValue(str(self.op["angle"]))
-        self.slider_halftone_angle.SetValue(self.op["angle"])
+        self.slider_halftone_angle.SetValue(int(self.op["angle"]))
         self.text_halftone_oversample.SetValue(str(self.op["oversample"]))
-        self.slider_halftone_oversample.SetValue(self.op["oversample"])
-        self.node.update(self.context)
+        self.slider_halftone_oversample.SetValue(int(self.op["oversample"]))
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_check_halftone_black(
         self, event=None
     ):  # wxGlade: HalftonePanel.<event_handler>
         self.op["black"] = self.check_halftone_black.GetValue()
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_slider_halftone_sample(
         self, event=None
     ):  # wxGlade: HalftonePanel.<event_handler>
         self.op["sample"] = int(self.slider_halftone_sample.GetValue())
         self.text_halftone_sample.SetValue(str(self.op["sample"]))
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_slider_halftone_angle(
         self, event=None
     ):  # wxGlade: HalftonePanel.<event_handler>
         self.op["angle"] = int(self.slider_halftone_angle.GetValue())
         self.text_halftone_angle.SetValue(str(self.op["angle"]))
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_slider_halftone_oversample(
         self, event=None
     ):  # wxGlade: HalftonePanel.<event_handler>
         self.op["oversample"] = int(self.slider_halftone_oversample.GetValue())
         self.text_halftone_oversample.SetValue(str(self.op["oversample"]))
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
 
 class ToneCurvePanel(wx.Panel):
     name = _("Tone")
-    priority = 20
+
+    @staticmethod
+    def priority(node):
+        base_priority = 20
+        prop = "tone"
+        if hasattr(node, "as_image"):
+            for idx, n in enumerate(node.operations):
+                if n.get("name") == prop:
+                    return base_priority + idx + 1
+        return base_priority
 
     @staticmethod
     def accepts(node):
-        if node.type != "elem image":
+        if not hasattr(node, "as_image"):
             return False
         for n in node.operations:
             if n.get("name") == "tone":
@@ -349,11 +389,12 @@ class ToneCurvePanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
         self.node = node
 
         self._tone_panel_buffer = None
-        self.check_enable_tone = wx.CheckBox(self, wx.ID_ANY, _("Enable"))
-        self.button_reset_tone = wx.Button(self, wx.ID_ANY, _("Reset"))
+        self.check_enable_tone = wxCheckBox(self, wx.ID_ANY, _("Enable"))
+        self.button_reset_tone = wxButton(self, wx.ID_ANY, _("Reset"))
         self.curve_panel = wx.Panel(self, wx.ID_ANY)
 
         self.__set_properties()
@@ -372,7 +413,7 @@ class ToneCurvePanel(wx.Panel):
 
         self.graph_brush = wx.Brush()
         self.graph_pen = wx.Pen()
-        c = wx.SystemSettings().GetColour(wx.SYS_COLOUR_WINDOW)
+        c = self.context.themes.get("win_bg")
         self.graph_brush.SetColour(c)
         self.graph_pen.SetColour(wx.Colour(255 - c[0], 255 - c[1], 255 - c[2]))
         self.graph_pen.SetWidth(5)
@@ -400,15 +441,13 @@ class ToneCurvePanel(wx.Panel):
         self.check_enable_tone.SetToolTip(_("Enable Tone Curve"))
         self.check_enable_tone.SetValue(1)
         self.button_reset_tone.SetToolTip(_("Reset Tone Curve"))
-        self.curve_panel.SetMinSize((256, 256))
-        self.curve_panel.SetMaxSize((256, 256))
+        self.curve_panel.SetMinSize(dip_size(self, 256, 256))
+        self.curve_panel.SetMaxSize(dip_size(self, 256, 256))
         # end wxGlade
 
     def __do_layout(self):
         # begin wxGlade: ToneCurvePanel.__do_layout
-        sizer_tone = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Tone Curve")), wx.VERTICAL
-        )
+        sizer_tone = StaticBoxSizer(self, wx.ID_ANY, _("Tone Curve"), wx.VERTICAL)
         sizer_tone_curve = wx.BoxSizer(wx.HORIZONTAL)
         sizer_tone_curve.Add(self.check_enable_tone, 0, 0, 0)
         sizer_tone_curve.Add(self.button_reset_tone, 0, 0, 0)
@@ -444,7 +483,7 @@ class ToneCurvePanel(wx.Panel):
                         self.op["values"][pos[0]] = (pos[0], v)
                 else:
                     self.op["values"][self.point] = (pos[0], v)
-                self.node.update(self.context)
+                self.context.elements.do_image_update(self.node, self.context)
                 self.update_in_gui_thread()
             except (KeyError, IndexError):
                 pass
@@ -504,26 +543,37 @@ class ToneCurvePanel(wx.Panel):
 
     def on_check_enable_tone(self, event=None):  # wxGlade: RasterWizard.<event_handler>
         self.op["enable"] = self.check_enable_tone.GetValue()
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_button_reset_tone(self, event=None):  # wxGlade: RasterWizard.<event_handler>
         self.op["enable"] = self.original_op["enable"]
         self.op["type"] = self.original_op["type"]
         self.op["values"].clear()
         self.op["values"].extend(self.original_op["values"])
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
         self.on_update_tone()
         self.update_in_gui_thread()
 
 
 class SharpenPanel(wx.Panel):
     name = _("Sharpen")
-    priority = 25
+
+    @staticmethod
+    def priority(node):
+        base_priority = 20
+        prop = "unsharp_mask"
+        if hasattr(node, "as_image"):
+            for idx, n in enumerate(node.operations):
+                if n.get("name") == prop:
+                    return base_priority + idx + 1
+        return base_priority
 
     @staticmethod
     def accepts(node):
-        if node.type != "elem image":
+        if not hasattr(node, "as_image"):
             return False
+        if node.operations is None:
+            node.operations = list()
         for n in node.operations:
             if n.get("name") == "unsharp_mask":
                 return True
@@ -533,26 +583,27 @@ class SharpenPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
         self.node = node
 
-        self.check_enable_sharpen = wx.CheckBox(self, wx.ID_ANY, _("Enable"))
-        self.button_reset_sharpen = wx.Button(self, wx.ID_ANY, _("Reset"))
+        self.check_enable_sharpen = wxCheckBox(self, wx.ID_ANY, _("Enable"))
+        self.button_reset_sharpen = wxButton(self, wx.ID_ANY, _("Reset"))
         self.slider_sharpen_percent = wx.Slider(
             self, wx.ID_ANY, 500, 0, 1000, style=wx.SL_AUTOTICKS | wx.SL_HORIZONTAL
         )
-        self.text_sharpen_percent = wx.TextCtrl(
+        self.text_sharpen_percent = TextCtrl(
             self, wx.ID_ANY, "", style=wx.TE_READONLY
         )
         self.slider_sharpen_radius = wx.Slider(
             self, wx.ID_ANY, 20, 0, 50, style=wx.SL_AUTOTICKS | wx.SL_HORIZONTAL
         )
-        self.text_sharpen_radius = wx.TextCtrl(
+        self.text_sharpen_radius = TextCtrl(
             self, wx.ID_ANY, "", style=wx.TE_READONLY
         )
         self.slider_sharpen_threshold = wx.Slider(
             self, wx.ID_ANY, 6, 0, 50, style=wx.SL_AUTOTICKS | wx.SL_HORIZONTAL
         )
-        self.text_sharpen_threshold = wx.TextCtrl(
+        self.text_sharpen_threshold = TextCtrl(
             self, wx.ID_ANY, "", style=wx.TE_READONLY
         )
 
@@ -592,9 +643,9 @@ class SharpenPanel(wx.Panel):
         self.op = op
         self.original_op = deepcopy(op)
         self.check_enable_sharpen.SetValue(op["enable"])
-        self.slider_sharpen_percent.SetValue(op["percent"])
-        self.slider_sharpen_radius.SetValue(op["radius"])
-        self.slider_sharpen_threshold.SetValue(op["threshold"])
+        self.slider_sharpen_percent.SetValue(int(op["percent"]))
+        self.slider_sharpen_radius.SetValue(int(op["radius"]))
+        self.slider_sharpen_threshold.SetValue(int(op["threshold"]))
         self.text_sharpen_percent.SetValue(str(op["percent"]))
         self.text_sharpen_radius.SetValue(str(op["radius"]))
         self.text_sharpen_threshold.SetValue(str(op["threshold"]))
@@ -618,17 +669,15 @@ class SharpenPanel(wx.Panel):
 
     def __do_layout(self):
         # begin wxGlade: SharpenPanel.__do_layout
-        sizer_sharpen = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Sharpen")), wx.VERTICAL
+        sizer_sharpen = StaticBoxSizer(self, wx.ID_ANY, _("Sharpen"), wx.VERTICAL)
+        sizer_sharpen_threshold = StaticBoxSizer(
+            self, wx.ID_ANY, _("Threshold"), wx.HORIZONTAL
         )
-        sizer_sharpen_threshold = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Threshold")), wx.HORIZONTAL
+        sizer_sharpen_radius = StaticBoxSizer(
+            self, wx.ID_ANY, _("Radius"), wx.HORIZONTAL
         )
-        sizer_sharpen_radius = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Radius")), wx.HORIZONTAL
-        )
-        sizer_sharpen_percent = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Percent")), wx.HORIZONTAL
+        sizer_sharpen_percent = StaticBoxSizer(
+            self, wx.ID_ANY, _("Percent"), wx.HORIZONTAL
         )
         sizer_sharpen_main = wx.BoxSizer(wx.HORIZONTAL)
         sizer_sharpen_main.Add(self.check_enable_sharpen, 0, 0, 0)
@@ -652,7 +701,7 @@ class SharpenPanel(wx.Panel):
         self, event=None
     ):  # wxGlade: RasterWizard.<event_handler>
         self.op["enable"] = self.check_enable_sharpen.GetValue()
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_button_reset_sharpen(
         self, event=None
@@ -660,20 +709,20 @@ class SharpenPanel(wx.Panel):
         self.op["percent"] = self.original_op["percent"]
         self.op["radius"] = self.original_op["radius"]
         self.op["threshold"] = self.original_op["threshold"]
-        self.slider_sharpen_percent.SetValue(self.op["percent"])
-        self.slider_sharpen_radius.SetValue(self.op["radius"])
-        self.slider_sharpen_threshold.SetValue(self.op["threshold"])
+        self.slider_sharpen_percent.SetValue(int(self.op["percent"]))
+        self.slider_sharpen_radius.SetValue(int(self.op["radius"]))
+        self.slider_sharpen_threshold.SetValue(int(self.op["threshold"]))
         self.text_sharpen_percent.SetValue(str(self.op["percent"]))
         self.text_sharpen_radius.SetValue(str(self.op["radius"]))
         self.text_sharpen_threshold.SetValue(str(self.op["threshold"]))
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_slider_sharpen_percent(
         self, event=None
     ):  # wxGlade: RasterWizard.<event_handler>
         self.op["percent"] = int(self.slider_sharpen_percent.GetValue())
         self.text_sharpen_percent.SetValue(str(self.op["percent"]))
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_text_sharpen_percent(
         self, event=None
@@ -683,7 +732,7 @@ class SharpenPanel(wx.Panel):
     def on_slider_sharpen_radius(self, event):  # wxGlade: RasterWizard.<event_handler>
         self.op["radius"] = int(self.slider_sharpen_radius.GetValue())
         self.text_sharpen_radius.SetValue(str(self.op["radius"]))
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_text_sharpen_radius(
         self, event=None
@@ -695,7 +744,7 @@ class SharpenPanel(wx.Panel):
     ):  # wxGlade: RasterWizard.<event_handler>
         self.op["threshold"] = int(self.slider_sharpen_threshold.GetValue())
         self.text_sharpen_threshold.SetValue(str(self.op["threshold"]))
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_text_sharpen_threshold(self, event):  # wxGlade: RasterWizard.<event_handler>
         pass
@@ -703,11 +752,20 @@ class SharpenPanel(wx.Panel):
 
 class GammaPanel(wx.Panel):
     name = _("Gamma")
-    priority = 30
+
+    @staticmethod
+    def priority(node):
+        base_priority = 20
+        prop = "gamma"
+        if hasattr(node, "as_image"):
+            for idx, n in enumerate(node.operations):
+                if n.get("name") == prop:
+                    return base_priority + idx + 1
+        return base_priority
 
     @staticmethod
     def accepts(node):
-        if node.type != "elem image":
+        if not hasattr(node, "as_image"):
             return False
         for n in node.operations:
             if n.get("name") == "gamma":
@@ -718,14 +776,15 @@ class GammaPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
         self.node = node
 
-        self.check_enable_gamma = wx.CheckBox(self, wx.ID_ANY, _("Enable"))
-        self.button_reset_gamma = wx.Button(self, wx.ID_ANY, _("Reset"))
+        self.check_enable_gamma = wxCheckBox(self, wx.ID_ANY, _("Enable"))
+        self.button_reset_gamma = wxButton(self, wx.ID_ANY, _("Reset"))
         self.slider_gamma_factor = wx.Slider(
             self, wx.ID_ANY, 100, 0, 500, style=wx.SL_AUTOTICKS | wx.SL_HORIZONTAL
         )
-        self.text_gamma_factor = wx.TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
+        self.text_gamma_factor = TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
 
         self.__set_properties()
         self.__do_layout()
@@ -746,7 +805,7 @@ class GammaPanel(wx.Panel):
         self.op = op
         self.original_op = deepcopy(op)
         self.text_gamma_factor.SetValue(str(op["factor"]))
-        self.slider_gamma_factor.SetValue(op["factor"] * 100.0)
+        self.slider_gamma_factor.SetValue(int(op["factor"] * 100.0))
         self.check_enable_gamma.SetValue(op["enable"])
 
     def __set_properties(self):
@@ -760,11 +819,9 @@ class GammaPanel(wx.Panel):
 
     def __do_layout(self):
         # begin wxGlade: GammaPanel.__do_layout
-        sizer_gamma = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Gamma")), wx.VERTICAL
-        )
-        sizer_gamma_factor = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Gamma Factor")), wx.HORIZONTAL
+        sizer_gamma = StaticBoxSizer(self, wx.ID_ANY, _("Gamma"), wx.VERTICAL)
+        sizer_gamma_factor = StaticBoxSizer(
+            self, wx.ID_ANY, _("Gamma Factor"), wx.HORIZONTAL
         )
         sizer_gamma_main = wx.BoxSizer(wx.HORIZONTAL)
         sizer_gamma_main.Add(self.check_enable_gamma, 0, 0, 0)
@@ -782,22 +839,22 @@ class GammaPanel(wx.Panel):
         self, event=None
     ):  # wxGlade: RasterWizard.<event_handler>
         self.op["enable"] = self.check_enable_gamma.GetValue()
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_button_reset_gamma(
         self, event=None
     ):  # wxGlade: RasterWizard.<event_handler>
         self.op["factor"] = self.original_op["factor"]
-        self.slider_gamma_factor.SetValue(self.op["factor"] * 100.0)
+        self.slider_gamma_factor.SetValue(int(self.op["factor"] * 100.0))
         self.text_gamma_factor.SetValue(str(self.op["factor"]))
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_slider_gamma_factor(
         self, event=None
     ):  # wxGlade: RasterWizard.<event_handler>
         self.op["factor"] = self.slider_gamma_factor.GetValue() / 100.0
         self.text_gamma_factor.SetValue(str(self.op["factor"]))
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
     def on_text_gamma_factor(self, event=None):  # wxGlade: RasterWizard.<event_handler>
         pass
@@ -805,11 +862,20 @@ class GammaPanel(wx.Panel):
 
 class EdgePanel(wx.Panel):
     name = _("Edge Enhance")
-    priority = 40
+
+    @staticmethod
+    def priority(node):
+        base_priority = 20
+        prop = "edge_enhance"
+        if hasattr(node, "as_image"):
+            for idx, n in enumerate(node.operations):
+                if n.get("name") == prop:
+                    return base_priority + idx + 1
+        return base_priority
 
     @staticmethod
     def accepts(node):
-        if node.type != "elem image":
+        if not hasattr(node, "as_image"):
             return False
         for n in node.operations:
             if n.get("name") == "edge_enhance":
@@ -820,9 +886,10 @@ class EdgePanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
         self.node = node
 
-        self.check_enable = wx.CheckBox(self, wx.ID_ANY, _("Enable"))
+        self.check_enable = wxCheckBox(self, wx.ID_ANY, _("Enable"))
 
         self.__set_properties()
         self.__do_layout()
@@ -849,9 +916,7 @@ class EdgePanel(wx.Panel):
 
     def __do_layout(self):
         # begin wxGlade: OutputPanel.__do_layout
-        sizer_output = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Enable")), wx.VERTICAL
-        )
+        sizer_output = StaticBoxSizer(self, wx.ID_ANY, _("Enable"), wx.VERTICAL)
         sizer_output.Add(self.check_enable, 0, 0, 0)
         self.SetSizer(sizer_output)
         sizer_output.Fit(self)
@@ -860,16 +925,25 @@ class EdgePanel(wx.Panel):
 
     def on_check_enable(self, event=None):
         self.op["enable"] = self.check_enable.GetValue()
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
 
 
 class AutoContrastPanel(wx.Panel):
     name = _("Auto Contrast")
-    priority = 45
+    @staticmethod
+
+    def priority(node):
+        base_priority = 20
+        prop = "auto_contrast"
+        if hasattr(node, "as_image"):
+            for idx, n in enumerate(node.operations):
+                if n.get("name") == prop:
+                    return base_priority + idx + 1
+        return base_priority
 
     @staticmethod
     def accepts(node):
-        if node.type != "elem image":
+        if not hasattr(node, "as_image"):
             return False
         for n in node.operations:
             if n.get("name") == "auto_contrast":
@@ -880,9 +954,10 @@ class AutoContrastPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
         self.node = node
 
-        self.check_enable = wx.CheckBox(self, wx.ID_ANY, _("Enable"))
+        self.check_enable = wxCheckBox(self, wx.ID_ANY, _("Enable"))
 
         self.__set_properties()
         self.__do_layout()
@@ -909,9 +984,7 @@ class AutoContrastPanel(wx.Panel):
 
     def __do_layout(self):
         # begin wxGlade: OutputPanel.__do_layout
-        sizer_output = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Enable")), wx.VERTICAL
-        )
+        sizer_output = StaticBoxSizer(self, wx.ID_ANY, _("Enable"), wx.VERTICAL)
         sizer_output.Add(self.check_enable, 0, 0, 0)
         self.SetSizer(sizer_output)
         sizer_output.Fit(self)
@@ -920,4 +993,4 @@ class AutoContrastPanel(wx.Panel):
 
     def on_check_enable(self, event=None):
         self.op["enable"] = self.check_enable.GetValue()
-        self.node.update(self.context)
+        self.context.elements.do_image_update(self.node, self.context)
