@@ -3,22 +3,35 @@ import platform
 import wx
 
 from meerk40t.gui.fonts import wxfont_to_svg
-from meerk40t.gui.wxutils import ScrolledPanel
+from meerk40t.gui.icons import STD_ICON_SIZE, icons8_choose_font, icons8_text
+from meerk40t.gui.laserrender import LaserRender, swizzlecolor
+from meerk40t.gui.wxutils import (
+    ScrolledPanel,
+    StaticBoxSizer,
+    TextCtrl,
+    dip_size,
+    wxBitmapButton,
+    wxButton,
+    wxCheckBox,
+    wxComboBox,
+    wxListBox,
+    wxRadioBox,
+    wxStaticText,
+    wxToggleButton,
+)
+from meerk40t.svgelements import Color
 
-from ...svgelements import Color
-from ..icons import icons8_choose_font_50, icons8_text_50
-from ..laserrender import swizzlecolor
 from ..mwindow import MWindow
-from .attributes import ColorPanel, IdPanel, PositionSizePanel
+from .attributes import ColorPanel, IdPanel, PositionSizePanel, PreventChangePanel
 
 _ = wx.GetTranslation
 
 
-class PromptingComboBox(wx.ComboBox):
+class PromptingComboBox(wxComboBox):
     def __init__(self, parent, choices=None, style=0, **kwargs):
         if choices is None:
             choices = []
-        wx.ComboBox.__init__(
+        wxComboBox.__init__(
             self,
             parent,
             wx.ID_ANY,
@@ -66,6 +79,7 @@ class FontHistory(wx.Panel):
         wx.Panel.__init__(self, *args, **kwds)
         self.FONTHISTORY = 4
         self.context = context
+        self.context.themes.set_window_colors(self)
         self.last_font = []
         self.callback = callback
         self.textbox = textbox
@@ -75,7 +89,7 @@ class FontHistory(wx.Panel):
 
         for i in range(self.FONTHISTORY):
             self.last_font.append(
-                wx.StaticText(
+                wxStaticText(
                     self,
                     wx.ID_ANY,
                     _("<empty>"),
@@ -84,14 +98,14 @@ class FontHistory(wx.Panel):
                     | wx.ST_NO_AUTORESIZE,
                 )
             )
-            self.last_font[i].SetMinSize((120, 90))
+            self.last_font[i].SetMinSize(dip_size(self, 120, 90))
             self.last_font[i].SetFont(self.default_font)
             self.last_font[i].SetToolTip(_("Choose last used font-settings"))
             self.textbox.Bind(wx.EVT_TEXT, self.on_text_change)
 
         self.load_font_history()
-        sizer_h_fonthistory = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Last Font-Entries")), wx.HORIZONTAL
+        sizer_h_fonthistory = StaticBoxSizer(
+            self, wx.ID_ANY, _("Last Font-Entries"), wx.HORIZONTAL
         )
         for i in range(self.FONTHISTORY):
             sizer_h_fonthistory.Add(
@@ -144,14 +158,15 @@ class TextVariables(wx.Panel):
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
         self.textbox = textbox
+        self.context.themes.set_window_colors(self)
         # populate listbox
         choices = self.context.elements.mywordlist.get_variable_list()
-        self.lb_variables = wx.ListBox(self, wx.ID_ANY, choices=choices)
+        self.lb_variables = wxListBox(self, wx.ID_ANY, choices=choices)
         self.lb_variables.SetToolTip(_("Double click a variable to add it to the text"))
-        sizer_h_variables = wx.StaticBoxSizer(
-            wx.StaticBox(
-                self, wx.ID_ANY, _("Available Variables (double click to use)")
-            ),
+        sizer_h_variables = StaticBoxSizer(
+            self,
+            wx.ID_ANY,
+            _("Available Variables (double click to use)"),
             wx.HORIZONTAL,
         )
         sizer_h_variables.Add(self.lb_variables, 1, wx.EXPAND, 0)
@@ -171,13 +186,18 @@ class TextPropertyPanel(ScrolledPanel):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         super().__init__(parent, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
+        self.renderer = LaserRender(self.context)
+        self.SetHelpText("textproperty")
+        # We neeed this to avoid a crash under Linux when textselection is called too quickly
+        self._islinux = platform.system() == "Linux"
 
-        self.text_text = wx.TextCtrl(self, wx.ID_ANY, "", style=wx.TE_PROCESS_ENTER)
+        self.text_text = TextCtrl(self, wx.ID_ANY, "", style=wx.TE_PROCESS_ENTER)
         self.node = node
-        self.label_fonttest = wx.StaticText(
+        self.label_fonttest = wxStaticText(
             self, wx.ID_ANY, "", style=wx.ST_ELLIPSIZE_END | wx.ST_NO_AUTORESIZE
         )
-        self.label_fonttest.SetMinSize((-1, 90))
+        self.label_fonttest.SetMinSize(dip_size(self, -1, 90))
         self.label_fonttest.SetFont(
             wx.Font(
                 16,
@@ -188,8 +208,11 @@ class TextPropertyPanel(ScrolledPanel):
                 "Segoe UI",
             )
         )
-        self.button_choose_font = wx.BitmapButton(
-            self, wx.ID_ANY, icons8_choose_font_50.GetBitmap(resize=25)
+        self.button_choose_font = wxBitmapButton(
+            self, wx.ID_ANY,
+            icons8_choose_font.GetBitmap(
+                resize=STD_ICON_SIZE * self.context.root.bitmap_correction_scale / 2
+            )
         )
         self.panel_id = IdPanel(
             self, id=wx.ID_ANY, context=self.context, node=self.node
@@ -212,6 +235,9 @@ class TextPropertyPanel(ScrolledPanel):
             context=self.context,
             node=self.node,
         )
+        self.panel_lock = PreventChangePanel(
+            self, id=wx.ID_ANY, context=self.context, node=self.node
+        )
         self.panel_xy = PositionSizePanel(
             self, id=wx.ID_ANY, context=self.context, node=self.node
         )
@@ -242,26 +268,26 @@ class TextPropertyPanel(ScrolledPanel):
             mysize = 40
         else:
             mysize = 23
-        self.button_attrib_larger = wx.Button(
-            self, id=wx.ID_ANY, label="A", size=wx.Size(mysize, mysize)
+        self.button_attrib_larger = wxButton(
+            self, id=wx.ID_ANY, label="A", size=dip_size(self, mysize, mysize)
         )
-        self.button_attrib_smaller = wx.Button(
-            self, id=wx.ID_ANY, label="a", size=wx.Size(mysize, mysize)
+        self.button_attrib_smaller = wxButton(
+            self, id=wx.ID_ANY, label="a", size=dip_size(self, mysize, mysize)
         )
-        self.button_attrib_bold = wx.ToggleButton(
-            self, id=wx.ID_ANY, label="b", size=wx.Size(mysize, mysize)
+        self.button_attrib_bold = wxToggleButton(
+            self, id=wx.ID_ANY, label="b", size=dip_size(self, mysize, mysize)
         )
-        self.button_attrib_italic = wx.ToggleButton(
-            self, id=wx.ID_ANY, label="i", size=wx.Size(mysize, mysize)
+        self.button_attrib_italic = wxToggleButton(
+            self, id=wx.ID_ANY, label="i", size=dip_size(self, mysize, mysize)
         )
-        self.button_attrib_underline = wx.ToggleButton(
-            self, id=wx.ID_ANY, label="u", size=wx.Size(mysize, mysize)
+        self.button_attrib_underline = wxToggleButton(
+            self, id=wx.ID_ANY, label="u", size=dip_size(self, mysize, mysize)
         )
-        self.button_attrib_strikethrough = wx.ToggleButton(
-            self, id=wx.ID_ANY, label="s", size=wx.Size(mysize, mysize)
+        self.button_attrib_strikethrough = wxToggleButton(
+            self, id=wx.ID_ANY, label="s", size=dip_size(self, mysize, mysize)
         )
 
-        self.check_variable = wx.CheckBox(self, wx.ID_ANY, _(" Translate Variables"))
+        self.check_variable = wxCheckBox(self, wx.ID_ANY, _(" Translate Variables"))
         self.check_variable.SetToolTip(_("If active, preview will translate variables"))
         self.check_variable.SetValue(True)
         self.__set_properties()
@@ -305,6 +331,7 @@ class TextPropertyPanel(ScrolledPanel):
         self.panel_id.set_widgets(node)
         self.panel_stroke.set_widgets(node)
         self.panel_fill.set_widgets(node)
+        self.panel_lock.set_widgets(node)
         self.panel_xy.set_widgets(node)
 
         if node is not None:
@@ -315,7 +342,9 @@ class TextPropertyPanel(ScrolledPanel):
                 display_string = self.node.text
                 if self.check_variable.GetValue():
                     display_string = self.context.elements.wordlist_translate(
-                        display_string, self.node, increment=False,
+                        display_string,
+                        self.node,
+                        increment=False,
                     )
                 self.label_fonttest.SetLabelText(display_string)
                 try:
@@ -328,7 +357,6 @@ class TextPropertyPanel(ScrolledPanel):
         self.text_text.SelectAll()
 
     def __set_properties(self):
-
         self.button_choose_font.SetSize(self.button_choose_font.GetBestSize())
 
         self.button_attrib_bold.SetFont(
@@ -367,18 +395,14 @@ class TextPropertyPanel(ScrolledPanel):
         self.button_attrib_strikethrough.SetToolTip(_("Toggle strikethrough"))
 
         align_options = [_("Left"), _("Center"), _("Right")]
-        self.rb_align = wx.RadioBox(
+        self.rb_align = wxRadioBox(
             self,
             wx.ID_ANY,
-            "",
-            wx.DefaultPosition,
-            wx.DefaultSize,
-            align_options,
-            len(align_options),
-            wx.RA_SPECIFY_COLS | wx.BORDER_NONE,
+            label="",
+            choices=align_options,
         )
         self.rb_align.SetToolTip(
-            _("Define where to place the origin (i.e. current mouse position")
+            _("Define where to place the origin (i.e. current mouse position)")
         )
 
         # end wxGlade
@@ -413,6 +437,8 @@ class TextPropertyPanel(ScrolledPanel):
         sizer_main.Add(sizer_anchor, 0, wx.EXPAND, 0)
 
         self.notebook = wx.Notebook(self, id=wx.ID_ANY)
+        self.context.themes.set_window_colors(self.notebook)
+
         page_main = wx.Panel(self.notebook, wx.ID_ANY)
         sizer_page_main = wx.BoxSizer(wx.VERTICAL)
         self.panel_fill.Reparent(page_main)
@@ -431,7 +457,9 @@ class TextPropertyPanel(ScrolledPanel):
 
         page_extended = wx.Panel(self.notebook, wx.ID_ANY)
         sizer_page_extended = wx.BoxSizer(wx.VERTICAL)
+        self.panel_lock.Reparent(page_extended)
         self.panel_xy.Reparent(page_extended)
+        sizer_page_extended.Add(self.panel_lock, 0, wx.EXPAND, 0)
         sizer_page_extended.Add(self.panel_xy, 0, wx.EXPAND, 0)
         page_extended.SetSizer(sizer_page_extended)
 
@@ -488,12 +516,13 @@ class TextPropertyPanel(ScrolledPanel):
         except AttributeError:
             pass
         mystyle = self.label_fonttest.GetWindowStyle()
-        if self.node.anchor == "start":
-            new_anchor = 0
-            # Align the text to the left.
-            mystyle1 = wx.ALIGN_LEFT
-            mystyle2 = wx.ST_ELLIPSIZE_END
-        elif self.node.anchor == "middle":
+        if self.node.anchor is None:
+            self.node.anchor = "start"
+        new_anchor = 0
+        # Align the text to the left.
+        mystyle1 = wx.ALIGN_LEFT
+        mystyle2 = wx.ST_ELLIPSIZE_END
+        if self.node.anchor == "middle":
             new_anchor = 1
             mystyle1 = wx.ALIGN_CENTER
             mystyle2 = wx.ST_ELLIPSIZE_MIDDLE
@@ -516,7 +545,9 @@ class TextPropertyPanel(ScrolledPanel):
         display_string = self.node.text
         if self.check_variable.GetValue():
             display_string = self.context.elements.wordlist_translate(
-                display_string, self.node, increment=False,
+                display_string,
+                self.node,
+                increment=False,
             )
         self.label_fonttest.SetLabelText(display_string)
         self.label_fonttest.SetForegroundColour(wx.Colour(swizzlecolor(self.node.fill)))
@@ -526,7 +557,10 @@ class TextPropertyPanel(ScrolledPanel):
         self.button_attrib_italic.SetValue(self.node.font_style != "normal")
         self.button_attrib_underline.SetValue(self.node.underline)
         self.button_attrib_strikethrough.SetValue(self.node.strikethrough)
-        self.combo_font.SetValue(self.node.wxfont.GetFaceName())
+        try:
+            self.combo_font.SetValue(self.node.wxfont.GetFaceName())
+        except AttributeError:
+            pass
 
     def font_callback(self, forecolor, newfont):
         self.node.wxfont = newfont
@@ -536,6 +570,8 @@ class TextPropertyPanel(ScrolledPanel):
         self.refresh()
 
     def refresh(self):
+        self.renderer.measure_text(self.node)
+        bb = self.node.bounds
         self.context.elements.signal("element_property_reload", self.node)
         self.context.signal("refresh_scene", "Scene")
 
@@ -670,7 +706,10 @@ class TextPropertyPanel(ScrolledPanel):
             font = data.GetChosenFont()
             try:
                 color = data.GetColour()
-                rgb = color.GetRGB()
+                if color and color.IsOk():
+                    rgb = color.GetRGB()
+                else:
+                    rgb = 0
                 color = swizzlecolor(rgb)
                 color = Color(color, 1.0)
                 self.node.fill = color
@@ -691,13 +730,14 @@ class TextPropertyPanel(ScrolledPanel):
         self.update_label()
         self.refresh()
 
-    # @signal_listener("textselect")
-    # def on_signal_select(self, origin, *args):
-    #     try:
-    #         self.text_text.SelectAll()
-    #         self.text_text.SetFocus()
-    #     except RuntimeError:
-    #         pass
+    def signal(self, signal, *args):
+        if signal == "textselect" and self.IsShown():
+            # This can crash for completely unknown reasons under Linux!
+            # Hypothesis: you can't focus / SelectStuff if the control is not yet shown.
+            if self._islinux:
+                return
+            self.text_text.SelectAll()
+            self.text_text.SetFocus()
 
 
 class TextProperty(MWindow):
@@ -705,12 +745,14 @@ class TextProperty(MWindow):
         super().__init__(317, 360, *args, **kwds)
 
         self.panel = TextPropertyPanel(self, wx.ID_ANY, context=self.context, node=node)
+        self.sizer.Add(self.panel, 1, wx.EXPAND, 0)
         self.add_module_delegate(self.panel)
         _icon = wx.NullIcon
-        _icon.CopyFromBitmap(icons8_text_50.GetBitmap())
+        _icon.CopyFromBitmap(icons8_text.GetBitmap())
         self.SetIcon(_icon)
         # begin wxGlade: TextProperty.__set_properties
         self.SetTitle(_("Text Properties"))
+        self.restore_aspect()
 
     def restore(self, *args, node=None, **kwds):
         self.panel.set_widgets(node)
