@@ -4,20 +4,19 @@ import time
 
 import wx
 
-from meerk40t.core.element_types import elem_nodes
-from meerk40t.core.units import Length
+from meerk40t.core.elements.element_types import elem_nodes
 from meerk40t.gui.laserrender import (
     DRAW_MODE_ANIMATE,
     DRAW_MODE_FLIPXY,
     DRAW_MODE_INVERT,
     DRAW_MODE_REFRESH,
 )
-from meerk40t.gui.scene.guicolors import GuiColors
 from meerk40t.gui.scene.sceneconst import (
     HITCHAIN_DELEGATE,
     HITCHAIN_DELEGATE_AND_HIT,
     HITCHAIN_HIT,
     HITCHAIN_HIT_AND_DELEGATE,
+    HITCHAIN_PRIORITY_HIT,
     ORIENTATION_RELATIVE,
     RESPONSE_ABORT,
     RESPONSE_CHAIN,
@@ -25,16 +24,21 @@ from meerk40t.gui.scene.sceneconst import (
     RESPONSE_DROP,
 )
 from meerk40t.gui.scene.scenespacewidget import SceneSpaceWidget
+from meerk40t.gui.wxutils import get_matrix_scale
 from meerk40t.kernel import Job, Module
 from meerk40t.svgelements import Matrix, Point
 
-# TODO: _buffer can be updated partially rather than fully rewritten, especially with some layering.
-
-
 _reused_identity_widget = Matrix()
 XCELLS = 15
 YCELLS = 15
 
+TYPE_BOUND = 0
+TYPE_POINT = 1
+TYPE_MIDDLE = 2
+TYPE_CENTER = 3
+TYPE_GRID = 4
+TYPE_MIDDLE_SMALL = 5
+
 
 class SceneToast:
     """
@@ -72,9 +76,22 @@ class SceneToast:
         """
         self.countdown -= 1
         if self.countdown <= 20:
-            self.scene.request_refresh_for_animation()
+            self.scene.invalidate(
+                self.left - 10,
+                self.top - 10,
+                self.right + 10,
+                self.bottom + 10,
+                animate=True,
+            )
         if self.countdown <= 0:
-            self.scene.request_refresh()
+            self.scene.invalidate(
+                self.left - 10,
+                self.top - 10,
+                self.right + 10,
+                self.bottom + 10,
+                animate=False,
+            )
+            # self.scene.request_refresh()
             self.message = None
             self.token = None
         return self.countdown > 0
@@ -176,7 +193,7 @@ class Scene(Module, Job):
     to the scene and then the interface widget which contains all the non-scene widget elements.
     """
 
-    def __init__(self, context, path, gui, **kwargs):
+    def __init__(self, context, path, gui, pane=None, **kwargs):
         Module.__init__(self, context, path)
         Job.__init__(
             self,
@@ -185,52 +202,37 @@ class Scene(Module, Job):
             conditional=lambda: self.screen_refresh_is_requested,
             run_main=True,
         )
+        self.stack = []
+
+        # Scene lock is used for widget structure modification and scene drawing.
+        self.scene_lock = threading.RLock()
+
         self.log = context.channel("scene")
         self.log_events = context.channel("scene-events")
         self.gui = gui
+        self.pane = pane
         self.hittable_elements = list()
         self.hit_chain = list()
-        self.widget_root = SceneSpaceWidget(self)
-        self.screen_refresh_lock = threading.Lock()
+        self.widget_root = None
+        self.push_stack(SceneSpaceWidget(self))
+
         self.interval = 1.0 / 60.0  # 60fps
         self.last_position = None
         self._down_start_time = None
         self._down_start_pos = None
         self._cursor = None
-        self._reference = None  # Reference Object
-        self.attraction_points = []  # Clear all
-        self.compute = True
-        self.has_background = False
+        self.suppress_changes = True
 
-        self.colors = GuiColors(self.context)
+        self.colors = self.context.colors
 
         self.screen_refresh_is_requested = True
+        self.clip = wx.Rect(0, 0, 0, 0)
+
         self.background_brush = wx.Brush(self.colors.color_background)
+        self._hasbackgrounds = {}
+        self._backgrounds = {}
         # If set this color will be used for the scene background (used during burn)
         self.overrule_background = None
-        # Stuff for magnet-lines
-        self.magnet_x = []
-        self.magnet_y = []
-        self.magnet_attraction = 2
-        # 0 off, `1..x` increasing strength (quadratic behaviour)
-
-        self.magnet_attract_x = True  # Shall the X-Axis be affected
-        self.magnet_attract_y = True  # Shall the Y-Axis be affected
-        self.magnet_attract_c = True  # Shall the center be affected
-
-        # Stuff related to grids and guides
-        self.tick_distance = 0
-        self.auto_tick = False  # by definition do not auto_tick
-        self.reset_grids()
-
-        self.tool_active = False
-        self.modif_active = False
-
-        self._last_snap_position = None
-        self._last_snap_ts = 0
-
-        self.active_tool = "none"
-        self.grid_points = None  # Points representing the grid - total of primary + secondary + circular
 
         self._animating = list()
         self._animate_lock = threading.Lock()
@@ -242,154 +244,38 @@ class Scene(Module, Job):
             interval=1.0 / 60.0,
         )
         self._toast = None
+        # Snap information
+        self.snap_display_points = None
+        self.snap_attraction_points = None
 
     @property
-    def last_snap(self):
-        result = self._last_snap_position
-        # Too old? Discard
-        if (time.time() - self._last_snap_ts) > 0.5:
-            result = None
-        return result
+    def has_background(self):
+        devlabel = self.context.device.label
+        if devlabel not in self._hasbackgrounds:
+            self._hasbackgrounds[devlabel] = False
+        return self._hasbackgrounds[devlabel]
 
-    @last_snap.setter
-    def last_snap(self, value):
-        self._last_snap_position = value
-        if value is None:
-            self._last_snap_ts = 0
-        else:
-            self._last_snap_ts = time.time()
+    @has_background.setter
+    def has_background(self, value):
+        devlabel = self.context.device.label
+        self._hasbackgrounds[devlabel] = value
+        if not value:
+            self.active_background = None
 
-    def reset_grids(self):
-        self.draw_grid_primary = True
-        self.tick_distance = 0
-        # Secondary grid, perpendicular, but with definable center and scaling
-        self.draw_grid_secondary = False
-        self.grid_secondary_cx = None
-        self.grid_secondary_cy = None
-        self.grid_secondary_scale_x = 1
-        self.grid_secondary_scale_y = 1
-        # Circular grid
-        self.draw_grid_circular = False
-        self.grid_circular_cx = None
-        self.grid_circular_cy = None
+    @property
+    def active_background(self):
+        devlabel = self.context.device.label
+        if devlabel not in self._hasbackgrounds:
+            self._backgrounds[devlabel] = None
+        return self._backgrounds[devlabel]
 
-    def clear_magnets(self):
-        self.magnet_x = []
-        self.magnet_y = []
-        self.context.signal("magnets", False)
-
-    def toggle_x_magnet(self, x_value):
-        prev = self.has_magnets()
-        if x_value in self.magnet_x:
-            self.magnet_x.remove(x_value)
-            # print("Remove x magnet for %.1f" % x_value)
-            now = self.has_magnets()
-        else:
-            self.magnet_x += [x_value]
-            # print("Add x magnet for %.1f" % x_value)
-            now = True
-        if prev != now:
-            self.context.signal("magnets", now)
-
-    def toggle_y_magnet(self, y_value):
-        prev = self.has_magnets()
-        if y_value in self.magnet_y:
-            self.magnet_y.remove(y_value)
-            # print("Remove y magnet for %.1f" % y_value)
-            now = self.has_magnets()
-        else:
-            self.magnet_y += [y_value]
-            now = True
-            # print("Add y magnet for %.1f" % y_value)
-        if prev != now:
-            self.context.signal("magnets", now)
-
-    def magnet_attracted_x(self, x_value, useit):
-        delta = float("inf")
-        x_val = None
-        if useit:
-            for mag_x in self.magnet_x:
-                if abs(x_value - mag_x) < delta:
-                    delta = abs(x_value - mag_x)
-                    x_val = mag_x
-        return delta, x_val
-
-    def magnet_attracted_y(self, y_value, useit):
-        delta = float("inf")
-        y_val = None
-        if useit:
-            for mag_y in self.magnet_y:
-                if abs(y_value - mag_y) < delta:
-                    delta = abs(y_value - mag_y)
-                    y_val = mag_y
-        return delta, y_val
-
-    def revised_magnet_bound(self, bounds=None):
-
-        dx = 0
-        dy = 0
-        if self.has_magnets() and self.magnet_attraction > 0:
-            if self.tick_distance > 0:
-                s = f"{self.tick_distance}{self.context.units_name}"
-                len_tick = float(Length(s))
-                # Attraction length is 1/3, 4/3, 9/3 of a grid-unit
-                # fmt: off
-                attraction_len = 1 / 3 * self.magnet_attraction * self.magnet_attraction * len_tick
-
-                # print("Attraction len=%s, attract=%d, alen=%.1f, tlen=%.1f, factor=%.1f" % (s, self.magnet_attraction, attraction_len, len_tick, attraction_len / len_tick ))
-                # fmt: on
-            else:
-                attraction_len = float(Length("1mm"))
-
-            delta_x1, x1 = self.magnet_attracted_x(bounds[0], self.magnet_attract_x)
-            delta_x2, x2 = self.magnet_attracted_x(bounds[2], self.magnet_attract_x)
-            delta_x3, x3 = self.magnet_attracted_x(
-                (bounds[0] + bounds[2]) / 2, self.magnet_attract_c
-            )
-            delta_y1, y1 = self.magnet_attracted_y(bounds[1], self.magnet_attract_y)
-            delta_y2, y2 = self.magnet_attracted_y(bounds[3], self.magnet_attract_y)
-            delta_y3, y3 = self.magnet_attracted_y(
-                (bounds[1] + bounds[3]) / 2, self.magnet_attract_c
-            )
-            if delta_x3 < delta_x1 and delta_x3 < delta_x2:
-                if delta_x3 < attraction_len:
-                    if x3 is not None:
-                        dx = x3 - (bounds[0] + bounds[2]) / 2
-                        # print("X Take center , x=%.1f, dx=%.1f" % ((bounds[0] + bounds[2]) / 2, dx)
-            elif delta_x1 < delta_x2 and delta_x1 < delta_x3:
-                if delta_x1 < attraction_len:
-                    if x1 is not None:
-                        dx = x1 - bounds[0]
-                        # print("X Take left side, x=%.1f, dx=%.1f" % (bounds[0], dx))
-            elif delta_x2 < delta_x1 and delta_x2 < delta_x3:
-                if delta_x2 < attraction_len:
-                    if x2 is not None:
-                        dx = x2 - bounds[2]
-                        # print("X Take right side, x=%.1f, dx=%.1f" % (bounds[2], dx))
-            if delta_y3 < delta_y1 and delta_y3 < delta_y2:
-                if delta_y3 < attraction_len:
-                    if y3 is not None:
-                        dy = y3 - (bounds[1] + bounds[3]) / 2
-                        # print("Y Take center , x=%.1f, dx=%.1f" % ((bounds[1] + bounds[3]) / 2, dy))
-            elif delta_y1 < delta_y2 and delta_y1 < delta_y3:
-                if delta_y1 < attraction_len:
-                    if y1 is not None:
-                        dy = y1 - bounds[1]
-                        # print("Y Take top side, y=%.1f, dy=%.1f" % (bounds[1], dy))
-            elif delta_y2 < delta_y1 and delta_y2 < delta_y3:
-                if delta_y2 < attraction_len:
-                    if y2 is not None:
-                        dy = y2 - bounds[3]
-                        # print("Y Take bottom side, y=%.1f, dy=%.1f" % (bounds[3], dy))
-
-        return dx, dy
-
-    def has_magnets(self):
-        return len(self.magnet_x) + len(self.magnet_y) > 0
+    @active_background.setter
+    def active_background(self, value):
+        devlabel = self.context.device.label
+        self._backgrounds[devlabel] = value
 
     def module_open(self, *args, **kwargs):
         context = self.context
-        context.schedule(self)
         context.setting(int, "draw_mode", 0)
         context.setting(bool, "mouse_zoom_invert", False)
         context.setting(bool, "mouse_pan_invert", False)
@@ -408,7 +294,7 @@ class Scene(Module, Job):
 
     def module_close(self, *args, **kwargs):
         self._final_widget(self.widget_root, self.context)
-        self.screen_refresh_lock.acquire()  # calling shutdown live locks here since it's already shutting down.
+        self.scene_lock.acquire()  # calling shutdown live locks here since it's already shutting down.
         self.context.unschedule(self)
 
     def _init_widget(self, widget, context):
@@ -431,6 +317,23 @@ class Scene(Module, Job):
                 continue
             self._final_widget(w, context)
 
+    def push_stack(self, widget):
+        if self.widget_root is not None:
+            try:
+                widget.init(self.context)
+            except AttributeError:
+                pass
+            self.stack.append(self.widget_root)
+        self.widget_root = widget
+
+    def pop_stack(self):
+        widget = self.stack.pop()
+        try:
+            self.widget_root.final(self.context)
+        except AttributeError:
+            pass
+        self.widget_root = widget
+
     def set_fps(self, fps):
         """
         Set the scene frames per second which sets the interval for the Job.
@@ -456,6 +359,13 @@ class Scene(Module, Job):
         except AttributeError:
             pass
 
+    def invalidate(self, min_x, min_y, max_x, max_y, animate=False):
+        self.clip.Union((min_x, min_y, max_x - min_x, max_y - min_y))
+        if animate:
+            self.request_refresh_for_animation()
+        else:
+            self.request_refresh()
+
     def animate(self, widget):
         with self._animate_lock:
             if widget not in self._adding_widgets:
@@ -469,13 +379,14 @@ class Scene(Module, Job):
             self.log("Animating Scene...")
         if self._adding_widgets:
             with self._animate_lock:
-                for widget in self._adding_widgets:
-                    self._animating.append(widget)
-                    try:
-                        widget.start_threaded()
-                    except AttributeError:
-                        pass
+                animate_add = list(self._adding_widgets)
                 self._adding_widgets.clear()
+            for widget in animate_add:
+                self._animating.append(widget)
+                try:
+                    widget.start_threaded()
+                except AttributeError:
+                    pass
         if self._animating:
             for idx in range(len(self._animating) - 1, -1, -1):
                 widget = self._animating[idx]
@@ -499,27 +410,36 @@ class Scene(Module, Job):
         Called by the Scheduler at a given the specified framerate.
         Called in the UI thread.
         """
-        if self.screen_refresh_is_requested:
-            if self.screen_refresh_lock.acquire(timeout=0.2):
-                try:
-                    self.update_buffer_ui_thread()  # May hit runtime error.
-                    self.gui.Refresh()
-                    self.gui.Update()
-                except RuntimeError:
-                    pass
-                self.screen_refresh_is_requested = False
-                self.screen_refresh_lock.release()
-            else:
-                self.screen_refresh_is_requested = False
+        if not self.screen_refresh_is_requested:
+            return
+        if self.scene_lock.acquire(timeout=0.2):
+            try:
+                self._update_buffer_ui_thread()
+                self.gui.Refresh()
+                self.gui.Update()
+            except RuntimeError:
+                # May hit runtime error.
+                pass
+            self.screen_refresh_is_requested = False
+            self.scene_lock.release()
+        else:
+            self.screen_refresh_is_requested = False
 
-    def update_buffer_ui_thread(self):
+    def _update_buffer_ui_thread(self):
         """Performs redrawing of the data in the UI thread."""
         dm = self.context.draw_mode
-        buf = self.gui._Buffer
+        buf = self.gui.scene_buffer
         if buf is None or buf.GetSize() != self.gui.ClientSize or not buf.IsOk():
             self.gui.set_buffer()
-            buf = self.gui._Buffer
+            buf = self.gui.scene_buffer
         dc = wx.MemoryDC()
+        if self.clip.width != 0 and self.clip.height != 0:
+            dc.SetClippingRegion(self.clip)
+            self.clip.SetX(0)
+            self.clip.SetY(0)
+            self.clip.SetWidth(0)
+            self.clip.SetHeight(0)
+
         dc.SelectObject(buf)
         if self.overrule_background is None:
             self.background_brush.SetColour(self.colors.color_background)
@@ -621,7 +541,7 @@ class Scene(Module, Job):
         """
         Scene Draw routine to be called on paint when the _Buffer bitmap needs to be redrawn.
         """
-        if self.widget_root is not None:
+        if self.widget_root is not None and not self.suppress_changes:
             self.widget_root.draw(canvas)
             if self.log:
                 self.log("Redraw Canvas")
@@ -678,8 +598,8 @@ class Scene(Module, Job):
         response = current_widget.hit()
         if response == HITCHAIN_HIT:
             self.hittable_elements.append((current_widget, matrix_within_scene))
-        # elif response == HITCHAIN_HIT_WITH_PRIORITY:
-        #    self.hittable_elements.insert(0, (current_widget, matrix_within_scene))
+        elif response == HITCHAIN_PRIORITY_HIT:
+            self.hittable_elements.insert(0, (current_widget, matrix_within_scene))
         elif response == HITCHAIN_DELEGATE:
             for w in current_widget:
                 self.rebuild_hit_chain(w, matrix_within_scene)
@@ -709,7 +629,9 @@ class Scene(Module, Job):
             if current_widget.contains(hit_point.x, hit_point.y):
                 self.hit_chain.append((current_widget, current_matrix))
 
-    def event(self, window_pos, event_type="", nearest_snap=None, modifiers=None):
+    def event(
+        self, window_pos, event_type="", nearest_snap=None, modifiers=None, keycode=None
+    ):
         """
         Scene event code. Processes all the events for a particular mouse event bound in the ScenePanel.
 
@@ -723,10 +645,12 @@ class Scene(Module, Job):
         RESPONSE_CHAIN: Permit the event to move to the next event in the hitchain
         RESPONSE_DROP: Remove this item from the hitchain and continue to process the events. Future events will not
         consider the dropped element within the hitchain.
+
+        Returns a flag whether the event was consumed by someone
         """
         if self.log_events:
             self.log_events(
-                f"{event_type}: {str(window_pos)} {str(nearest_snap)} {str(modifiers)}"
+                f"{event_type}: {str(window_pos)} {str(nearest_snap)} {str(modifiers)} {str(keycode)}"
             )
 
         if window_pos is None:
@@ -742,7 +666,7 @@ class Scene(Module, Job):
                     nearest_snap=None,
                     modifiers=None,
                 )
-            return
+            return False
         if self.last_position is None:
             self.last_position = window_pos
         dx = window_pos[0] - self.last_position[0]
@@ -795,16 +719,19 @@ class Scene(Module, Job):
                     event_type=event_type,
                     nearest_snap=nearest_snap,
                     modifiers=modifiers,
+                    keycode=keycode,
                 )
 
                 if response == RESPONSE_ABORT:
                     self.hit_chain.clear()
-                    return
+                    # print (f"Response abort of {event_type}, {keycode}/{modifiers} by {current_widget}")
+                    return True
                 elif response == RESPONSE_CONSUME:
                     # if event_type in ("leftdown", "middledown", "middleup", "leftup", "move", "leftclick"):
                     #      widgetname = type(current_widget).__name__
                     #      print("Event %s was consumed by %s" % (event_type, widgetname))
-                    return
+                    # print (f"Response consume of {event_type}, {keycode}/{modifiers} by {current_widget}")
+                    return True
                 elif response == RESPONSE_CHAIN:
                     continue
                 elif response == RESPONSE_DROP:
@@ -813,7 +740,7 @@ class Scene(Module, Job):
                 #
                 # if response == RESPONSE_ABORT:
                 #     self.hit_chain.clear()
-            return
+            return False
 
         if event_type in (
             "leftdown",
@@ -862,6 +789,7 @@ class Scene(Module, Job):
                         event_type="hover_end",
                         nearest_snap=None,
                         modifiers=modifiers,
+                        keycode=keycode,
                     )
                 current_widget.event(
                     window_pos=window_pos,
@@ -869,6 +797,7 @@ class Scene(Module, Job):
                     event_type="hover_start",
                     nearest_snap=None,
                     modifiers=modifiers,
+                    keycode=keycode,
                 )
                 if self.log_events:
                     self.log_events(f"Converted hover_start: {str(window_pos)}")
@@ -885,6 +814,7 @@ class Scene(Module, Job):
                     event_type="leftclick",
                     nearest_snap=nearest_snap,
                     modifiers=modifiers,
+                    keycode=keycode,
                 )
                 if self.log_events:
                     self.log_events(f"Converted leftclick: {str(window_pos)}")
@@ -899,6 +829,7 @@ class Scene(Module, Job):
                     event_type=event_type,
                     nearest_snap=nearest_snap,
                     modifiers=modifiers,
+                    keycode=keycode,
                 )
                 # print ("Leftup called for widget #%d" % i )
                 # print (response)
@@ -909,6 +840,7 @@ class Scene(Module, Job):
                     event_type=event_type,
                     nearest_snap=nearest_snap,
                     modifiers=modifiers,
+                    keycode=keycode,
                 )
 
             ##################
@@ -924,12 +856,10 @@ class Scene(Module, Job):
                     new_y_space = params[1]
                     new_x = window_pos[0]
                     new_y = window_pos[1]
-                    snap_x = None
-                    snap_y = None
 
                     sdx = new_x_space - space_pos[0]
                     if current_matrix is not None and not current_matrix.is_identity():
-                        sdx *= current_matrix.value_scale_x()
+                        sdx *= get_matrix_scale(current_matrix)
                     snap_x = window_pos[0] + sdx
                     sdy = new_y_space - space_pos[1]
                     if current_matrix is not None and not current_matrix.is_identity():
@@ -937,8 +867,8 @@ class Scene(Module, Job):
                     # print("Shift x by %.1f pixel (%.1f), Shift y by %.1f pixel (%.1f)" % (sdx, odx, sdy, ody))
                     snap_y = window_pos[1] + sdy
 
-                    dx = new_x - self.last_position[0]
-                    dy = new_y - self.last_position[1]
+                    # dx = new_x - self.last_position[0]
+                    # dy = new_y - self.last_position[1]
                     if snap_x is None:
                         nearest_snap = None
                     else:
@@ -952,18 +882,16 @@ class Scene(Module, Job):
                             snap_x,
                             snap_y,
                         )
-                        self.last_snap = nearest_snap
-            else:
-                params = None
+                        self.pane.last_snap = nearest_snap
 
             if response == RESPONSE_ABORT:
                 self.hit_chain.clear()
-                return
+                return True
             elif response == RESPONSE_CONSUME:
                 # if event_type in ("leftdown", "middledown", "middleup", "leftup", "move", "leftclick"):
                 #      widgetname = type(current_widget).__name__
                 #      print("Event %s was consumed by %s" % (event_type, widgetname))
-                return
+                return True
             elif response == RESPONSE_CHAIN:
                 continue
             elif response == RESPONSE_DROP:
@@ -971,6 +899,7 @@ class Scene(Module, Job):
                 continue
             else:
                 break
+        return False
 
     def cursor(self, cursor, always=False):
         """
@@ -1030,32 +959,142 @@ class Scene(Module, Job):
         """
         self.widget_root.interface_widget.add_widget(-1, widget, properties)
 
-    def validate_reference(self):
+    # --- Centralised snap_point calculation
+    def _calculate_snap_points(self, my_x, my_y, length):
         """
-        Check whether the reference is still valid
+        Recalculate the snap element attraction points
+
+        @param length:
+        @return:
         """
-        found = False
-        if self._reference:
-            for e in self.context.elements.flat(types=elem_nodes):
-                # Here we ignore the lock-status of an element
-                if e is self._reference:
-                    found = True
-                    break
-        if not found:
-            self._reference = None
+        for pts in self.snap_attraction_points:
+            if self.pane.modif_active:
+                if pts[3]:
+                    # No snap points for emphasized objects.
+                    continue
+            if abs(pts[0] - my_x) <= length and abs(pts[1] - my_y) <= length:
+                self.snap_display_points.append([pts[0], pts[1], pts[2]])
 
-    @property
-    def reference_object(self):
-        return self._reference
+    def _calculate_grid_points(self, my_x, my_y, length):
+        """
+        Recalculate the local grid points
 
-    @reference_object.setter
-    def reference_object(self, ref_object):
-        prev = self._reference
-        self._reference = ref_object
-        dlist = []
-        if prev is not None:
-            dlist.append(prev)
-        if self._reference is not None:
-            dlist.append(self._reference)
-        if len(dlist) > 0:
-            self.context.signal("element_property_update", dlist)
+        @param length:
+        @return:
+        """
+        for pts in self.pane.grid.grid_points:
+            if abs(pts[0] - my_x) <= length and abs(pts[1] - my_y) <= length:
+                self.snap_display_points.append([pts[0], pts[1], TYPE_GRID])
+
+    def _calculate_attraction_points(self):
+        """
+        Looks at all elements (all_points=True) or at non-selected elements (all_points=False) and identifies all
+        attraction points (center, corners, sides)
+        """
+        self.context.elements.set_start_time("attr_calc_points")
+        self.snap_attraction_points = []  # Clear all
+        translation_table = {
+            "bounds top_left": TYPE_BOUND,
+            "bounds top_right": TYPE_BOUND,
+            "bounds bottom_left": TYPE_BOUND,
+            "bounds bottom_right": TYPE_BOUND,
+            "bounds center_center": TYPE_CENTER,
+            "bounds top_center": TYPE_MIDDLE,
+            "bounds bottom_center": TYPE_MIDDLE,
+            "bounds center_left": TYPE_MIDDLE,
+            "bounds center_right": TYPE_MIDDLE,
+            "endpoint": TYPE_POINT,
+            "point": TYPE_POINT,
+            "midpoint": TYPE_MIDDLE_SMALL,
+        }
+
+        for e in self.context.elements.flat(types=elem_nodes):
+            emph = e.emphasized
+            if hasattr(e, "points"):
+                for pt in e.points:
+                    try:
+                        pt_type = translation_table[pt[2]]
+                    except:
+                        print(f"Unknown type: {pt[2]}")
+                        pt_type = TYPE_POINT
+                    self.snap_attraction_points.append([pt[0], pt[1], pt_type, emph])
+
+        self.context.elements.set_end_time(
+            "attr_calc_points",
+            message=f"points added={len(self.snap_attraction_points)}",
+        )
+
+    def calculate_display_points(self, my_x, my_y, snap_points, snap_grid):
+        """
+        Recalculate the points that need to be displayed for the user.
+
+        @return:
+        """
+        self.snap_display_points = []
+        if my_x is None:
+            return
+        if self.snap_attraction_points is None and snap_points:
+            self._calculate_attraction_points()
+
+        matrix = self.widget_root.scene_widget.matrix
+        length = self.context.show_attract_len / get_matrix_scale(matrix)
+
+        if snap_points and self.snap_attraction_points:
+            self._calculate_snap_points(my_x, my_y, length)
+
+        if snap_grid and self.pane.grid.grid_points:
+            self._calculate_grid_points(my_x, my_y, length)
+
+    def calculate_snap(self, my_x, my_y):
+        """
+        Calculates the nearest_snap
+        """
+        # Loop through display points, find closest.
+        res_x = None
+        res_y = None
+        if self.snap_display_points and my_x is not None:
+            # Has to be lower than the action threshold
+            min_delta = float("inf")
+            new_x = None
+            new_y = None
+            for pt in self.snap_display_points:
+                dx = pt[0] - my_x
+                dy = pt[1] - my_y
+                delta = dx * dx + dy * dy
+                if delta < min_delta:
+                    new_x = pt[0]
+                    new_y = pt[1]
+                    min_delta = delta
+            if new_x is not None:
+                matrix = self.widget_root.scene_widget.matrix
+                pixel = self.context.action_attract_len / get_matrix_scale(matrix)
+                if abs(new_x - my_x) <= pixel and abs(new_y - my_y) <= pixel:
+                    # If the distance is small enough: snap.
+                    res_x = new_x
+                    res_y = new_y
+        return res_x, res_y
+
+    def get_snap_point(self, sx, sy, modifiers):
+        resx = sx
+        resy = sy
+        sgrid = self.context.snap_grid
+        spoints = self.context.snap_points
+        if "shift" in modifiers:
+            sgrid = not sgrid
+            spoints = False
+        if sgrid or spoints:
+            if "shift" in modifiers:
+                # Need to recalculate
+                self.reset_snap_attraction()
+            self.calculate_display_points(sx, sy, spoints, sgrid)
+            nx, ny = self.calculate_snap(sx, sy)
+            if nx is not None:
+                resx = nx
+                resy = ny
+            if "shift" in modifiers:
+                # Need to recalculate again
+                self.reset_snap_attraction()
+        return resx, resy
+
+    def reset_snap_attraction(self):
+        self.snap_attraction_points = None
