@@ -1,3 +1,10 @@
+"""
+Relocate widget adds a widget relocation widget. Moving the widget will cause the direct parent of this widget to
+move. The expectation is that the parent type of this widget will DELEGATE or DELEGATE and HIT.
+
+This is usually used for a group of widgets which should be moved together.
+"""
+
 import wx
 
 from meerk40t.gui import icons
@@ -9,7 +16,7 @@ class RelocateWidget(Widget):
     def __init__(self, scene, x, y):
         size = 10000
         Widget.__init__(self, scene, x - size, y - size, x + size, y + size)
-        self.bitmap = icons.icons8_center_of_gravity_50.GetBitmap()
+        self.bitmap = icons.icons8_center_of_gravity.GetBitmap()
 
     def hit(self):
         return HITCHAIN_HIT
