@@ -4,20 +4,35 @@ import re
 import wx
 from wx import aui
 
+from ..extra.encode_detect import EncodingDetectFile
+from ..kernel import signal_listener
 from .icons import (
-    icons8_add_new_25,
-    icons8_curly_brackets_50,
-    icons8_edit_25,
-    icons8_paste_25,
-    icons8_remove_25,
-    icons8_circled_right_50,
-    icons8_circled_left_50,
+    get_default_icon_size,
+    icon_add_new,
+    icon_edit,
+    icon_trash,
+    icons8_circled_left,
+    icons8_circled_right,
+    icons8_curly_brackets,
+    icons8_paste,
 )
 from .mwindow import MWindow
-from ..kernel import signal_listener
+from .wxutils import (
+    StaticBoxSizer,
+    TextCtrl,
+    dip_size,
+    wxButton,
+    wxCheckBox,
+    wxComboBox,
+    wxListCtrl,
+    wxRadioBox,
+    wxStaticBitmap,
+    wxStaticText,
+)
 
 _ = wx.GetTranslation
 
+
 def register_panel_wordlist(window, context):
     pane = (
         aui.AuiPaneInfo()
@@ -32,26 +47,41 @@ def register_panel_wordlist(window, context):
     pane.dock_proportion = 225
     pane.control = WordlistMiniPanel(window, wx.ID_ANY, context=context)
     pane.submenu = "_50_" + _("Tools")
+    pane.helptext = _("Display wordlist advancement controls")
     window.on_pane_create(pane)
     context.register("pane/wordlist", pane)
 
+
 class WordlistMiniPanel(wx.Panel):
     def __init__(self, *args, context=None, **kwds):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
+        self.SetHelpText("wordlist")
+
         main_sizer = wx.BoxSizer(wx.HORIZONTAL)
-        self.button_edit = wx.Button(self, wx.ID_ANY, _("Edit"))
-        self.button_edit.SetBitmap(icons8_curly_brackets_50.GetBitmap(resize=25))
+        self.button_edit = wxButton(self, wx.ID_ANY, _("Edit"))
+        self.button_edit.SetBitmap(
+            icons8_curly_brackets.GetBitmap(resize=0.5 * get_default_icon_size(self.context))
+        )
         self.button_edit.SetToolTip(_("Manages Wordlist-Entries"))
 
-        self.button_next = wx.Button(self, wx.ID_ANY, _("Next"))
-        self.button_next.SetBitmap(icons8_circled_right_50.GetBitmap(resize=25))
-        self.button_next.SetToolTip(_("Wordlist: go to next page (right-click to next entry)"))
+        self.button_next = wxButton(self, wx.ID_ANY, _("Next"))
+        self.button_next.SetBitmap(
+            icons8_circled_right.GetBitmap(resize=0.5 * get_default_icon_size(self.context))
+        )
+        self.button_next.SetToolTip(
+            _("Wordlist: go to next page (right-click to next entry)")
+        )
 
-        self.button_prev = wx.Button(self, wx.ID_ANY, _("Prev"))
-        self.button_prev.SetBitmap(icons8_circled_left_50.GetBitmap(resize=25))
-        self.button_prev.SetToolTip(_("Wordlist: go to previous page (right-click to previous entry)"))
+        self.button_prev = wxButton(self, wx.ID_ANY, _("Prev"))
+        self.button_prev.SetBitmap(
+            icons8_circled_left.GetBitmap(resize=0.5 * get_default_icon_size(self.context))
+        )
+        self.button_prev.SetToolTip(
+            _("Wordlist: go to previous page (right-click to previous entry)")
+        )
 
         main_sizer.Add(self.button_prev, 1, wx.EXPAND, 0)
         main_sizer.Add(self.button_edit, 1, wx.EXPAND, 0)
@@ -74,16 +104,17 @@ class WordlistMiniPanel(wx.Panel):
             sample = ""
             if node.type == "elem text":
                 if node.text is not None:
-                    sample  = node.text
+                    sample = str(node.text)
             elif node.type == "elem path" and hasattr(node, "mktext"):
                 if node.mktext is not None:
-                    sample  = node.mktext
+                    sample = str(node.mktext)
             if sample == "":
                 continue
-            # we can be rather agnostic on the individual variable, we are interested in the highest {variable#+offset} pattern
+            # we can be rather agnostic on the individual variable,
+            # we are interested in the highest {variable#+offset} pattern
             brackets = re.compile(r"\{[^}]+\}")
             for bracketed_key in brackets.findall(sample):
-    #            print(f"Key found: {bracketed_key}")
+                #            print(f"Key found: {bracketed_key}")
                 key = bracketed_key[1:-1].lower().strip()
                 relative = 0
                 pos = key.find("#")
@@ -125,6 +156,8 @@ class WordlistPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
+        self.SetHelpText("wordlist")
         self.parent_panel = None
         self.wlist = self.context.elements.mywordlist
         self.current_entry = None
@@ -146,14 +179,14 @@ class WordlistPanel(wx.Panel):
         sizer_index_left = wx.BoxSizer(wx.HORIZONTAL)
         sizer_grid_left.Add(sizer_index_left, 0, wx.EXPAND, 0)
 
-        label_2 = wx.StaticText(self, wx.ID_ANY, _("Start Index for CSV-based data:"))
+        label_2 = wxStaticText(self, wx.ID_ANY, _("Start Index for CSV-based data:"))
         sizer_index_left.Add(label_2, 0, wx.ALIGN_CENTER_VERTICAL, 0)
-        self.cbo_Index = wx.ComboBox(
+        self.cbo_Index = wxComboBox(
             self, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN | wx.CB_READONLY
         )
         sizer_index_left.Add(self.cbo_Index, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.grid_wordlist = wx.ListCtrl(
+        self.grid_wordlist = wxListCtrl(
             self,
             wx.ID_ANY,
             style=wx.LC_HRULES
@@ -161,10 +194,12 @@ class WordlistPanel(wx.Panel):
             | wx.LC_VRULES
             | wx.LC_SINGLE_SEL
             | wx.LC_EDIT_LABELS,
+            context=self.context,
+            list_name="list_wordlist"
         )
         sizer_grid_left.Add(self.grid_wordlist, 1, wx.EXPAND, 0)
 
-        self.grid_content = wx.ListCtrl(
+        self.grid_content = wxListCtrl(
             self,
             wx.ID_ANY,
             style=wx.LC_HRULES
@@ -172,41 +207,64 @@ class WordlistPanel(wx.Panel):
             | wx.LC_VRULES
             | wx.LC_SINGLE_SEL
             | wx.LC_EDIT_LABELS,
+            context=self.context,
+            list_name="list_wordlist_content",
         )
 
         sizer_edit_wordlist_buttons = wx.BoxSizer(wx.HORIZONTAL)
         sizer_edit_content_buttons = wx.BoxSizer(wx.HORIZONTAL)
 
-        dummylabel = wx.StaticText(self, wx.ID_ANY, " ")
+        dummylabel = wxStaticText(self, wx.ID_ANY, " ")
         sizer_index_left.Add(dummylabel, 1, wx.ALIGN_CENTER_VERTICAL, 0)
         sizer_index_left.Add(
             sizer_edit_wordlist_buttons, 0, wx.ALIGN_CENTER_VERTICAL, 0
         )
+        testsize = dip_size(self, 20, 20)
+        icon_size = testsize[0]
 
-        self.btn_edit_wordlist_del = wx.StaticBitmap(
-            self, wx.ID_ANY, size=wx.Size(25, 25)
+        self.btn_edit_wordlist_del = wxStaticBitmap(
+            self, wx.ID_ANY, size=dip_size(self, 25, 25)
         )
-        self.btn_edit_wordlist_edit = wx.StaticBitmap(
-            self, wx.ID_ANY, size=wx.Size(25, 25)
+        self.btn_edit_wordlist_edit = wxStaticBitmap(
+            self, wx.ID_ANY, size=dip_size(self, 25, 25)
         )
-        self.btn_edit_content_add = wx.StaticBitmap(
-            self, wx.ID_ANY, size=wx.Size(25, 25)
+        self.btn_edit_content_add = wxStaticBitmap(
+            self, wx.ID_ANY, size=dip_size(self, 25, 25)
         )
-        self.btn_edit_content_del = wx.StaticBitmap(
-            self, wx.ID_ANY, size=wx.Size(25, 25)
+        self.btn_edit_content_del = wxStaticBitmap(
+            self, wx.ID_ANY, size=dip_size(self, 25, 25)
         )
-        self.btn_edit_content_edit = wx.StaticBitmap(
-            self, wx.ID_ANY, size=wx.Size(25, 25)
+        self.btn_edit_content_edit = wxStaticBitmap(
+            self, wx.ID_ANY, size=dip_size(self, 25, 25)
         )
-        self.btn_edit_content_paste = wx.StaticBitmap(
-            self, wx.ID_ANY, size=wx.Size(25, 25)
+        self.btn_edit_content_paste = wxStaticBitmap(
+            self, wx.ID_ANY, size=dip_size(self, 25, 25)
+        )
+        # Circumvent a WXPython bug at high resolutions under Windows
+        bmp = icon_trash.GetBitmap(resize=icon_size, buffer=1)
+        self.btn_edit_wordlist_del.SetBitmap(bmp)
+        testsize = self.btn_edit_wordlist_del.GetBitmap().Size
+        if testsize[0] != icon_size:
+            icon_size = int(icon_size * icon_size / testsize[0])
+
+        self.btn_edit_wordlist_del.SetBitmap(
+            icon_trash.GetBitmap(resize=icon_size, buffer=1)
+        )
+        self.btn_edit_wordlist_edit.SetBitmap(
+            icon_edit.GetBitmap(resize=icon_size, buffer=1)
+        )
+        self.btn_edit_content_add.SetBitmap(
+            icon_add_new.GetBitmap(resize=icon_size, buffer=1)
+        )
+        self.btn_edit_content_del.SetBitmap(
+            icon_trash.GetBitmap(resize=icon_size, buffer=1)
+        )
+        self.btn_edit_content_edit.SetBitmap(
+            icon_edit.GetBitmap(resize=icon_size, buffer=1)
+        )
+        self.btn_edit_content_paste.SetBitmap(
+            icons8_paste.GetBitmap(resize=icon_size, buffer=1)
         )
-        self.btn_edit_wordlist_del.SetBitmap(icons8_remove_25.GetBitmap())
-        self.btn_edit_wordlist_edit.SetBitmap(icons8_edit_25.GetBitmap())
-        self.btn_edit_content_add.SetBitmap(icons8_add_new_25.GetBitmap())
-        self.btn_edit_content_del.SetBitmap(icons8_remove_25.GetBitmap())
-        self.btn_edit_content_edit.SetBitmap(icons8_edit_25.GetBitmap())
-        self.btn_edit_content_paste.SetBitmap(icons8_paste_25.GetBitmap())
 
         self.btn_edit_wordlist_del.SetToolTip(_("Delete the current variable"))
         self.btn_edit_wordlist_edit.SetToolTip(
@@ -227,12 +285,12 @@ class WordlistPanel(wx.Panel):
             )
         )
         minsize = 23
-        self.btn_edit_wordlist_del.SetMinSize(wx.Size(minsize, minsize))
-        self.btn_edit_wordlist_edit.SetMinSize(wx.Size(minsize, minsize))
-        self.btn_edit_content_add.SetMinSize(wx.Size(minsize, minsize))
-        self.btn_edit_content_del.SetMinSize(wx.Size(minsize, minsize))
-        self.btn_edit_content_edit.SetMinSize(wx.Size(minsize, minsize))
-        self.btn_edit_content_paste.SetMinSize(wx.Size(minsize, minsize))
+        self.btn_edit_wordlist_del.SetMinSize(dip_size(self, minsize, minsize))
+        self.btn_edit_wordlist_edit.SetMinSize(dip_size(self, minsize, minsize))
+        self.btn_edit_content_add.SetMinSize(dip_size(self, minsize, minsize))
+        self.btn_edit_content_del.SetMinSize(dip_size(self, minsize, minsize))
+        self.btn_edit_content_edit.SetMinSize(dip_size(self, minsize, minsize))
+        self.btn_edit_content_paste.SetMinSize(dip_size(self, minsize, minsize))
 
         sizer_edit_wordlist_buttons.Add(self.btn_edit_wordlist_del, 0, wx.EXPAND, 0)
         sizer_edit_wordlist_buttons.Add(self.btn_edit_wordlist_edit, 0, wx.EXPAND, 0)
@@ -243,14 +301,14 @@ class WordlistPanel(wx.Panel):
         sizer_edit_content_buttons.Add(self.btn_edit_content_paste, 0, wx.EXPAND, 0)
 
         sizer_index_right = wx.BoxSizer(wx.HORIZONTAL)
-        label_2 = wx.StaticText(self, wx.ID_ANY, _("Start Index for field:"))
+        label_2 = wxStaticText(self, wx.ID_ANY, _("Start Index for field:"))
         sizer_index_right.Add(label_2, 0, wx.ALIGN_CENTER_VERTICAL, 0)
-        self.cbo_index_single = wx.ComboBox(
+        self.cbo_index_single = wxComboBox(
             self, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN | wx.CB_READONLY
         )
         sizer_index_right.Add(self.cbo_index_single, 1, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        dummylabel = wx.StaticText(self, wx.ID_ANY, " ")
+        dummylabel = wxStaticText(self, wx.ID_ANY, " ")
         sizer_index_right.Add(dummylabel, 1, wx.ALIGN_CENTER_VERTICAL, 0)
         sizer_index_right.Add(
             sizer_edit_content_buttons, 0, wx.ALIGN_CENTER_VERTICAL, 0
@@ -259,40 +317,40 @@ class WordlistPanel(wx.Panel):
         sizer_grid_right.Add(sizer_index_right, 0, wx.EXPAND, 0)
         sizer_grid_right.Add(self.grid_content, 1, wx.EXPAND, 0)
 
-        self.txt_pattern = wx.TextCtrl(self, wx.ID_ANY, "")
+        self.txt_pattern = TextCtrl(self, wx.ID_ANY, "")
 
         sizer_buttons = wx.BoxSizer(wx.HORIZONTAL)
         sizer_buttons.Add(self.txt_pattern, 1, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_add = wx.Button(self, wx.ID_ANY, _("Add Text"))
+        self.btn_add = wxButton(self, wx.ID_ANY, _("Add Text"))
         self.btn_add.SetToolTip(_("Add another wordlist entry"))
         sizer_buttons.Add(self.btn_add, 0, 0, 0)
 
-        self.btn_add_counter = wx.Button(self, wx.ID_ANY, _("Add Counter"))
+        self.btn_add_counter = wxButton(self, wx.ID_ANY, _("Add Counter"))
         sizer_buttons.Add(self.btn_add_counter, 0, 0, 0)
 
-        self.btn_delete = wx.Button(self, wx.ID_ANY, _("Delete"))
+        self.btn_delete = wxButton(self, wx.ID_ANY, _("Delete"))
         self.btn_delete.SetToolTip(_("Delete the current wordlist entry"))
         sizer_buttons.Add(self.btn_delete, 0, 0, 0)
 
         sizer_exit = wx.BoxSizer(wx.HORIZONTAL)
 
-        self.btn_backup = wx.Button(self, wx.ID_ANY, _("Backup Wordlist"))
+        self.btn_backup = wxButton(self, wx.ID_ANY, _("Backup Wordlist"))
         self.btn_backup.SetToolTip(_("Save current wordlist to disk"))
         sizer_exit.Add(self.btn_backup, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_restore = wx.Button(self, wx.ID_ANY, _("Restore Wordlist"))
+        self.btn_restore = wxButton(self, wx.ID_ANY, _("Restore Wordlist"))
         self.btn_restore.SetToolTip(_("Load wordlist from disk"))
         sizer_exit.Add(self.btn_restore, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.check_autosave = wx.CheckBox(self, wx.ID_ANY, _("Autosave"))
+        self.check_autosave = wxCheckBox(self, wx.ID_ANY, _("Autosave"))
         self.check_autosave.SetToolTip(
             _("All changes to the wordlist will be saved immediately")
         )
         sizer_exit.Add(self.check_autosave, 0, wx.ALIGN_CENTER_VERTICAL, 0)
         self.check_autosave.SetValue(self.context.wordlist_autosave)
 
-        self.lbl_message = wx.StaticText(self, wx.ID_ANY, "")
+        self.lbl_message = wxStaticText(self, wx.ID_ANY, "")
         sizer_exit.Add(self.lbl_message, 1, wx.ALIGN_CENTER_VERTICAL, 0)
 
         sizer_lower = wx.BoxSizer(wx.HORIZONTAL)
@@ -323,6 +381,7 @@ class WordlistPanel(wx.Panel):
         self.grid_content.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_grid_content)
         self.grid_content.Bind(wx.EVT_LIST_BEGIN_LABEL_EDIT, self.on_begin_edit_content)
         self.grid_content.Bind(wx.EVT_LIST_END_LABEL_EDIT, self.on_end_edit_content)
+        self.grid_content.Bind(wx.EVT_LEFT_DCLICK, self.on_content_dblclick)
         self.check_autosave.Bind(wx.EVT_CHECKBOX, self.on_checkbox_autosave)
 
         self.btn_edit_wordlist_del.Bind(wx.EVT_LEFT_DOWN, self.on_btn_edit_wordlist_del)
@@ -344,11 +403,14 @@ class WordlistPanel(wx.Panel):
         self.parent_panel = par_panel
 
     def pane_show(self):
+        self.grid_wordlist.load_column_widths()
+        self.grid_content.load_column_widths()
         self.populate_gui()
         self.grid_wordlist.SetFocus()
 
     def pane_hide(self):
-        pass
+        self.grid_wordlist.save_column_widths()
+        self.grid_content.save_column_widths()
 
     def autosave(self):
         if self.check_autosave.GetValue():
@@ -422,17 +484,19 @@ class WordlistPanel(wx.Panel):
         if skey is None:
             return
         text_data = wx.TextDataObject()
+        success = False
         if wx.TheClipboard.Open():
             success = wx.TheClipboard.GetData(text_data)
             wx.TheClipboard.Close()
-        if success:
-            msg = text_data.GetText()
-            if msg is not None and len(msg) > 0:
-                lines = msg.splitlines()
-                for entry in lines:
-                    self.wlist.add_value(skey, entry, 0)
-                self.refresh_grid_content(skey, 0)
-                self.autosave()
+        if not success:
+            return
+        msg = text_data.GetText()
+        if msg is not None and len(msg) > 0:
+            lines = msg.splitlines()
+            for entry in lines:
+                self.wlist.add_value(skey, entry, 0)
+            self.refresh_grid_content(skey, 0)
+            self.autosave()
 
     def refresh_grid_wordlist(self):
         self.current_entry = None
@@ -451,6 +515,7 @@ class WordlistPanel(wx.Panel):
         self.grid_wordlist.SetColumnWidth(0, wx.LIST_AUTOSIZE)
         self.grid_wordlist.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
         self.grid_wordlist.SetColumnWidth(2, wx.LIST_AUTOSIZE_USEHEADER)
+        self.grid_wordlist.resize_columns()
 
     def get_column_text(self, grid, index, col):
         item = grid.GetItem(index, col)
@@ -481,6 +546,8 @@ class WordlistPanel(wx.Panel):
                 self.grid_content.SetItemTextColour(index, wx.RED)
         wsize = self.grid_content.GetSize()
         self.grid_content.SetColumnWidth(0, wsize[0] - 10)
+        self.grid_content.resize_columns()
+
         self.cbo_index_single.Set(choices)
         if selidx >= 0:
             self.cbo_index_single.SetSelection(selidx)
@@ -508,6 +575,12 @@ class WordlistPanel(wx.Panel):
         self.txt_pattern.SetValue(skey)
         event.Skip()
 
+    def on_content_dblclick(self, event):
+        index = self.grid_content.GetFirstSelected()
+        if index >= 0:
+            self.cbo_index_single.SetSelection(index)
+            self.on_single_index(event)
+
     def on_grid_content(self, event):
         # Single Click
         event.Skip()
@@ -577,8 +650,8 @@ class WordlistPanel(wx.Panel):
                 self.cbo_Index.Append(str(i))
             self.cbo_Index.SetValue("0")
         self.cbo_Index.Enable(True)
-        # Lets refresh the scene to acknowledge new changes
-        self.context.signal("refresh_scene")
+        # Let's refresh the scene to acknowledge new changes
+        self.context.signal("refresh_scene", "Scene")
 
     def edit_message(self, text):
         self.lbl_message.Label = text
@@ -658,36 +731,35 @@ class WordlistPanel(wx.Panel):
         self.autosave()
         self.refresh_grid_wordlist()
 
+
 class ImportPanel(wx.Panel):
     def __init__(self, *args, context=None, **kwds):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.parent_panel = None
         self.context = context
+        self.context.themes.set_window_colors(self)
         self.wlist = self.context.elements.mywordlist
         main_sizer = wx.BoxSizer(wx.VERTICAL)
-        info_box = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Import CSV")),
-            wx.VERTICAL,
-        )
+        info_box = StaticBoxSizer(self, wx.ID_ANY, _("Import CSV"), wx.VERTICAL)
         sizer_csv = wx.BoxSizer(wx.HORIZONTAL)
         info_box.Add(sizer_csv, 1, wx.EXPAND, 0)
 
-        label_1 = wx.StaticText(self, wx.ID_ANY, _("Import CSV-File"))
+        label_1 = wxStaticText(self, wx.ID_ANY, _("Import CSV-File"))
         sizer_csv.Add(label_1, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.txt_filename = wx.TextCtrl(self, wx.ID_ANY, "")
+        self.txt_filename = TextCtrl(self, wx.ID_ANY, "")
         sizer_csv.Add(self.txt_filename, 1, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_fileDialog = wx.Button(self, wx.ID_ANY, "...")
-        self.btn_fileDialog.SetMinSize((23, 23))
+        self.btn_fileDialog = wxButton(self, wx.ID_ANY, "...")
+        self.btn_fileDialog.SetMinSize(dip_size(self, 23, 23))
         sizer_csv.Add(self.btn_fileDialog, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
-        self.btn_import = wx.Button(self, wx.ID_ANY, _("Import CSV"))
+        self.btn_import = wxButton(self, wx.ID_ANY, _("Import CSV"))
         sizer_csv.Add(self.btn_import, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
         sizer_header = wx.BoxSizer(wx.HORIZONTAL)
-        self.rbox_header = wx.RadioBox(
+        self.rbox_header = wxRadioBox(
             self,
             wx.ID_ANY,
             _("What does the first row contain:"),
@@ -703,7 +775,7 @@ class ImportPanel(wx.Panel):
         sizer_header.Add(self.rbox_header, 1, wx.EXPAND, 0)
         info_box.Add(sizer_header, 0, wx.EXPAND)
 
-        self.text_preview = wx.TextCtrl(
+        self.text_preview = TextCtrl(
             self, wx.ID_ANY, style=wx.TE_READONLY | wx.TE_MULTILINE
         )
 
@@ -783,13 +855,15 @@ class ImportPanel(wx.Panel):
 
     def preview_it(self):
         filename = self.txt_filename.GetValue()
+
         buffer = ""
         if os.path.exists(filename):
-            try:
-                with open(filename, newline="", mode="r") as csvfile:
-                    buffer = csvfile.read(1024)
-            except (PermissionError, OSError, FileNotFoundError):
-                pass
+            decoder = EncodingDetectFile()
+            result = decoder.load(filename)
+            if result:
+                encoding, bom_marker, file_content = result
+                buffer = file_content
+
         self.text_preview.SetValue(buffer)
 
 
@@ -798,14 +872,12 @@ class AboutPanel(wx.Panel):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
         self.context = context
+        self.context.themes.set_window_colors(self)
         main_sizer = wx.BoxSizer(wx.VERTICAL)
-        info_box = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("How to use...")),
-            wx.VERTICAL,
-        )
+        info_box = StaticBoxSizer(self, wx.ID_ANY, _("How to use..."), wx.VERTICAL)
         self.parent_panel = None
         s = self.context.asset("wordlist_howto")
-        info_label = wx.TextCtrl(
+        info_label = TextCtrl(
             self, wx.ID_ANY, value=s, style=wx.TE_READONLY | wx.TE_MULTILINE
         )
         font = wx.Font(
@@ -838,7 +910,7 @@ class WordlistEditor(MWindow):
         self.panel_about.set_parent(self)
 
         _icon = wx.NullIcon
-        _icon.CopyFromBitmap(icons8_curly_brackets_50.GetBitmap())
+        _icon.CopyFromBitmap(icons8_curly_brackets.GetBitmap())
         self.SetIcon(_icon)
         self.notebook_main = wx.aui.AuiNotebook(
             self,
@@ -848,6 +920,13 @@ class WordlistEditor(MWindow):
             | wx.aui.AUI_NB_TAB_SPLIT
             | wx.aui.AUI_NB_TAB_MOVE,
         )
+        self.window_context.themes.set_window_colors(self.notebook_main)
+        bg_std = self.window_context.themes.get("win_bg")
+        bg_active = self.window_context.themes.get("highlight")
+        self.notebook_main.GetArtProvider().SetColour(bg_std)
+        self.notebook_main.GetArtProvider().SetActiveColour(bg_active)
+
+        self.sizer.Add(self.notebook_main, 1, wx.EXPAND, 0)
         self.notebook_main.AddPage(self.panel_editor, _("Editing"))
         self.notebook_main.AddPage(self.panel_import, _("Import/Export"))
         self.notebook_main.AddPage(self.panel_about, _("How to use"))
@@ -855,6 +934,7 @@ class WordlistEditor(MWindow):
         self.DragAcceptFiles(True)
         self.Bind(wx.EVT_DROP_FILES, self.on_drop_file)
         self.SetTitle(_("Wordlist Editor"))
+        self.restore_aspect()
 
     def on_drop_file(self, event):
         """
@@ -875,9 +955,10 @@ class WordlistEditor(MWindow):
         kernel.register(
             "button/config/Wordlist",
             {
-                "label": _("Wordlist"),
-                "icon": icons8_curly_brackets_50,
+                "label": _("Wordlist Editor"),
+                "icon": icons8_curly_brackets,
                 "tip": _("Manages Wordlist-Entries"),
+                "help": "wordlist",
                 "action": lambda v: kernel.console("window toggle Wordlist\n"),
             },
         )
@@ -896,4 +977,9 @@ class WordlistEditor(MWindow):
 
     @staticmethod
     def submenu():
-        return ("Editing", "Variables + Wordlists")
+        # Suppress to avoid double menu-appearance
+        return "Editing", "Variables + Wordlists", True
+
+    @staticmethod
+    def helptext():
+        return _("Configure the wordlist (text variable) entries")
