@@ -2,7 +2,6 @@ from typing import Any, Callable, Dict, Generator, Optional, Tuple, Union
 
 from .jobs import ConsoleFunction
 from .lifecycles import *
-from .states import *
 
 
 class Context:
@@ -21,7 +20,7 @@ class Context:
     def __init__(self, kernel: "Kernel", path: str):
         self._kernel = kernel
         self._path = path
-        self._state = STATE_UNKNOWN
+        self._state = "unknown"
         self.opened = {}
 
     def __repr__(self):
@@ -85,14 +84,6 @@ class Context:
         """
         return self._kernel.get_context(path)
 
-    def derivable(self) -> Generator[str, None, None]:
-        """
-        Generate all sub derived paths.
-
-        @return:
-        """
-        yield from self._kernel.derivable(self._path)
-
     def subpaths(self) -> Generator["Context", None, None]:
         """
         Generate all subpaths of the current context with their path name and the relevant context.
@@ -169,7 +160,7 @@ class Context:
         The attribute type of the value depends on the provided object value default values.
 
         @param t: type of value
-        @param key: relative key for the value
+        @param key: relative key
         @return: the value associated with the key otherwise None
         """
         return self._kernel.read_persistent(t, self._path, key)
