@@ -2,8 +2,20 @@ import threading
 
 import wx
 
-from meerk40t.gui.icons import icons8_connected_50, icons8_disconnected_50
+from meerk40t.gui.icons import (
+    get_default_icon_size,
+    icons8_connected,
+    icons8_disconnected,
+)
 from meerk40t.gui.mwindow import MWindow
+from meerk40t.gui.wxutils import (
+    StaticBoxSizer,
+    TextCtrl,
+    dip_size,
+    wxButton,
+    wxCheckBox,
+    wxStaticText,
+)
 from meerk40t.kernel import signal_listener
 
 _ = wx.GetTranslation
@@ -17,36 +29,38 @@ class MoshiControllerPanel(wx.Panel):
     def __init__(self, *args, context=None, **kwds):
         kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
         wx.Panel.__init__(self, *args, **kwds)
+        context.themes.set_window_colors(self)
         self.context = context.device
+        self.SetHelpText("moshicontroller")
 
-        self.button_device_connect = wx.Button(self, wx.ID_ANY, _("Connection"))
-        self.text_connection_status = wx.TextCtrl(
+        self.button_device_connect = wxButton(self, wx.ID_ANY, _("Connection"))
+        self.text_connection_status = TextCtrl(
             self, wx.ID_ANY, "", style=wx.TE_READONLY
         )
-        self.checkbox_mock_usb = wx.CheckBox(
+        self.checkbox_mock_usb = wxCheckBox(
             self, wx.ID_ANY, _("Mock USB Connection Mode")
         )
-        self.text_device_index = wx.TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
+        self.text_device_index = TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
         self.spin_device_index = wx.SpinCtrl(self, wx.ID_ANY, "-1", min=-1)
-        self.text_device_address = wx.TextCtrl(
+        self.text_device_address = TextCtrl(
             self, wx.ID_ANY, "", style=wx.TE_READONLY
         )
         self.spin_device_address = wx.SpinCtrl(self, wx.ID_ANY, "-1", min=-1)
-        self.text_device_bus = wx.TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
+        self.text_device_bus = TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
         self.spin_device_bus = wx.SpinCtrl(self, wx.ID_ANY, "-1", min=-1)
-        self.text_device_version = wx.TextCtrl(
+        self.text_device_version = TextCtrl(
             self, wx.ID_ANY, "", style=wx.TE_READONLY
         )
         self.spin_device_version = wx.SpinCtrl(self, wx.ID_ANY, "-1", min=-1)
-        self.text_byte_0 = wx.TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
-        self.text_byte_1 = wx.TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
-        self.text_desc = wx.TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
-        self.text_byte_2 = wx.TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
-        self.text_byte_3 = wx.TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
-        self.text_byte_4 = wx.TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
-        self.text_byte_5 = wx.TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
-        self.checkbox_show_usb_log = wx.CheckBox(self, wx.ID_ANY, _("Show USB Log"))
-        self.text_usb_log = wx.TextCtrl(
+        self.text_byte_0 = TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
+        self.text_byte_1 = TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
+        self.text_desc = TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
+        self.text_byte_2 = TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
+        self.text_byte_3 = TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
+        self.text_byte_4 = TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
+        self.text_byte_5 = TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
+        self.checkbox_show_usb_log = wxCheckBox(self, wx.ID_ANY, _("Show USB Log"))
+        self.text_usb_log = TextCtrl(
             self, wx.ID_ANY, "", style=wx.TE_MULTILINE | wx.TE_READONLY
         )
 
@@ -93,7 +107,9 @@ class MoshiControllerPanel(wx.Panel):
         self.button_device_connect.SetBackgroundColour(wx.Colour(102, 255, 102))
         self.button_device_connect.SetForegroundColour(wx.BLACK)
         self.button_device_connect.SetBitmap(
-            icons8_disconnected_50.GetBitmap(use_theme=False)
+            icons8_disconnected.GetBitmap(
+                use_theme=False, resize=get_default_icon_size(self.context)
+            )
         )
         self.button_device_connect.SetFont(
             wx.Font(
@@ -114,46 +130,46 @@ class MoshiControllerPanel(wx.Panel):
                 "DEBUG. Without a K40 connected continue to process things as if there was one."
             )
         )
-        self.text_device_index.SetMinSize((55, 23))
-        self.spin_device_index.SetMinSize((40, 23))
+        self.text_device_index.SetMinSize(dip_size(self, 55, -1))
+        self.spin_device_index.SetMinSize(dip_size(self, 40, -1))
         self.spin_device_index.SetToolTip(
             _(
                 "Optional: Distinguish between different lasers using the match criteria below.\n"
                 "-1 match anything. 0+ match exactly that value."
             )
         )
-        self.text_device_address.SetMinSize((55, 23))
-        self.spin_device_address.SetMinSize((40, 23))
+        self.text_device_address.SetMinSize(dip_size(self, 55, -1))
+        self.spin_device_address.SetMinSize(dip_size(self, 40, -1))
         self.spin_device_address.SetToolTip(
             _(
                 "Optional: Distinguish between different lasers using the match criteria below.\n"
                 "-1 match anything. 0+ match exactly that value."
             )
         )
-        self.text_device_bus.SetMinSize((55, 23))
-        self.spin_device_bus.SetMinSize((40, 23))
+        self.text_device_bus.SetMinSize(dip_size(self, 55, -1))
+        self.spin_device_bus.SetMinSize(dip_size(self, 40, -1))
         self.spin_device_bus.SetToolTip(
             _(
                 "Optional: Distinguish between different lasers using the match criteria below.\n"
                 "-1 match anything. 0+ match exactly that value."
             )
         )
-        self.text_device_version.SetMinSize((55, 23))
-        self.spin_device_version.SetMinSize((40, 23))
+        self.text_device_version.SetMinSize(dip_size(self, 55, -1))
+        self.spin_device_version.SetMinSize(dip_size(self, 40, -1))
         self.spin_device_version.SetToolTip(
             _(
                 "Optional: Distinguish between different lasers using the match criteria below.\n"
                 "-1 match anything. 0+ match exactly that value."
             )
         )
-        self.text_byte_0.SetMinSize((77, 23))
-        self.text_byte_1.SetMinSize((77, 23))
-        self.text_desc.SetMinSize((75, 23))
+        self.text_byte_0.SetMinSize(dip_size(self, 77, -1))
+        self.text_byte_1.SetMinSize(dip_size(self, 77, -1))
+        self.text_desc.SetMinSize(dip_size(self, 75, 23))
         self.text_desc.SetToolTip(_("The meaning of Byte 1"))
-        self.text_byte_2.SetMinSize((77, 23))
-        self.text_byte_3.SetMinSize((77, 23))
-        self.text_byte_4.SetMinSize((77, 23))
-        self.text_byte_5.SetMinSize((77, 23))
+        self.text_byte_2.SetMinSize(dip_size(self, 77, -1))
+        self.text_byte_3.SetMinSize(dip_size(self, 77, -1))
+        self.text_byte_4.SetMinSize(dip_size(self, 77, -1))
+        self.text_byte_5.SetMinSize(dip_size(self, 77, -1))
         self.checkbox_show_usb_log.SetValue(1)
         # end wxGlade
 
@@ -162,11 +178,9 @@ class MoshiControllerPanel(wx.Panel):
         sizer_24 = wx.BoxSizer(wx.HORIZONTAL)
         sizer_1 = wx.BoxSizer(wx.VERTICAL)
         sizer_show_usb_log = wx.BoxSizer(wx.HORIZONTAL)
-        packet_count = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Packet Info")), wx.VERTICAL
-        )
-        byte_data_status = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Byte Data Status")), wx.HORIZONTAL
+        packet_count = StaticBoxSizer(self, wx.ID_ANY, _("Packet Info"), wx.VERTICAL)
+        byte_data_status = StaticBoxSizer(
+            self, wx.ID_ANY, _("Byte Data Status"), wx.HORIZONTAL
         )
         byte5sizer = wx.BoxSizer(wx.VERTICAL)
         byte4sizer = wx.BoxSizer(wx.VERTICAL)
@@ -174,24 +188,16 @@ class MoshiControllerPanel(wx.Panel):
         byte2sizer = wx.BoxSizer(wx.VERTICAL)
         byte1sizer = wx.BoxSizer(wx.VERTICAL)
         byte0sizer = wx.BoxSizer(wx.VERTICAL)
-        sizer_usb_settings = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("USB Settings")), wx.VERTICAL
+        sizer_usb_settings = StaticBoxSizer(
+            self, wx.ID_ANY, _("USB Settings"), wx.VERTICAL
         )
         sizer_23 = wx.BoxSizer(wx.HORIZONTAL)
-        sizer_12 = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Chip Version")), wx.HORIZONTAL
-        )
-        sizer_11 = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Device Bus:")), wx.HORIZONTAL
-        )
-        sizer_10 = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Device Address:")), wx.HORIZONTAL
-        )
-        sizer_3 = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("Device Index:")), wx.HORIZONTAL
-        )
-        sizer_usb_connect = wx.StaticBoxSizer(
-            wx.StaticBox(self, wx.ID_ANY, _("USB Connection")), wx.VERTICAL
+        sizer_12 = StaticBoxSizer(self, wx.ID_ANY, _("Chip Version"), wx.HORIZONTAL)
+        sizer_11 = StaticBoxSizer(self, wx.ID_ANY, _("Device Bus:"), wx.HORIZONTAL)
+        sizer_10 = StaticBoxSizer(self, wx.ID_ANY, _("Device Address:"), wx.HORIZONTAL)
+        sizer_3 = StaticBoxSizer(self, wx.ID_ANY, _("Device Index:"), wx.HORIZONTAL)
+        sizer_usb_connect = StaticBoxSizer(
+            self, wx.ID_ANY, _("USB Connection"), wx.VERTICAL
         )
         sizer_usb_connect.Add(self.button_device_connect, 0, wx.EXPAND, 0)
         sizer_usb_connect.Add(self.text_connection_status, 0, wx.EXPAND, 0)
@@ -212,33 +218,33 @@ class MoshiControllerPanel(wx.Panel):
         sizer_usb_settings.Add(sizer_23, 1, wx.EXPAND, 0)
         sizer_1.Add(sizer_usb_settings, 0, wx.EXPAND, 0)
         byte0sizer.Add(self.text_byte_0, 0, 0, 0)
-        label_1 = wx.StaticText(self, wx.ID_ANY, _("Byte 0"))
+        label_1 = wxStaticText(self, wx.ID_ANY, _("Byte 0"))
         byte0sizer.Add(label_1, 0, 0, 0)
         byte_data_status.Add(byte0sizer, 1, wx.EXPAND, 0)
         byte1sizer.Add(self.text_byte_1, 0, 0, 0)
-        label_2 = wx.StaticText(self, wx.ID_ANY, _("Byte 1"))
+        label_2 = wxStaticText(self, wx.ID_ANY, _("Byte 1"))
         byte1sizer.Add(label_2, 0, 0, 0)
         byte1sizer.Add(self.text_desc, 0, 0, 0)
         byte_data_status.Add(byte1sizer, 1, wx.EXPAND, 0)
         byte2sizer.Add(self.text_byte_2, 0, 0, 0)
-        label_3 = wx.StaticText(self, wx.ID_ANY, _("Byte 2"))
+        label_3 = wxStaticText(self, wx.ID_ANY, _("Byte 2"))
         byte2sizer.Add(label_3, 0, 0, 0)
         byte_data_status.Add(byte2sizer, 1, wx.EXPAND, 0)
         byte3sizer.Add(self.text_byte_3, 0, 0, 0)
-        label_4 = wx.StaticText(self, wx.ID_ANY, _("Byte 3"))
+        label_4 = wxStaticText(self, wx.ID_ANY, _("Byte 3"))
         byte3sizer.Add(label_4, 0, 0, 0)
         byte_data_status.Add(byte3sizer, 1, wx.EXPAND, 0)
         byte4sizer.Add(self.text_byte_4, 0, 0, 0)
-        label_5 = wx.StaticText(self, wx.ID_ANY, _("Byte 4"))
+        label_5 = wxStaticText(self, wx.ID_ANY, _("Byte 4"))
         byte4sizer.Add(label_5, 0, 0, 0)
         byte_data_status.Add(byte4sizer, 1, wx.EXPAND, 0)
         byte5sizer.Add(self.text_byte_5, 0, 0, 0)
-        label_18 = wx.StaticText(self, wx.ID_ANY, _("Byte 5"))
+        label_18 = wxStaticText(self, wx.ID_ANY, _("Byte 5"))
         byte5sizer.Add(label_18, 0, 0, 0)
         byte_data_status.Add(byte5sizer, 1, wx.EXPAND, 0)
         packet_count.Add(byte_data_status, 0, wx.EXPAND, 0)
         sizer_1.Add(packet_count, 0, 0, 0)
-        label_6 = wx.StaticText(self, wx.ID_ANY, "")
+        label_6 = wxStaticText(self, wx.ID_ANY, "")
         sizer_show_usb_log.Add(label_6, 10, wx.EXPAND, 0)
         sizer_show_usb_log.Add(self.checkbox_show_usb_log, 0, 0, 0)
         sizer_1.Add(sizer_show_usb_log, 1, wx.EXPAND, 0)
@@ -249,16 +255,17 @@ class MoshiControllerPanel(wx.Panel):
         # end wxGlade
 
     def pane_show(self):
-        active = self.context.path.split("/")[-1]
-        self.context.channel(f"{active}/usb", buffer_size=500).watch(self.update_text)
+        self._channel_watch = f"{self.context.safe_label}/usb"
+        self.context.channel(self._channel_watch, buffer_size=500).watch(
+            self.update_text
+        )
         self.context.listen("pipe;status", self.update_status)
         self.context.listen("pipe;usb_status", self.on_connection_status_change)
         self.context.listen("pipe;state", self.on_connection_state_change)
         self.context.listen("active", self.on_active_change)
 
     def pane_hide(self):
-        active = self.context.path.split("/")[-1]
-        self.context.channel(f"{active}/usb").unwatch(self.update_text)
+        self.context.channel(self._channel_watch).unwatch(self.update_text)
         self.context.unlisten("pipe;status", self.update_status)
         self.context.unlisten("pipe;usb_status", self.on_connection_status_change)
         self.context.unlisten("pipe;state", self.on_connection_state_change)
@@ -328,35 +335,45 @@ class MoshiControllerPanel(wx.Panel):
             if usb_status is not None:
                 self.button_device_connect.SetLabel(str(usb_status[0]))
             self.button_device_connect.SetBitmap(
-                icons8_disconnected_50.GetBitmap(use_theme=False)
+                icons8_disconnected.GetBitmap(
+                    use_theme=False, resize=get_default_icon_size(self.context)
+                )
             )
             self.button_device_connect.Enable()
         elif state == "STATE_UNINITIALIZED" or state == "STATE_USB_DISCONNECTED":
             self.button_device_connect.SetBackgroundColour("#ffff00")
             self.button_device_connect.SetLabel(_("Connect"))
             self.button_device_connect.SetBitmap(
-                icons8_connected_50.GetBitmap(use_theme=False)
+                icons8_connected.GetBitmap(
+                    use_theme=False, resize=get_default_icon_size(self.context)
+                )
             )
             self.button_device_connect.Enable()
         elif state == "STATE_USB_SET_DISCONNECTING":
             self.button_device_connect.SetBackgroundColour("#ffff00")
             self.button_device_connect.SetLabel(_("Disconnecting..."))
             self.button_device_connect.SetBitmap(
-                icons8_disconnected_50.GetBitmap(use_theme=False)
+                icons8_disconnected.GetBitmap(
+                    use_theme=False, resize=get_default_icon_size(self.context)
+                )
             )
             self.button_device_connect.Disable()
         elif state == "STATE_USB_CONNECTED" or state == "STATE_CONNECTED":
             self.button_device_connect.SetBackgroundColour("#00ff00")
             self.button_device_connect.SetLabel(_("Disconnect"))
             self.button_device_connect.SetBitmap(
-                icons8_connected_50.GetBitmap(use_theme=False)
+                icons8_connected.GetBitmap(
+                    use_theme=False, resize=get_default_icon_size(self.context)
+                )
             )
             self.button_device_connect.Enable()
         elif state == "STATE_CONNECTING":
             self.button_device_connect.SetBackgroundColour("#ffff00")
             self.button_device_connect.SetLabel(_("Connecting..."))
             self.button_device_connect.SetBitmap(
-                icons8_connected_50.GetBitmap(use_theme=False)
+                icons8_connected.GetBitmap(
+                    use_theme=False, resize=get_default_icon_size(self.context)
+                )
             )
             self.button_device_connect.Disable()
 
@@ -432,11 +449,13 @@ class MoshiControllerGui(MWindow):
         # ==========
 
         self.panel = MoshiControllerPanel(self, wx.ID_ANY, context=self.context)
+        self.sizer.Add(self.panel, 1, wx.EXPAND, 0)
         self.add_module_delegate(self.panel)
         _icon = wx.NullIcon
-        _icon.CopyFromBitmap(icons8_connected_50.GetBitmap())
+        _icon.CopyFromBitmap(icons8_connected.GetBitmap())
         self.SetIcon(_icon)
         self.SetTitle(_("Moshiboard-Controller"))
+        self.restore_aspect()
 
     def create_menu(self, append):
         wxglade_tmp_menu = wx.Menu()
@@ -512,4 +531,9 @@ class MoshiControllerGui(MWindow):
 
     @staticmethod
     def submenu():
-        return ("Device-Control", "Controller")
+        # Hint for translation: _("Device-Control"), _("Controller")
+        return "Device-Control", "Controller"
+
+    @staticmethod
+    def helptext():
+        return _("Display the device controller window")
