@@ -62,8 +62,8 @@ def read_int_32le(stream):
 
 
 def read_string(stream):
+    bb = bytearray()
     try:
-        bb = bytearray()
         while True:
             b = stream.read(1)
             if b == b"":
@@ -180,6 +180,7 @@ class ShxFont:
     """
 
     def __init__(self, filename, debug=False):
+        self.STROKE_BASED = True
         self.format = None  # format (usually AutoCAD-86)
         self.type = None  # Font type: shapes, bigfont, unifont
         self.version = None  # Font file version (usually 1.0).
@@ -204,12 +205,17 @@ class ShxFont:
         self._last_y = 0
         self._scale = 1
         self._stack = []
+        self.active = True
+        self._line_information = []
 
         self._parse(filename)
 
     def __str__(self):
         return f'{self.type}("{self.font_name}", {self.version}, glyphs: {len(self.glyphs)})'
 
+    def line_information(self):
+        return self._line_information
+
     def _parse(self, filename):
         with open(filename, "br") as f:
             self._parse_header(f)
@@ -223,6 +229,22 @@ class ShxFont:
                 self._parse_unifont(f)
             else:
                 raise ShxFontParseError(f"{self.type} is not a valid shx file type.")
+            self.translate_names_to_codes()
+
+    def translate_names_to_codes(self):
+        # We can't properly deal with long names, 
+        # so we translate them a virtual character 
+        gdict = list(self.glyphs.items())
+        virtual_char = len(self.glyphs)
+        for key, data in gdict:
+            if isinstance(key, str) and len(key) > 1:
+                while True:
+                    newkey = chr(virtual_char)
+                    if newkey not in self.glyphs:
+                        break
+                    virtual_char += 1
+                self.glyphs[newkey] = data
+                del self.glyphs[key]
 
     def _parse_header(self, f):
         header = read_string(f)
@@ -345,33 +367,131 @@ class ShxFont:
         try:
             return self._code.pop()
         except IndexError as e:
+            print (f"Error in {self.font_name}: no codes to pop")
             raise ShxFontParseError("No codes to pop()") from e
 
-    def render(self, path, text, horizontal=True, font_size=12.0):
-        if self.above is None:
-            self.above = 1
-        self._scale = font_size / self.above
-        self._horizontal = horizontal
-        self._path = path
-        for letter in text:
-            self._letter = letter
-            try:
-                self._code = bytearray(reversed(self.glyphs[ord(letter)]))
-            except KeyError:
-                # Letter is not found.
-                continue
-            self._pen = True
-            while self._code:
-                try:
-                    self._parse_code()
-                except IndexError as e:
-                    raise ShxFontParseError("Stack Error during render.") from e
-            self._skip = False
-        if self._debug:
-            print(f"Render Complete.\n\n\n")
+    def render(
+        self,
+        path,
+        vtext,
+        horizontal=True,
+        font_size=12.0,
+        h_spacing=1.0,
+        v_spacing=1.1,
+        align="start",
+    ):
+        def _do_render(to_render, offsets):
+            if self.above is None:
+                self.above = 1
+            if self.below is None:
+                self.below = 0
+            self._scale = font_size / self.above
+            self._horizontal = horizontal
+            self._path = path
+            replacer = []
+            for tchar in to_render:
+                to_replace = None
+                if tchar == "\n":
+                    continue
+                # Yes, I am German :-)
+                if ord(tchar) not in self.glyphs:
+                    if tchar == "ä":
+                        to_replace = (tchar, "ae")
+                    elif tchar == "ö":
+                        to_replace = (tchar, "ue")
+                    elif tchar == "ü":
+                        to_replace = (tchar, "ue")
+                    elif tchar == "Ä":
+                        to_replace = (tchar, "Ae")
+                    elif tchar == "Ö":
+                        to_replace = (tchar, "Oe")
+                    elif tchar == "Ü":
+                        to_replace = (tchar, "Ue")
+                    elif tchar == "ß":
+                        to_replace = (tchar, "ss")
+                if to_replace is not None and to_replace not in replacer:
+                    replacer.append(to_replace)
+            for to_replace in replacer:
+                # print (f"Replace all '{to_replace[0]}' with '{to_replace[1]}'")
+                to_render = to_render.replace(to_replace[0], to_replace[1])
+            lines = to_render.split("\n")
+            offset_y = 0
+            if offsets is None:
+                offsets = [0] * len(lines)
+            self._line_information.clear()
+            for text, offs in zip(lines, offsets):
+                self._y = offset_y * self._scale
+                self._last_y = self._y
+                self._x = offs
+                self._last_x = offs
+                maxx = offs
+                line_start_x = self._x
+                line_start_y = self._y
+                for letter in text:
+                    last_letter_x = self._last_x
+                    last_letter_y = self._last_y
+                    self._letter = letter
+                    try:
+                        self._code = bytearray(reversed(self.glyphs[ord(letter)]))
+                    except KeyError:
+                        # Letter is not found.
+                        continue
+                    self._pen = True
+                    while self._code:
+                        try:
+                            self._parse_code()
+                        except IndexError as e:
+                            raise ShxFontParseError("Stack Error during render.") from e
+                    self._skip = False
+                    if h_spacing != 1.0:
+                        dx = (h_spacing - 1) * (self._last_x - last_letter_x)
+                        self._last_x += dx
+                        self._x += dx
+                    maxx = max(maxx, self._x)
+                    if self.active:
+                        path.character_end()
+                # Store start point, nonscaled width plus scaled width and height of line
+                self._line_information.append(
+                    (
+                        line_start_x,
+                        line_start_y,
+                        maxx,
+                        maxx - line_start_x,
+                        self._scale * (self.above + self.below),
+                    )
+                )
+                offset_y -= v_spacing * (self.above + self.below)
+
+            if self._debug:
+                print(f"Render Complete.\n\n\n")
+            line_lens = [e[2] for e in self._line_information]
+            return line_lens
+
+        if vtext is None or vtext == "":
+            return
+        self.active = False
+        line_lengths = _do_render(vtext, None)
+        max_len = max(line_lengths)
+        offsets = []
+        for ll in line_lengths:
+            # NB anchor not only defines the alignment of the individual
+            # lines to another but as well of the whole block relative
+            # to the origin
+            if align == "middle":
+                offs = -max_len / 2 + (max_len - ll) / 2
+            elif align == "end":
+                offs = -ll
+            else:
+                offs = 0
+            offsets.append(offs)
+        self.active = True
+        line_lengths = _do_render(vtext, offsets)
 
     def _parse_code(self):
-        b = self.pop()
+        try:
+            b = self.pop()
+        except ShxFontParseError:
+            return
         direction = b & 0x0F
         length = (b & 0xF0) >> 4
         if length == 0:
@@ -418,10 +538,11 @@ class ShxFont:
             dy = -1.0
         self._x += dx * length * self._scale
         self._y += dy * length * self._scale
-        if self._pen:
-            self._path.line(self._last_x, self._last_y, self._x, self._y)
-        else:
-            self._path.move(self._x, self._y)
+        if self.active:
+            if self._pen:
+                self._path.line(self._last_x, self._last_y, self._x, self._y)
+            else:
+                self._path.move(self._x, self._y)
         self._last_x, self._last_y = self._x, self._y
 
     def _parse_code_special(self, special):
@@ -471,7 +592,8 @@ class ShxFont:
         if self._skip:
             self._skip = False
             return
-        self._path.new_path()
+        if self.active:
+            self._path.new_path()
 
     def _pen_down(self):
         """
@@ -484,7 +606,8 @@ class ShxFont:
             self._skip = False
             return
         self._pen = True
-        self._path.move(self._x, self._y)
+        if self.active:
+            self._path.move(self._x, self._y)
 
     def _pen_up(self):
         """
@@ -505,7 +628,10 @@ class ShxFont:
 
         :return:
         """
-        factor = self.pop()
+        try:
+            factor = self.pop()
+        except ShxFontParseError:
+            return
         if self._debug:
             print(
                 f"DIVIDE_VECTOR {self._scale}/{factor} {'(Skipped)' if self._skip else ''}"
@@ -556,7 +682,7 @@ class ShxFont:
     def _pop_stack(self):
         """
         Stack is considered four units deep. You may not pop more locations than have
-        been pushed onto the stack. Attempts to do so will respond with a error message.
+        been pushed onto the stack. Attempts to do so will respond with an error message.
         :return:
         """
         if self._debug:
@@ -571,11 +697,15 @@ class ShxFont:
             self._x, self._y = self._stack.pop()
         except IndexError:
             raise IndexError(f"Position stack underflow in shape {self._letter}")
-        self._path.move(self._x, self._y)
+        if self.active:
+            self._path.move(self._x, self._y)
         self._last_x, self._last_y = self._x, self._y
 
     def _draw_subshape_shapes(self):
-        subshape = self.pop()
+        try:
+            subshape = self.pop()
+        except ShxFontParseError:
+            return
         if self._debug:
             print(
                 f"Appending glyph {subshape} (Type={self.type}). {'(Skipped)' if self._skip else ''}"
@@ -590,21 +720,28 @@ class ShxFont:
         self._code += bytearray(reversed(shape))
 
     def _draw_subshape_bigfont(self):
-        subshape = self.pop()
+        try:
+            subshape = self.pop()
+        except ShxFontParseError:
+            return
         if self._debug:
             print(
                 f"Appending glyph {subshape} (Type={self.type}). {'(Skipped)' if self._skip else ''}"
             )
         if subshape == 0:
-            subshape = int_16le([self.pop(), self.pop()])
-            origin_x = self.pop() * self._scale
-            origin_y = self.pop() * self._scale
-            width = self.pop() * self._scale
-            height = self.pop() * self._scale
-            if self._debug:
-                print(
-                    f"Extended Bigfont Glyph: {subshape}, origin_x = {origin_x}, origin_y = {origin_y}. {width}x{height}"
-                )
+            try:
+                subshape = int_16le([self.pop(), self.pop()])
+            
+                origin_x = self.pop() * self._scale
+                origin_y = self.pop() * self._scale
+                width = self.pop() * self._scale
+                height = self.pop() * self._scale
+                if self._debug:
+                    print(
+                        f"Extended Bigfont Glyph: {subshape}, origin_x = {origin_x}, origin_y = {origin_y}. {width}x{height}"
+                    )
+            except ShxFontParseError:
+                return
         if self._skip:
             self._skip = False
             return
@@ -615,7 +752,11 @@ class ShxFont:
         self._code += bytearray(reversed(shape))
 
     def _draw_subshape_unifont(self):
-        subshape = int_16le([self.pop(), self.pop()])
+        try:
+            subshape = int_16le([self.pop(), self.pop()])
+        except ShxFontParseError:
+            return
+
         if self._debug:
             print(
                 f"Appending glyph {subshape} (Type={self.type}). {'(Skipped)' if self._skip else ''}"
@@ -651,8 +792,12 @@ class ShxFont:
         ranges from -128 to +127.
         :return:
         """
-        dx = signed8(self.pop()) * self._scale
-        dy = signed8(self.pop()) * self._scale
+        try:
+            dx = signed8(self.pop()) * self._scale
+            dy = signed8(self.pop()) * self._scale
+        except ShxFontParseError:
+            return
+
         if self._debug:
             print(f"XY_DISPLACEMENT {dx} {dy} {'(Skipped)' if self._skip else ''}")
         if self._skip:
@@ -660,10 +805,11 @@ class ShxFont:
             return
         self._x += dx
         self._y += dy
-        if self._pen:
-            self._path.line(self._last_x, self._last_y, self._x, self._y)
-        else:
-            self._path.move(self._x, self._y)
+        if self.active:
+            if self._pen:
+                self._path.line(self._last_x, self._last_y, self._x, self._y)
+            else:
+                self._path.move(self._x, self._y)
         self._last_x, self._last_y = self._x, self._y
 
     def _poly_xy_displacement(self):
@@ -672,8 +818,11 @@ class ShxFont:
         :return:
         """
         while True:
-            dx = signed8(self.pop()) * self._scale
-            dy = signed8(self.pop()) * self._scale
+            try:
+                dx = signed8(self.pop()) * self._scale
+                dy = signed8(self.pop()) * self._scale
+            except ShxFontParseError:
+                return
             if self._debug:
                 print(
                     f"POLY_XY_DISPLACEMENT {dx} {dy} {'(Skipped)' if self._skip else ''}"
@@ -686,10 +835,11 @@ class ShxFont:
                 continue
             self._x += dx
             self._y += dy
-            if self._pen:
-                self._path.line(self._last_x, self._last_y, self._x, self._y)
-            else:
-                self._path.move(self._x, self._y)
+            if self.active:
+                if self._pen:
+                    self._path.line(self._last_x, self._last_y, self._x, self._y)
+                else:
+                    self._path.move(self._x, self._y)
             self._last_x, self._last_y = self._x, self._y
         if self._skip:
             self._skip = False
@@ -700,7 +850,7 @@ class ShxFont:
         Octants are numbered ccw starting from 0° at the 3 o'clock position.
 
         3 2 1
-         \ /
+         ⍀ /
         4-O-0
          / \
         5 6 7
@@ -710,8 +860,11 @@ class ShxFont:
         and the span.
         :return:
         """
-        radius = self.pop() * self._scale
-        sc = signed8(self.pop())
+        try:
+            radius = self.pop() * self._scale
+            sc = signed8(self.pop())
+        except ShxFontParseError:
+            return
         s = (sc >> 4) & 0x7
         c = sc & 0x7
         if self._debug:
@@ -735,10 +888,11 @@ class ShxFont:
         my = cy + radius * sin(mid_angle)
         self._x = cx + radius * cos(end_angle)
         self._y = cy + radius * sin(end_angle)
-        if self._pen:
-            self._path.arc(self._last_x, self._last_y, mx, my, self._x, self._y)
-        else:
-            self._path.move(self._x, self._y)
+        if self.active:
+            if self._pen:
+                self._path.arc(self._last_x, self._last_y, mx, my, self._x, self._y)
+            else:
+                self._path.move(self._x, self._y)
         self._last_x, self._last_y = self._x, self._y
 
     def _fractional_arc(self):
@@ -751,13 +905,15 @@ class ShxFont:
         90° + (28/256 * 45°) = 95°
         """
         octant = tau / 8.0
-        start_offset = octant * self.pop() / 256.0
-        end_offset = octant * self.pop() / 256.0
-        radius = (256 * self.pop() + self.pop()) * self._scale
-        sc = signed8(self.pop())
-        s = (sc >> 4) & 0x7
-        c = sc & 0x7
-
+        try:
+            start_offset = octant * self.pop() / 256.0
+            end_offset = octant * self.pop() / 256.0
+            radius = (256 * self.pop() + self.pop()) * self._scale
+            sc = signed8(self.pop())
+            s = (sc >> 4) & 0x7
+            c = sc & 0x7
+        except ShxFontParseError:
+            return
         if self._debug:
             print(
                 f"FRACTION_ARC {start_offset}, {end_offset}, {radius}, {s}, {c} {'(Skipped)' if self._skip else ''}"
@@ -780,10 +936,11 @@ class ShxFont:
         my = cy + radius * sin(mid_angle)
         self._x = cx + radius * cos(end_angle)
         self._y = cy + radius * sin(end_angle)
-        if self._pen:
-            self._path.arc(self._last_x, self._last_y, mx, my, self._x, self._y)
-        else:
-            self._path.move(self._x, self._y)
+        if self.active:
+            if self._pen:
+                self._path.arc(self._last_x, self._last_y, mx, my, self._x, self._y)
+            else:
+                self._path.move(self._x, self._y)
         self._last_x, self._last_y = self._x, self._y
 
     def _bulge_arc(self):
@@ -795,10 +952,12 @@ class ShxFont:
 
         :return:
         """
-
-        dx = signed8(self.pop()) * self._scale
-        dy = signed8(self.pop()) * self._scale
-        h = signed8(self.pop())
+        try:
+            dx = signed8(self.pop()) * self._scale
+            dy = signed8(self.pop()) * self._scale
+            h = signed8(self.pop())
+        except ShxFontParseError:
+            return
 
         if self._debug:
             print(f"BULGE_ARC {dx}, {dy}, {h} {'(Skipped)' if self._skip else ''}")
@@ -814,13 +973,14 @@ class ShxFont:
         my = by + r * bulge * sin(bulge_angle)
         self._x += dx
         self._y += dy
-        if self._pen:
-            if bulge == 0:
-                self._path.line(self._last_x, self._last_y, self._x, self._y)
+        if self.active:
+            if self._pen:
+                if bulge == 0:
+                    self._path.line(self._last_x, self._last_y, self._x, self._y)
+                else:
+                    self._path.arc(self._last_x, self._last_y, mx, my, self._x, self._y)
             else:
-                self._path.arc(self._last_x, self._last_y, mx, my, self._x, self._y)
-        else:
-            self._path.move(self._x, self._y)
+                self._path.move(self._x, self._y)
         self._last_x, self._last_y = self._x, self._y
 
     def _poly_bulge_arc(self):
@@ -830,8 +990,12 @@ class ShxFont:
         """
         h = 0
         while True:
-            dx = signed8(self.pop()) * self._scale
-            dy = signed8(self.pop()) * self._scale
+            try:
+                dx = signed8(self.pop()) * self._scale
+                dy = signed8(self.pop()) * self._scale
+            except ShxFontParseError:
+                return
+
             if self._debug:
                 print(
                     f"POLY_BULGE_ARC {dx}, {dy}, {h} {'(Skipped)' if self._skip else ''}"
@@ -840,7 +1004,10 @@ class ShxFont:
                 if self._debug:
                     print(f"POLY_BULGE_ARC (TERMINATED)")
                 break
-            h = signed8(self.pop())
+            try:
+                h = signed8(self.pop())
+            except ShxFontParseError:
+                return
             if self._skip:
                 continue
             r = abs(complex(dx, dy)) / 2
@@ -852,13 +1019,16 @@ class ShxFont:
             my = by + r * bulge * sin(bulge_angle)
             self._x += dx
             self._y += dy
-            if self._pen:
-                if bulge == 0:
-                    self._path.line(self._last_x, self._last_y, self._x, self._y)
+            if self.active:
+                if self._pen:
+                    if bulge == 0:
+                        self._path.line(self._last_x, self._last_y, self._x, self._y)
+                    else:
+                        self._path.arc(
+                            self._last_x, self._last_y, mx, my, self._x, self._y
+                        )
                 else:
-                    self._path.arc(self._last_x, self._last_y, mx, my, self._x, self._y)
-            else:
-                self._path.move(self._x, self._y)
+                    self._path.move(self._x, self._y)
             self._last_x, self._last_y = self._x, self._y
         if self._skip:
             self._skip = False
