@@ -30,7 +30,7 @@ class ZinglPlotter:
         arc_required = int(ceil(abs(arc.sweep) / sweep_limit))
         if arc_required == 0:
             return
-        slice = arc.sweep / float(arc_required)
+        arc_slice = arc.sweep / float(arc_required)
 
         theta = arc.get_rotation()
         rx = arc.rx
@@ -43,9 +43,11 @@ class ZinglPlotter:
         sin_theta = sin(theta)
 
         for i in range(0, arc_required):
-            next_t = current_t + slice
+            next_t = current_t + arc_slice
 
-            alpha = sin(slice) * (sqrt(4 + 3 * pow(tan(slice / 2.0), 2)) - 1) / 3.0
+            alpha = (
+                sin(arc_slice) * (sqrt(4 + 3 * pow(tan(arc_slice / 2.0), 2)) - 1) / 3.0
+            )
 
             cos_start_t = cos(current_t)
             sin_start_t = sin(current_t)
@@ -68,7 +70,7 @@ class ZinglPlotter:
             p_c1 = (p_start[0] + alpha * ePrimen1x, p_start[1] + alpha * ePrimen1y)
             p_c2 = (p_end[0] - alpha * ePrimen2x, p_end[1] - alpha * ePrimen2y)
 
-            for value in ZinglPlotter.plot_cubic_bezier(
+            yield from ZinglPlotter.plot_cubic_bezier(
                 p_start[0],
                 p_start[1],
                 p_c1[0],
@@ -77,8 +79,7 @@ class ZinglPlotter:
                 p_c2[1],
                 p_end[0],
                 p_end[1],
-            ):
-                yield value
+            )
             p_start = Point(p_end)
             current_t = next_t
 
@@ -584,10 +585,9 @@ class ZinglPlotter:
                 fy2 *= fy0
             if x0 != x3 or y0 != y3:  # /* segment t1 - t2 */
                 # plotCubicBezierSeg(x0,y0, x0+fx1,y0+fy1, x0+fx2,y0+fy2, x3,y3)
-                for plot in ZinglPlotter.plot_cubic_bezier_seg(
+                yield from ZinglPlotter.plot_cubic_bezier_seg(
                     x0, y0, x0 + fx1, y0 + fy1, x0 + fx2, y0 + fy2, x3, y3
-                ):
-                    yield plot
+                )
             x0 = x3
             y0 = y3
             fx0 = fx3
