@@ -1,4 +1,5 @@
 import unittest
+
 from test import bootstrap
 
 
@@ -13,10 +14,11 @@ class TestPenbox(unittest.TestCase):
         try:
             kernel_root = kernel.get_context("/")
             kernel_root("penbox testpasses add 5 set 0-4 hatch_angle 0-90\n")
-            self.assertEqual(len(kernel_root.elements.penbox["testpasses"]), 5)
+            self.assertEqual(len(kernel_root.penbox.pens["testpasses"]), 5)
             self.assertEqual(
-                kernel_root.elements.penbox["testpasses"][-1]["hatch_angle"], 90
+                kernel_root.elements.penbox.pens["testpasses"][-1]["hatch_angle"], 90
             )
+            kernel_root("penbox testpasses del 999\n")
 
         finally:
-            kernel.shutdown()
+            kernel()
